<?php
//Display records using wp_list_table.

if (!class_exists('WP_List_Table')) {
    require_once(ABSPATH . 'wp-admin/includes/class-wp-list-table.php');
}

class DynamicBlock_List_Table extends WP_List_Table
{

    /*common array print */
    public function print($p)
    {
        echo "<pre>";
        print_r($p);
        echo "</pre>";
    }

    /** Prepare the items for the table to process **/
    public function prepare_items()
    {
        $columns = $this->get_columns();
        $hidden = $this->get_hidden_columns();
        $sortable = $this->get_sortable_columns();

        $data = $this->table_data();
        usort($data, array(&$this, 'sort_data'));


        $perPage = get_option('posts_per_page');
        $currentPage = $this->get_pagenum();
        $totalItems = count($data);

        $this->set_pagination_args(array(
            'total_items' => $totalItems,
            'per_page' => $perPage
        ));

        $data = array_slice($data, (($currentPage - 1) * $perPage), $perPage);
        $this->_column_headers = array($columns, $hidden, $sortable);
        $this->items = $data;
    }

    /** Override the parent columns method. Defines the columns to use in your listing table **/
    public function get_columns()
    {
        $columns = array(
            'id' => 'ID',
            'post_title' => __('Block Title', DYNAMICBLOCK_TEXT),
            'post_block_type' => __('Type', DYNAMICBLOCK_TEXT),
            'page' => __('Page', DYNAMICBLOCK_TEXT),
            'hook' => __('Position', DYNAMICBLOCK_TEXT),
            'priority' => __('Priority', DYNAMICBLOCK_TEXT),
            'post_modified' => __('Last Edited', DYNAMICBLOCK_TEXT),
            'post_status' => __('Post status', DYNAMICBLOCK_TEXT),
        );
        return $columns;
    }

    /** Define which columns are hidden **/
    public function get_hidden_columns()
    {
        return array();
    }

    /** Define the sortable columns **/
    public function get_sortable_columns()
    {
        $sortable = array(
            'post_title' => array('post_title', true),
            'post_block_type' => array('post_block_type', true),
            'page' => array('page', true),
            'hook' => array('hook', true),
            'id' => array('id', true),
        );
        return $sortable;
    }

    /** Get the table data **/
    private function table_data()
    {
        $data = array();
        global $wpdb;
        $_block_obj = $wpdb->get_results("SELECT * FROM $wpdb->posts where post_type='dynamicblock-block' and post_status = 'publish'");
        foreach ($_block_obj as $block) {
            $rules = get_post_meta($block->ID, 'dynamicblock_post');
            if (!empty($rules) && (isset($rules[0]) || $rules[0])) {
                foreach ($rules as $blocks) {
                    $blocks = $blocks['dynamicblock_post_' . $block->ID]['block_info']['post_block_type'];
                    $_where_to_display_control = get_post_meta($block->ID, 'where_to_display_control', true);
                    $i = 0;
                    if (!empty($_where_to_display_control) && isset($_where_to_display_control)) {
                        $priority = array();
                        $position = array();
                        $_page = array();
                        foreach ($_where_to_display_control as $control) {
                                $position[] = $control["hook"];
                                $priority[] = $control["priority"];
                                $wc_page = $control["wc_page"];
                                $dynamicblock_page = DynamicBlock_Main::dynamicblock_default_locations();
                                if(!empty($dynamicblock_page)){
                                    foreach ($dynamicblock_page as $key => $page) {
                                        if ($wc_page == $key) {
                                            $_page[] = $page["title"];
                                        }
                                    }
                                }
                            $i++;
                        }
                        $data[] = $this->dashboard_datatable_array(implode(', ', $_page), $block->ID, $block->post_title, $blocks, implode(', ', $position), implode(', ', $priority), $block->post_modified, $block->post_status);
                    } else {
                        $data[] = $this->dashboard_datatable_array('-', $block->ID, $block->post_title, $blocks, '-', '-', $block->post_modified, $block->post_status);
                    }
                }
            } else {
                $data[] = $this->dashboard_datatable_array('-', $block->ID, $block->post_title, '-', '-', '-', $block->post_modified, $block->post_status);
            }
        }
        return $data;
    }

    public function dashboard_datatable_array($page_title = "-", $id = "-", $post_title = "-", $post_block_type = "-", $hook = "-", $priority = "-", $post_modified = "-", $post_status = "-")
    {
        $page = $page_title;
        $data = array(
            'id' => $id,
            'post_title' => $post_title,
            'page' => $page,
            'post_block_type' => $post_block_type,
            'hook' => $hook,
            'priority' => $priority,
            'post_modified' => date('F j, Y', strtotime($post_modified)),
            'post_status' => $post_status,
        );
        return $data;
    }

    /*get the single column to assign the class for css. */
    public function _column_post_status($item, $classes, $data, $primary)
    {
        $_status = $this->column_default($item, 'post_status');
        echo '<td class="' . esc_attr($classes) . '" ' . wp_strip_all_tags($data) . '>';
        echo '<span class="block-item-status ii item-status-' . esc_attr($_status) . '">';
        esc_html_e($_status);
        echo $this->handle_row_actions($item, 'post_status', $primary);
        echo '</span>';
        echo '</td>';
    }

    /** Define what data to show on each column of the table  */
    public function column_default($item, $column_name)
    {
        switch ($column_name) {
            case 'id':
            case 'post_title':
            case 'post_block_type':
            case 'page':
            case 'hook':
            case 'priority':
            case 'post_modified':
            case 'post_status':
                return $item[$column_name];

            default:
                return print_r($item, true);
        }
    }

    /** Allows you to sort the data by the variables set in the $_GET  */
    private function sort_data($a, $b)
    {
        $orderby = 'id,post_title,post_block_type,page,hook';
        $order = 'asc';

        if (!empty($_GET['orderby'])) {
            $orderby = sanitize_text_field($_GET['orderby']);
        }

        if (!empty($_GET['order'])) {
            $order = sanitize_text_field($_GET['order']);
        }

        @$result = strnatcmp($a[$orderby], $b[$orderby]);

        if ($order === 'asc') {
            return $result;
        }

        return -$result;
    }
}
