<?php
/**
 * DynamicBlock Register Post Type, Taxonomy, Custom Meta Fields
 */
defined('ABSPATH') || exit;

class DynamicBlock_Admin
{
    static $basename = null;

    // Register DynamicBlock Block Post type
    function __construct()
    {
        add_action('bp_before_post_content', [$this, 'meso_add_feat_img_single']);
        add_action('admin_notices', [$this, 'dynamicblock_plugin_admin_header']);
        add_action('admin_enqueue_scripts', [$this, 'dynamicblock_scripts']);
        add_action('admin_menu', [$this, 'dynamicblock_admin_menu']);
        add_action('admin_enqueue_scripts', [$this, 'codemirror_enqueue_scripts']);
        add_action('wp_ajax_nopriv_dynamicblock_what_display_dataset_ajax',
            array($this, 'dynamicblock_what_display_dataset_ajax'));
        add_action('wp_ajax_dynamicblock_what_display_dataset_ajax',
            array($this, 'dynamicblock_what_display_dataset_ajax'));
        add_action('wp_ajax_nopriv_dynamicblock_when_display_dataset_ajax',
            array($this, 'dynamicblock_when_display_dataset_ajax'));
        add_action('wp_ajax_dynamicblock_when_display_dataset_ajax',
            array($this, 'dynamicblock_when_display_dataset_ajax'));
        add_action('restrict_manage_posts', array($this, 'filter_by_block_type'));
        add_filter('pre_get_posts', array($this, 'dynamicblock_custom_filters'));
        add_filter('manage_dynamicblock-block_posts_columns',
            array($this, 'dynamicblock_custom_manage_edit_block_columns'));
        add_action('manage_posts_custom_column', array($this, 'custom_columns'));
        add_filter('manage_edit-dynamicblock-block_sortable_columns', array($this, 'register_sortable_columns'));
        //add_action('add_meta_boxes', [$this, 'dynamicblock_product_set_add_meta_box']);
        add_action('save_post', [$this, 'dynamicblock_save_product_set_meta'], 10, 2);

        //Upgrade to Pro Links
        add_action('admin_footer', array($this, 'addProMenuItemClass'));
        include_once DYNAMICBLOCK_ABSPATH . 'admin/class/class-admin-dashboard-grid.php';
    }



    public function dynamicblock_product_set_add_meta_box()
    {
        add_meta_box(
            'dynamicblock_product_set_meta', 'DynamicBlock Product Set',
            array($this, 'dynamicblock_product_set_callback'),
            'product',
            'side'
        );
    }

    public function dynamicblock_product_set_callback($post)
    {
        wp_nonce_field('dynamicblock_product_set_control', 'dynamicblock_product_set_control_nonce');
        $_dynamicblock_product_set_control = get_post_meta($post->ID, 'dynamicblock_product_set_control', true);
        require_once DYNAMICBLOCK_ABSPATH . 'admin/meta-boxes/dynamicblock-product-set-meta-box.php';
    }

    public function dynamicblock_save_product_set_meta($post_id)
    {
        if (isset($_POST['dynamicblock_product_set_control'])) {
            update_post_meta($post_id, 'dynamicblock_product_set_control',
                self::dynamicblock_sanitize_array($_POST['dynamicblock_post']));
        } else {
            update_post_meta($post_id, 'dynamicblock_product_set_control', array());
        }
    }

    public function dynamicblock_custom_manage_edit_block_columns($columns)
    {
        $column_meta = array('type' => 'Type', 'page' => 'Page', 'position' => 'Position', 'priority' => 'Priority');
        $columns     = $columns + $column_meta;
        $columns     = $this->moveElementInArray($columns, 'date', 5);
        $columns     = ($this->moveElementInArray($columns, 'priority', 5));

        return $columns;
    }

    public function register_sortable_columns($columns)
    {
        $columns['type'] = 'Type';

        return $columns;
    }

    public function custom_columns($column)
    {
        global $post;
        $rules                     = get_post_meta($post->ID, 'dynamicblock_post');
        $dynamicblock_current_post = '';
        if (isset($rules[0])) {
            $dynamicblock_current_post = $rules[0]['dynamicblock_post_' . $post->ID];
        }
        $_where_to_display_control = '';
        if (get_post_meta($post->ID, 'where_to_display_control', true)) {
            $_where_to_display_control = get_post_meta($post->ID, 'where_to_display_control', true);
        }
        $i             = 0;
        $priority      = array();
        $position      = array();
        $selected_page = array();
        if (isset($_where_to_display_control) && ! empty($_where_to_display_control)) {
            foreach ($_where_to_display_control as $control) {
                $position[]        = (isset($control["hook"])) ? $control["hook"] : '-';
                $priority[]        = (isset($control["priority"])) ? $control["priority"] : '-';
                $wc_page           = (isset($control["wc_page"])) ? $control["wc_page"] : '';
                $dynamicblock_page = DynamicBlock_Main::dynamicblock_default_locations();
                foreach ($dynamicblock_page as $key => $page) {
                    if (isset($wc_page) && $wc_page == $key) {
                        $selected_page[] = $page["title"];
                    }
                }
                $i++;
            }
        }
        switch ($column) {
            case 'type':
                esc_html_e(isset($dynamicblock_current_post['block_info']['post_block_type']) ? $dynamicblock_current_post['block_info']['post_block_type'] : '-');
                break;
            case 'page':
                esc_html_e(implode(', ', $selected_page));
                break;
            case 'position':
                esc_html_e(implode(', ', $position));
                break;
            case 'priority':
                esc_html_e(implode(', ', $priority));
                break;
        }
    }

    public function moveElementInArray($array, $toMove, $fromMove)
    {
        $indexToMove = array_search($toMove, array_keys($array));
        $itemToMove  = $array[$toMove];
        array_splice($array, $indexToMove, 1);
        $i      = 0;
        $output = array();
        foreach ($array as $key => $item) {
            if ($i == $fromMove) {
                $output[$toMove] = $itemToMove;
            }
            $output[$key] = $item;
            $i++;
        }

        return $output;
    }

    public function filter_by_block_type()
    {
        global $typenow;
        global $wp_query;
        $dynamicblock_page = DynamicBlock_Main::dynamicblock_default_locations();
        //filter by block type
        if ($typenow == 'dynamicblock-block') {
            $plugins              = array(
                'html'             => __('Html', DYNAMICBLOCK_TEXT),
                'related-products' => __('Related Products', DYNAMICBLOCK_TEXT),
                'bundle-pack'      => __('Bundle Pack', DYNAMICBLOCK_TEXT),
                'accessories'      => __('Accessories', DYNAMICBLOCK_TEXT),
            ); // Options for the filter select field
            $filter_by_block_type = '';
            if (isset($_GET['filter_by_block_type'])) {
                $filter_by_block_type = sanitize_text_field($_GET['filter_by_block_type']);
            } ?>
            <select name="filter_by_block_type" id="dynamicblock_filter_by_block_type">
                <option value="all" <?php selected('All', $filter_by_block_type); ?>><?php _e('Filter by Block Type',
                        'dynamicblock-block'); ?></option>
                <?php foreach ($plugins as $key => $value) { ?>
                    <option value="<?php echo esc_attr($key); ?>" <?php selected($key,
                        $filter_by_block_type); ?>><?php echo esc_attr($value); ?></option>
                <?php } ?>
            </select>
        <?php }
        //filter by page
        if ($typenow == 'dynamicblock-block') {
            $filter_by_page = '';
            if (isset($_GET['filter_by_page'])) {
                $filter_by_page = sanitize_text_field($_GET['filter_by_page']);
            }
            ?>
            <select name="filter_by_page" id="dynamicblock_filter_by_page"
                    selected-hook="<?php echo esc_attr($filter_by_page); ?>" onchange="filter_by_page_on_change(this)">
                <option value="all" <?php selected('All', $filter_by_page); ?>><?php _e('Filter by Page',
                        'dynamicblock-block'); ?></option>
                <?php foreach ($dynamicblock_page as $key => $value) { ?>
                    <option value="<?php echo esc_attr($key); ?>" <?php selected($key, $filter_by_page); ?>
                            data-hook="<?php echo htmlentities(json_encode($value['hooks'],
                                JSON_UNESCAPED_SLASHES)); ?>"><?php echo esc_attr($value["title"]); ?></option>
                <?php } ?>
            </select>
        <?php }
        //filter by position
        if ($typenow == 'dynamicblock-block') {
            $filter_by_position = '';
            if (isset($_GET['filter_by_position'])) {
                $filter_by_position = sanitize_text_field($_GET['filter_by_position']);
            }
            $_locations = apply_filters('dynamicblock_conditional_content_get_locations', array(), 0);
            ?>
            <select name="filter_by_position" id="dynamicblock_filter_by_position">
                <option value="all" <?php selected('All', $filter_by_position); ?> ><?php _e('Filter by Position',
                        'dynamicblock-block'); ?></option>
                <?php
                if (! empty($_locations)) {
                    foreach ($_locations as $_location) {
                        ?>
                        <optgroup label="<?php echo esc_attr($_location['title']); ?>">
                            <?php
                            if (! empty($_location['hooks'])) {
                                foreach ($_location['hooks'] as $key => $hook) { ?>
                                    <option value="<?php echo esc_attr($hook['action']); ?>"><?php echo esc_html($hook['title']); ?></option>
                                    <?php
                                }
                            }
                            ?>
                        </optgroup>
                        <?php
                    }
                }
                ?>
            </select>
            <?php
        }
    }

    public function dynamicblock_custom_filters($query)
    {
        $_meta_query = array();
        if (isset($_GET['filter_by_block_type']) && $_GET['filter_by_block_type'] != 'all') {
            $_meta_query[] = array(
                'key'     => 'dynamicblock_post',
                'value'   => sanitize_text_field($_GET['filter_by_block_type']),
                'compare' => 'LIKE',
            );
        }
        if (isset($_GET['filter_by_page']) && $_GET['filter_by_page'] != 'all') {
            $_meta_query[] = array(
                'key'     => 'where_to_display_control',
                'value'   => sanitize_text_field($_GET['filter_by_page']),
                'compare' => 'LIKE',
            );
        }
        if (isset($_GET['filter_by_position']) && $_GET['filter_by_position'] != 'all') {
            $_meta_query[] = array(
                'key'     => 'where_to_display_control',
                'value'   => sanitize_text_field($_GET['filter_by_position']),
                'compare' => 'LIKE',
            );
        }
        if (isset($_GET['orderby']) && sanitize_text_field($_GET['orderby']) == 'Type') {
            $_meta_query[] = array(
                'relation' => 'OR',
                array(
                    'key'     => 'dynamicblock_block_type',
                    'compare' => 'EXISTS',
                ),
                array(
                    'key'     => 'dynamicblock_block_type',
                    'compare' => 'NOT EXISTS',
                ),
            );
            $query->set('orderby', 'meta_value title');
        }
        if (! isset($query->query_vars['meta_query']) && ! empty($_meta_query)) {
            $query->set('meta_query', $_meta_query);
        } else if (! empty($_meta_query)) {
            $query->set('meta_query', array_merge($_meta_query, $query->query_vars['meta_query']));
        }
    }

    public function dynamicblock_scripts()
    {
        wp_enqueue_script('popper_min_script', DYNAMICBLOCK_PLUGIN_URL . 'assets/js/popper.min.js');
        wp_enqueue_script('dynamicblock_select2-script', DYNAMICBLOCK_PLUGIN_URL . 'assets/js/select2.min.js');
        wp_enqueue_style('dynamicblock_select2-style', DYNAMICBLOCK_PLUGIN_URL . 'assets/css/select2.min.css');

        wp_enqueue_script('dynamicblock_admin-flatpickr',
            DYNAMICBLOCK_PLUGIN_URL . 'assets/js/flatpickr.js', IS_LOCALHOST ? array() : null, IS_LOCALHOST ? time() : '');
        wp_enqueue_style('dynamicblock_flatpickr-style', DYNAMICBLOCK_PLUGIN_URL . 'assets/css/flatpickr.css');

        // deps and ver for clear cache
        wp_enqueue_style('dynamicblock_admin-style',
            DYNAMICBLOCK_PLUGIN_URL . 'assets/css/dynamicblock-admin-style.css', IS_LOCALHOST ? array() : null, IS_LOCALHOST ? time() : '');
        wp_enqueue_script('dynamicblock_admin-script',
            DYNAMICBLOCK_PLUGIN_URL . 'assets/js/dynamicblock-admin-script.js', IS_LOCALHOST ? array() : null, IS_LOCALHOST ? time() : '');

        $is_active = false;
        if (class_exists('DynamicBlocksBuilderActivator')) {
            $_activator = new DynamicBlocksBuilderActivator();
            $is_active  = $_activator->is_active();
        }

        wp_localize_script('dynamicblock_admin-script', 'dynamicblock_ajax', array(
                'ajaxurl'                 => admin_url('admin-ajax.php'),
                'search_products_nonce'   => wp_create_nonce('search-products'),
                'search_customers_nonce'  => wp_create_nonce('search-customers'),
                'search_categories_nonce' => wp_create_nonce('search-categories'),
                'search_pages_nonce'      => wp_create_nonce('search-pages'),
                'is_active'               => $is_active,
            )
        );
    }

    public function dynamicblock_admin_menu()
    {
        global $submenu;
        $icon_url = DYNAMICBLOCK_PLUGIN_URL . "assets/images/dynamicblock_fav.svg";
        $proTitle = '<span id="dynamicblock-link-upgrade-highlight">' . __('Upgrade to Pro', DYNAMICBLOCK_TEXT) . '</span>';

        add_menu_page('Dynamicblock',
            'Dynamic Blocks',
            'manage_options',
            'dynamicblock-dashboard',
            [$this, 'dynamicblock_dashboard'],
            $icon_url
        );
        add_submenu_page('dynamicblock-dashboard',
            'Blocks',
            'Blocks',
            'manage_options',
            'edit.php?post_type=dynamicblock-block'
        );
        $submenu['dynamicblock-dashboard'][0][0] = 'Dashboard';

        if (class_exists('DynamicBlocksBuilderActivator')) {
            add_submenu_page('dynamicblock-dashboard',
                'License',
                'License',
                'manage_options',
                'global_setting',
                [$this, 'dynamicblock_global_setting']
            );
        }

        add_submenu_page('dynamicblock-dashboard',
            'Documentation',
            'Documentation',
            'manage_options',
            'docs',
            [$this, 'dynamicblock_docs']
        );

        add_submenu_page('dynamicblock-dashboard',
            'Get Support',
            'Get Support',
            'manage_options',
            'help',
            [$this, 'dynamicblock_help']
        );

        if (!class_exists('DynamicBlocksBuilderActivator')) {
            add_submenu_page('dynamicblock-dashboard',
                $proTitle,
                $proTitle,
                'manage_options',
                'upgrade',
                [$this, 'dynamicblock_upgrade_to_pro']
            );
        }

    }

    public function dynamicblock_dashboard()
    {
        include_once DYNAMICBLOCK_TEMPLATE_PATH . 'admin/dynamicblock_dashboard.php';
    }

    public function dynamicblock_global_setting()
    {
        if (class_exists('DynamicBlocksBuilderActivator')) {
            include_once DYNAMICBLOCK_TEMPLATE_PATH . 'admin/dynamicblock_globle_setting.php';
        }
    }

    public function dynamicblock_help()
    {
        wp_redirect('https://wordpress.org/support/plugin/dynamic-blocks-builder/', 301);
        exit;
        include_once DYNAMICBLOCK_TEMPLATE_PATH . 'admin/dynamicblock_help.php';
    }

    public function dynamicblock_docs()
    {
        wp_redirect('https://www.dynamicblock.ai/knowledge-base/', 301);
        exit;
        include_once DYNAMICBLOCK_TEMPLATE_PATH . 'admin/dynamicblock_docs.php';
    }

    public function dynamicblock_upgrade_to_pro()
    {
        if (!class_exists('DynamicBlocksBuilderActivator')) {

            wp_redirect('https://www.dynamicblock.ai/pricing-plans/', 301);
            exit;
            include_once DYNAMICBLOCK_TEMPLATE_PATH . 'admin/dynamicblock_upgrade.php';
        }
    }



    /**
     * Add the PRO Class to left sidebar menu item.
     *
     * @return void
     */
    public static function addProMenuItemClass()
    {
        //Add to footer so it's applied on hovered item too
        ?>
            <script>jQuery(function($) {
                $('#dynamicblock-link-upgrade-highlight').parent().attr('target','_blank');
                $('#dynamicblock-link-upgrade-highlight').closest('li').addClass('dynamicblock-submenu-upgrade-highlight')
            });
            </script>
            <style>
                .dynamicblock-submenu-upgrade-highlight,
                .dynamicblock-submenu-upgrade-highlight a,
                .dynamicblock-submenu-upgrade-highlight a span#dynamicblock-link-upgrade-highlight {
                    background-color: #1da867!important;
                    color: #fff!important;
                    border-color: #fff!important;
                    font-weight: 600!important;
                }

            </style>
        <?php
    }

    public function dynamicblock_plugin_admin_header()
    {
        global $pagenow;
        $post     = (isset($_GET['post_type']) && ! empty($_GET['post_type'])) ? sanitize_text_field($_GET['post_type']) : '';
        $_post_id = (isset($_GET['post']) && ! empty($_GET['post'])) ? sanitize_text_field($_GET['post']) : '';
        if ($_post_id) {
            $post = get_post($_post_id);
        }
        $rules            = get_post_meta($_post_id, 'dynamicblock_post');
        $post_block_type  = (isset($rules[0]['dynamicblock_post_' . $_post_id]['block_info']['post_block_type'])) ? $rules[0]['dynamicblock_post_' . $_post_id]['block_info']['post_block_type'] : '';
        $is_all_post_type = ('edit.php' == $pagenow && 'dynamicblock-block' == $post);
        if ($is_all_post_type && sanitize_text_field(@$_GET['page']) != 'global_setting' && sanitize_text_field(@$_GET['page']) != 'help') {
            $_data = array(
                'title' => 'Block Type',
                'text'  => 'Inserire testo e link documentazione. <a>Link.</a>',
                'page'  => 'Dashboard',
            );
            include DYNAMICBLOCK_TEMPLATE_PATH . 'admin/dynamicblock-admin-breadcrumbs.php';
            include DYNAMICBLOCK_TEMPLATE_PATH . 'admin/dynamicblock-nav-banner.php';
            include DYNAMICBLOCK_TEMPLATE_PATH . 'admin/dynamicblock-add-block-nav.php';
        } else if (isset($_GET['action']) && sanitize_text_field($_GET['action']) == 'edit' && $post->post_type == 'dynamicblock-block') {
            $_data = array(
                'title'      => 'Edit Block',
                'text'       => 'Inserire testo e link documentazione. <a>Link.</a>',
                'page'       => 'Block',
                'block_type' => $post_block_type,
            );
            include DYNAMICBLOCK_TEMPLATE_PATH . 'admin/dynamicblock-admin-breadcrumbs.php';
            include DYNAMICBLOCK_TEMPLATE_PATH . 'admin/dynamicblock-nav-banner.php';
        } else if (isset($_GET['page']) && @$_GET['page'] == 'dynamicblock-dashboard' && sanitize_text_field(@$_GET['page']) != 'help') {
            $_data = array(
                'title' => 'Dashboard',
                'text'  => 'Inserire testo e link documentazione. <a>Link.</a>',
                'page'  => 'Dashboard',
            );
            include DYNAMICBLOCK_TEMPLATE_PATH . 'admin/dynamicblock-admin-breadcrumbs.php';
            include DYNAMICBLOCK_TEMPLATE_PATH . 'admin/dynamicblock-nav-banner.php';
            include DYNAMICBLOCK_TEMPLATE_PATH . 'admin/dynamicblock-dashboard-nav.php';
        } else if ($pagenow == 'post-new.php' && 'dynamicblock-block' == $post) {
            $block_type = isset($_GET["block_type"]) ? sanitize_text_field($_GET["block_type"]) : '';
            $_data      = array(
                'title'      => 'Add New Block',
                'text'       => 'Inserire testo e link documentazione. <a>Link.</a>',
                'page'       => 'Block',
                'block_type' => $block_type,
            );
            include DYNAMICBLOCK_TEMPLATE_PATH . 'admin/dynamicblock-admin-breadcrumbs.php';
            include DYNAMICBLOCK_TEMPLATE_PATH . 'admin/dynamicblock-nav-banner.php';
        }
    }

    public static function dynamicblock_get_field($_field)
    {
        switch ($_field['type']) {
            case 'text':
                include DYNAMICBLOCK_TEMPLATE_PATH . 'admin/dynamicblock-text.php';
                break;
            case 'textarea':
                include DYNAMICBLOCK_TEMPLATE_PATH . 'admin/dynamicblock-textarea.php';
                break;
            case 'select':
                include DYNAMICBLOCK_TEMPLATE_PATH . 'admin/dynamicblock-select.php';
                break;
            case 'onoff':
                include DYNAMICBLOCK_TEMPLATE_PATH . 'admin/dynamicblock-onoff.php';
                break;
            case 'radio':
                include DYNAMICBLOCK_TEMPLATE_PATH . 'admin/dynamicblock-radio-button.php';
                break;
            default:
                include DYNAMICBLOCK_TEMPLATE_PATH . 'admin/dynamicblock-text.php';
                break;
        }
    }

    public function codemirror_enqueue_scripts($hook)
    {
        $cm_settings['codeEditor'] = wp_enqueue_code_editor(array('type' => 'text/css'));
        wp_localize_script('jquery', 'cm_settings', $cm_settings);
        wp_enqueue_style('wp-codemirror');
    }

    public static function default_rule_types($types)
    {
        $types = array(
            __('General', 'wc_conditional_content')    => array(
                'general_always' => __('Always', 'wc_conditional_content'),
            ),
            __("Product", 'wc_conditional_content')    => array(
                'product_select'    => __('Products', 'wc_conditional_content'),
                'product_type'      => __("Product Type", 'wc_conditional_content'),
                'product_category'  => __("Product Category", 'wc_conditional_content'),
                'product_attribute' => __("Product Attributes", 'wc_conditional_content'),
                'product_tag'       => __("Product Tag", 'wc_conditional_content'),
                'product_price'     => __("Product Price", 'wc_conditional_content'),
            ),
            __('Schedule', 'wc_conditional_content')   => array(
                'schedule_date' => __('Date', 'wc_conditional_content'),
                'schedule_day'  => __('Day', 'wc_conditional_content'),
                'schedule_time' => __('Time', 'wc_conditional_content'),
            ),
            __('Stock', 'wc_conditional_content')      => array(
                'stock_status' => __('Stock Status', 'wc_conditional_content'),
                'stock_level'  => __('Stock Level', 'wc_conditional_content'),
            ),
            __('Sales', 'wc_conditional_content')      => array(
                'sale_schedule' => __('Sale Date', 'wc_conditional_content'),
                'sale_status'   => __('Sale Status', 'wc_conditional_content'),
            ),
            __("User Role", 'wc_conditional_content') => array(
                'users_user'           => __("User", 'wc_conditional_content'),
                'users_role'           => __("Role", 'wc_conditional_content'),
                'users_authentication' => __('Authentication', 'wc_conditional_content'),
            ),
            __("Store", 'wc_conditional_content')      => array(
                'store_order_count' => __("Order Count and Status", 'wc_conditional_content'),
            ),
            __("Cart", 'wc_conditional_content')       => array(
                'cart_total'              => __("Cart Total", 'wc_conditional_content'),
                'cart_product'            => __("Cart Products", 'wc_conditional_content'),
                'cart_category'           => __("Cart Categories", 'wc_conditional_content'),
                'cart_line_item_product'  => __("Cart Line Item Product", 'wc_conditional_content'),
                'cart_line_item_quantity' => __("Cart Line Item Quantity", 'wc_conditional_content'),
            ),
            __('Geography', 'wc_conditional_content')  => array(
                'geo_country_code' => __('Country', 'wc_conditional_content'),
                'wpml_language'    => __('WPML Language Code', 'wc_conditional_content'),
            ),
            __('Taxonomy', 'wc_conditional_content')   => array(
                'taxonomy_product_category' => __('Product Category Archive'),
            ),
        );

        return $types;
    }

    public static function what_to_display_products($types)
    {
        $types = array(
            __('General', 'wc_conditional_content')    => array(
                'general_always' => __('Always', 'wc_conditional_content'),
            ),
            __("Product", 'wc_conditional_content')    => array(
                'product_select'    => __('Products', 'wc_conditional_content'),
                'product_type'      => __("Product Type", 'wc_conditional_content'),
                'product_category'  => __("Product Category", 'wc_conditional_content'),
                'product_attribute' => __("Product Attributes", 'wc_conditional_content'),
                'product_tag'       => __("Product Tag", 'wc_conditional_content'),
                'product_price'     => __("Product Price", 'wc_conditional_content'),
            ),
            __('Schedule', 'wc_conditional_content')   => array(
                'schedule_date' => __('Date', 'wc_conditional_content'),
                'schedule_day'  => __('Day', 'wc_conditional_content'),
                'schedule_time' => __('Time', 'wc_conditional_content'),
            ),
            __('Stock', 'wc_conditional_content')      => array(
                'stock_status' => __('Stock Status', 'wc_conditional_content'),
                'stock_level'  => __('Stock Level', 'wc_conditional_content'),
            ),
            __('Sales', 'wc_conditional_content')      => array(
                'sale_schedule' => __('Sale Date', 'wc_conditional_content'),
                'sale_status'   => __('Sale Status', 'wc_conditional_content'),
            ),
            __("User Role", 'wc_conditional_content') => array(
                'users_user'           => __("User", 'wc_conditional_content'),
                'users_role'           => __("Role", 'wc_conditional_content'),
                'users_authentication' => __('Authentication', 'wc_conditional_content'),
            ),
            __("Store", 'wc_conditional_content')      => array(
                'store_order_count' => __("Order Count and Status", 'wc_conditional_content'),
            ),
            __("Cart", 'wc_conditional_content')       => array(
                'cart_total'              => __("Cart Total", 'wc_conditional_content'),
                'cart_product'            => __("Cart Products", 'wc_conditional_content'),
                'cart_category'           => __("Cart Categories", 'wc_conditional_content'),
                'cart_line_item_product'  => __("Cart Line Item Product", 'wc_conditional_content'),
                'cart_line_item_quantity' => __("Cart Line Item Quantity", 'wc_conditional_content'),
            ),
            __('Geography', 'wc_conditional_content')  => array(
                'geo_country_code' => __('Country', 'wc_conditional_content'),
                'wpml_language'    => __('WPML Language Code', 'wc_conditional_content'),
            ),
            __('Taxonomy', 'wc_conditional_content')   => array(
                'taxonomy_product_category' => __('Product Category Archive'),
            ),
        );

        return $types;
    }

    public static function dynamicblock_what_display_dataset_ajax($options = array())
    {
        $is_ajax = false;
        if (isset($_POST['action']) && sanitize_text_field($_POST['action']) == 'dynamicblock_what_display_dataset_ajax') {
            $is_ajax = true;
        }
        $post_id = get_the_ID();
        if ($is_ajax) {
            $options = array(
                'group_id'  => sanitize_text_field($_POST['group_id']),
                'rule_id'   => sanitize_text_field($_POST['rule_id']),
                'rule_type' => sanitize_text_field($_POST['rule_type']),
                'condition' => null,
                'operator'  => null,
                'action'    => sanitize_text_field($_POST['action']),
                'post_id'   => sanitize_text_field($_POST['post_id']),
            );
            $post_id = sanitize_text_field($_POST['post_id']);
        }
        $rule_object = self::dynamicblock_woocommerce_conditional_content_get_rule_object($options['rule_type']);
        if (! empty($rule_object)) {
            $values               = $rule_object->get_possible_rule_values();
            $operators            = $rule_object->get_possible_rule_operators();
            $condition_input_type = $rule_object->get_condition_input_type();
            // create operators field
            $operator_args = array(
                'input'   => 'select',
                'name'    => 'dynamicblock_post[dynamicblock_post_' . $post_id . '][what_to_display][post_query_rules][' . $options['group_id'] . '][dynamicblock_query][' . $options['rule_id'] . '][condition]',
                'choices' => $operators,
            );
            echo '<div class="dynamicblock-select-condition">';
            if (! empty($operators)) {
                DynamicBlock_Conditional_Content_Input_Builder::create_input_field($operator_args,
                    $options['operator']);
            } else {
                echo '<input type="hidden" name="' . esc_attr($operator_args['name']) . '" value="==" />';
            }
            echo '</div>';
            // create values field
            $value_args = array(
                'input'   => $condition_input_type,
                'name'    => 'dynamicblock_post[dynamicblock_post_' . $post_id . '][what_to_display][post_query_rules][' . $options['group_id'] . '][dynamicblock_query][' . $options['rule_id'] . '][tagfields]',
                'choices' => $values,
            );
            echo '<div class="dynamicblock-select-value">';
            Dynamicblock_Conditional_Content_Input_Builder::create_input_field($value_args, $options['condition']);
            echo '</div>';
        }
        if ($is_ajax) {
            die();
        }
    }

    public static function dynamicblock_when_display_dataset_ajax($options = array())
    {
        $is_ajax = false;
        if (isset($_POST['action']) && sanitize_text_field($_POST['action']) == 'dynamicblock_when_display_dataset_ajax') {
            $is_ajax = true;
        }
        $post_id = get_the_ID();
        if ($is_ajax) {
            $options = array(
                'group_id'  => sanitize_text_field($_POST['group_id']),
                'rule_id'   => sanitize_text_field($_POST['rule_id']),
                'rule_type' => sanitize_text_field($_POST['rule_type']),
                'condition' => null,
                'operator'  => null,
                'action'    => sanitize_text_field($_POST['action']),
                'post_id'   => sanitize_text_field($_POST['post_id']),
            );
            $post_id = $_POST['post_id'];
        }
        $rule_object = self::dynamicblock_woocommerce_conditional_content_get_rule_object($options['rule_type']);
        if (! empty($rule_object)) {
            $values               = $rule_object->get_possible_rule_values();
            $operators            = $rule_object->get_possible_rule_operators();
            $condition_input_type = $rule_object->get_condition_input_type();
            // create operators field
            $operator_args = array(
                'input'   => 'select',
                'name'    => 'dynamicblock_post[dynamicblock_post_' . $post_id . '][when_to_display][post_query_rules][' . $options['group_id'] . '][dynamicblock_query][' . $options['rule_id'] . '][condition]',
                'choices' => $operators,
            );
            echo '<div  class="dynamicblock-select-condition">';
            if (! empty($operators)) {
                DynamicBlock_Conditional_Content_Input_Builder::create_input_field($operator_args,
                    $options['operator']);
            } else {
                echo '<input type="hidden" name="' . esc_attr($operator_args['name']) . '" value="==" />';
            }
            echo '</div>';
            // create values field

            $placeholder = 'Enter your text here.';
            $class       = '';
            if ($options['rule_type'] == 'checkout_billing_region_province_field' || $options['rule_type'] == 'checkout_shipping_region_province_field') {
                $placeholder = 'Enter State Code Ex. California : "CA"';
                $class       = 'dynamicblock_field_capitalize ';
            }
            if ($options['rule_type'] == 'checkout_billing_zip_field' || $options['rule_type'] == 'checkout_billing_phone' || $options['rule_type'] == 'checkout_shipping_zip_field' || $options['rule_type'] == 'checkout_shipping_phone') {
                $class = 'dynamicblock_field_appearance_none ';
            }
            $value_args = array(
                'input'       => $condition_input_type,
                'placeholder' => $placeholder,
                'class'       => $class,
                'name'        => 'dynamicblock_post[dynamicblock_post_' . $post_id . '][when_to_display][post_query_rules][' . $options['group_id'] . '][dynamicblock_query][' . $options['rule_id'] . '][tagfields]',
                'choices'     => $values,
            );
            echo '<div class="dynamicblock-select-value">';
            Dynamicblock_Conditional_Content_Input_Builder::create_input_field($value_args, $options['condition']);
            echo '</div>';
        }
        if ($is_ajax) {
            die();
        }
    }

    public static function dynamicblock_woocommerce_conditional_content_get_rule_object($rule_type)
    {
        global $dynamicblock_woocommerce_conditional_content_rules;
        if (isset($dynamicblock_woocommerce_conditional_content_rules[$rule_type])) {
            return $dynamicblock_woocommerce_conditional_content_rules[$rule_type];
        }
        $class = 'Dynamicblock_Conditional_Content_Rule_' . $rule_type;
        if (class_exists($class)) {
            $dynamicblock_woocommerce_conditional_content_rules[$rule_type] = new $class;

            return $dynamicblock_woocommerce_conditional_content_rules[$rule_type];
        } else {
            return null;
        }
    }

    public static function dynamicblock_woocommerce_conditional_content_get_input_object($input_type)
    {
        global $dynamicblock_woocommerce_conditional_content_inputs;
        if (isset($dynamicblock_woocommerce_conditional_content_inputs[$input_type])) {
            return $dynamicblock_woocommerce_conditional_content_inputs[$input_type];
        }
        $class = 'DynamicBlock_Conditional_Content_Input_' . str_replace(' ', '_',
                ucwords(str_replace('-', ' ', $input_type)));
        if (class_exists($class)) {
            $dynamicblock_woocommerce_conditional_content_inputs[$input_type] = new $class;
        } else {
            $dynamicblock_woocommerce_conditional_content_inputs[$input_type] = apply_filters('woocommerce_conditional_content_get_input_object',
                $input_type);
        }

        return $dynamicblock_woocommerce_conditional_content_inputs[$input_type];
    }

    public static function Dynamicblock_Conditional_Content_get_product_types()
    {
        return get_terms('product_type', array('hide_empty' => false));
    }

    public static function Dynamicblock_conditional_content_get_all_product_categories()
    {
        return get_terms('product_cat', array('hide_empty' => false));
    }

    public static function Dynamicblock_conditional_content_get_all_yith_product_brand()
    {
        return get_terms(DYNAMICBLOCK_YITH_WOOCOMMERCE_PRODUCT_BRAND, array('hide_empty' => false));
    }

    public static function Dynamicblock_conditional_content_get_all_product_tags()
    {
        return get_terms('product_tag', array('hide_empty' => false));
    }

    public function add_custom_fees(WC_Cart $cart)
    {
        if ($cart->cart_contents_count < 5) {
            return;
        }
        // Calculate the amount to reduce
        $discount = $cart->subtotal * 0.1;
        $cart->add_fee('You have more than 5 items in your cart, a 10% discount has been added.', -$discount);
    }

    public static function dynamicblock_sanitize_array(&$array)
    {
        foreach ($array as &$value) {
            // sanitize if value is not an array
            if (! is_array($value)) {
                if (array_key_exists('dynamicblock_editor_html', $array)) {
                    $value = wp_kses_post($value);
                } else {
                    $value = sanitize_text_field($value);
                }
            } else {
                // go inside this function again
                DynamicBlock_Admin::dynamicblock_sanitize_array($value);
            }
        }

        return $array;
    }
}


new DynamicBlock_Admin;
