<?php
if (!defined('ABSPATH'))
    exit; // Exit if accessed directly

class Dynamicblock_Content_Compatibility
{

    public static function dynamicblock_get_wc_version()
    {

        // WOOCOMMERCE_VERSION is now WC_VERSION, though WOOCOMMERCE_VERSION is still available for backwards compatibility, we'll disregard it on 2.1+
        if (defined('WC_VERSION') && WC_VERSION)
            return WC_VERSION;
        if (defined('WOOCOMMERCE_VERSION') && WOOCOMMERCE_VERSION)
            return WOOCOMMERCE_VERSION;

        return null;
    }


    public static function dynamicblock_is_wc_version_gte_2_1()
    {
        // can't use gte 2.1 at the moment because 2.1-BETA < 2.1
        return self::dynamicblock_is_wc_version_gt('2.0.20');
    }


    public static function dynamicblock_woocommerce_get_formatted_product_name($product)
    {
        if (self::dynamicblock_is_wc_version_gte_2_1()) {
            return $product->get_formatted_name();
        } else {
            return woocommerce_get_formatted_product_name($product);
        }
    }

    public static function is_wc_version_gte_2_7()
    {
        return version_compare(self::dynamicblock_get_wc_version(), '2.7.0', 'ge');
    }

    public static function dynamicblock_is_wc_version_gt($version)
    {
        return self::dynamicblock_get_wc_version() && version_compare(self::dynamicblock_get_wc_version(), $version, '>');
    }

    public static function dynamicblock_get_attribute_taxonomies()
    {

        global $woocommerce;
        if (self::dynamicblock_is_wc_version_gte_2_1()) {
            return wc_get_attribute_taxonomies();
        } else {

            return $woocommerce->get_attribute_taxonomies();
        }
    }

    public static function dynamicblock_attribute_taxonomy_name($name)
    {
        if (self::dynamicblock_is_wc_version_gte_2_1()) {

            return wc_attribute_taxonomy_name($name);
        } else {

            global $woocommerce;
            return $woocommerce->attribute_taxonomy_name($name);
        }
    }


}
