<?php
/**
 * DynamicBlock Register Post Type, Taxonomy, Custom Meta Fields
 */

defined('ABSPATH') || exit;

class DynamicBlock_Post_Types
{

    // Register DynamicBlock Block Post type

    function __construct()
    {
        add_action('print_array', [$this, 'pr']);
        add_action('init', [$this, 'dynamicblock_register']);
        add_action('bp_before_post_content', [$this, 'meso_add_feat_img_single']);
        add_action('save_post', [$this, 'dynamicblock_save_note_control']);
        add_action('save_post', [$this, 'dynamicblock_save_post_meta'], 10, 2);
        add_action('save_post', [$this, 'dynamicblock_save_short_code_control']);
        add_filter('the_content', [$this, 'dynamicblock_title_control_before']);
        add_action('add_meta_boxes', [$this, 'dynamicblock_shortcode_add_meta_box'], 2);
        add_filter('the_content', [$this, 'dynamicblock_short_code_control_before']);
        add_action('add_meta_boxes', [$this, 'dynamicblock_where_to_display_add_meta_box']);
        add_filter('the_content', [$this, 'dynamicblock_where_to_display_control_before']);
        add_action('admin_notices', [$this, 'dynamicblock_title_page']);
        add_filter('the_content', [$this, 'dynamicblock_note_control_before']);
        add_filter('use_block_editor_for_post_type', [$this, 'dynamicblock_disable_gutenberg'], 10, 2);
        add_filter('screen_options_show_screen', [$this, 'dynamicblock_remove_screen_options']);
        add_action('edit_form_after_title', [$this, 'dynamicblock_form_after_title']);
    }

    public function pr($r)
    {
        echo '<pre>';
        print_r($r);
        echo '</pre>';
    }

    public function dynamicblock_form_after_title($post)
    {
        if ($post->post_type === 'dynamicblock-block') {
            include DYNAMICBLOCK_TEMPLATE_PATH . 'admin/dynamicblock-edit-block.php';
        }
    }

    public function dynamicblock_disable_gutenberg($current_status, $post_type)
    {
        // Use your post type key instead of 'dynamicblock-block'
        if ($post_type === 'dynamicblock-block') return false;
        return $current_status;
    }

    public function dynamicblock_remove_screen_options()
    {
        if (isset($_GET['post_type']) && sanitize_text_field($_GET['post_type']) == 'dynamicblock-block') {
            return false;
        }
    }

    public function dynamicblock_title_page($post)
    {
        global $pagenow;
        $post = (isset($_GET['post']) == 75 && !empty($_GET['post'])) ? sanitize_text_field($_GET['post']) : '';

        $is_all_post_type = ('post.php' == $pagenow && 75 == $post);

        if ($is_all_post_type) {
            include DYNAMICBLOCK_TEMPLATE_PATH . 'admin/dynamicblock_page_title.php';
        }
    }

    public function dynamicblock_register()
    {
        $labels = array(
            'name' => _x('Add Block', DYNAMICBLOCK_TEXT),
            'singular_name' => _x('DynamicBlock', DYNAMICBLOCK_TEXT),
            'menu_name' => _x('DynamicBlock', DYNAMICBLOCK_TEXT),
            'name_admin_bar' => _x('block', DYNAMICBLOCK_TEXT),
            'add_new' => __('block', DYNAMICBLOCK_TEXT),
            'add_new_item' => __('Add New block', DYNAMICBLOCK_TEXT),
            'new_item' => __('New block', DYNAMICBLOCK_TEXT),
            'edit_item' => __('Edit block', DYNAMICBLOCK_TEXT),
            'view_item' => __('View block', DYNAMICBLOCK_TEXT),
            'all_items' => __('Dashboard', DYNAMICBLOCK_TEXT),
            'search_items' => __('Search block', DYNAMICBLOCK_TEXT),
            'parent_item_colon' => __('Parent block:', DYNAMICBLOCK_TEXT),
            'not_found' => __('No block found.', DYNAMICBLOCK_TEXT),
            'not_found_in_trash' => __('No block found in Trash.', DYNAMICBLOCK_TEXT),
            'featured_image' => _x('block Cover Image', DYNAMICBLOCK_TEXT),
            'set_featured_image' => _x('Set cover image', DYNAMICBLOCK_TEXT),
            'remove_featured_image' => _x('Remove cover image', DYNAMICBLOCK_TEXT),
            'use_featured_image' => _x('Use as cover image', DYNAMICBLOCK_TEXT),
            'archives' => _x('block archives', DYNAMICBLOCK_TEXT),
            'insert_into_item' => _x('Insert into block', DYNAMICBLOCK_TEXT),
            'uploaded_to_this_item' => _x('Uploaded to this block', DYNAMICBLOCK_TEXT),
            'filter_items_list' => _x('Filter block list', DYNAMICBLOCK_TEXT),
            'items_list_navigation' => _x('Block list navigation', DYNAMICBLOCK_TEXT),
            'items_list' => _x('Block list', DYNAMICBLOCK_TEXT),
        );
        $args = array(
            'labels' => $labels,
            'description' => 'block custom post type.',
            'public' => false,
            'publicly_queryable' => true,
            'show_ui' => true,
            'show_in_menu' => false,
            'query_var' => true,
            'rewrite' => array('slug' => 'dynamicblock-block'),
            'capability_type' => 'post',
            'has_archive' => true,
            'hierarchical' => false,
            'menu_position' => 20,
            'supports' => array(''),
            'show_in_rest' => true
        );

        register_post_type('dynamicblock-block', $args);
    }

    public function dynamicblock_save_note_control($post_id)
    {
        if (!isset($_POST['note_control_nonce'])) {
            return;
        }

        if (!wp_verify_nonce($_POST['note_control_nonce'], 'note_control_nonce')) {
            return;
        }

        if (defined('DOING_AUTOSAVE') && DOING_AUTOSAVE) {
            return;
        }

        if (isset($_POST['post_type']) && 'page' == sanitize_text_field($_POST['post_type'])) {
            if (!current_user_can('edit_page', $post_id)) {
                return;
            }
        } else {
            if (!current_user_can('edit_post', $post_id)) {
                return;
            }
        }

        if (!isset($_POST['note_control'])) {
            return;
        }

        // Sanitize user input.
        $my_data = sanitize_text_field($_POST['note_control']);
        // Update the meta field in the database.
        update_post_meta($post_id, 'note_control', $my_data);
    }

    public function dynamicblock_note_control_before($content)
    {
        global $post;

        // retrieve the global notice for the current post
        $global_notice = esc_attr(get_post_meta($post->ID, 'note_control', true));
        echo esc_attr($global_notice);
        $notice = "<div class='sp_global_notice'>$global_notice</div>";

        return $notice . $content;
    }

    public function dynamicblock_callback($post)
    {
        wp_nonce_field('title_control_nonce', 'title_control_nonce');
        $value = get_post_meta($post->ID, "title_control", true);
        echo "<input type='text' style='width:100%' id='title_control' name='title_control' value='" . esc_attr($value) . "'>";
    }

    public function dynamicblock_title_control_before($content)
    {
        global $post;

        // retrieve the global notice for the current post
        $global_notice = esc_attr(get_post_meta($post->ID, 'title_control', true));
        echo esc_attr($global_notice);
        $notice = "<div class='sp_global_notice'>$global_notice</div>";

        return $notice . $content;
    }

    public function dynamicblock_shortcode_add_meta_box()
    {
        add_meta_box(
            'short_code_meta', 'Shortcode', array($this, 'dynamicblock_shortcode_callback'),
            'dynamicblock-block',
            'side'
        );
    }

    public function dynamicblock_shortcode_callback($post)
    {
        wp_nonce_field('short_code_control_nonce', 'short_code_control_nonce');
        $rules = get_post_meta(get_the_ID(), 'dynamicblock_post');
        $block_type = $rules[0]['dynamicblock_post_' . get_the_ID()]['block_info']['post_block_type'] ?? (sanitize_text_field($_GET['block_type']) ?? '');
        $value = get_post_meta($post->ID, "short_code_control", true);
        ?>
        <div class="dynamicblock-shortcode short_code_meta">
            <p id="">You can also display the block on custom pages using this shortcode</p>
                <input type="text" readonly onclick="copy_shortcode()" id="short_code_control" name="short_code_control" value="[dynamicblock_block id=<?php echo esc_attr(get_the_ID())?>]">
            <p id="short_code_control_copied"></p>
        </div>
        <?php
    }

    public function dynamicblock_save_short_code_control($post_id)
    {
        if (!isset($_POST['short_code_control_nonce'])) {
            return;
        }

        if (!wp_verify_nonce($_POST['short_code_control_nonce'], 'short_code_control_nonce')) {
            return;
        }

        if (defined('DOING_AUTOSAVE') && DOING_AUTOSAVE) {
            return;
        }

        if (isset($_POST['post_type']) && 'page' == sanitize_text_field($_POST['post_type'])) {
            if (!current_user_can('edit_page', $post_id)) {
                return;
            }

        } else {
            if (!current_user_can('edit_post', $post_id)) {
                return;
            }
        }

        if (!isset($_POST['short_code_control'])) {
            return;
        }

        // Sanitize user input.
        $my_data = sanitize_text_field($_POST['short_code_control']);
        // Update the meta field in the database.
        update_post_meta($post_id, 'short_code_control', $my_data);
    }

    public function dynamicblock_short_code_control_before($content)
    {
        global $post;
        // retrieve the global notice for the current post
        $global_notice = esc_attr(get_post_meta($post->ID, 'short_code_control', true));
        echo esc_attr($global_notice);
        $notice = "<div class='sp_global_notice'>$global_notice</div>";
        return $notice . $content;
    }

    public function dynamicblock_where_to_display_add_meta_box()
    {
        $screens = array('post', 'page');
        add_meta_box(
            'dynamicblock_where_to_display_meta', 'Where to Display', array($this, 'dynamicblock_where_to_display_callback'),
            'dynamicblock-block',
            'side'
        );
    }

    public function dynamicblock_where_to_display_callback($post)
    {
        wp_nonce_field('where_to_display_control', 'where_to_display_control_nonce');
        $_where_to_display_control = get_post_meta($post->ID, 'where_to_display_control', true);
        require_once DYNAMICBLOCK_ABSPATH . 'admin/meta-boxes/where-display-meta-box.php';
    }

    public function dynamicblock_save_post_meta($post_id)
    {
        if (isset($_POST['where_to_display_control'])) {
            $where_to_display_control = DynamicBlock_Admin::dynamicblock_sanitize_array($_POST['where_to_display_control']);
            update_post_meta($post_id, 'where_to_display_control', $where_to_display_control);
        }
        if (isset($_POST['dynamicblock_post'])) {
            $dynamicblock_post = DynamicBlock_Admin::dynamicblock_sanitize_array($_POST['dynamicblock_post']);
            update_post_meta($post_id, 'dynamicblock_post', $dynamicblock_post);
        }
		if (isset($_POST['dynamicblock_block_type'])) {
            $dynamicblock_block_type = DynamicBlock_Admin::dynamicblock_sanitize_array($_POST['dynamicblock_block_type']);
            update_post_meta($post_id, 'dynamicblock_block_type', $dynamicblock_block_type);
        }
    }

    public function dynamicblock_where_to_display_control_before($content)
    {
        global $post;

        // retrieve the global notice for the current post
        $global_notice = esc_attr(get_post_meta($post->ID, 'where_to_display_control', true));
        $global_notice1 = esc_attr(get_post_meta($post->ID, 'wp_hook_control', true));
        $global_notice2 = esc_attr(get_post_meta($post->ID, 'wp_priority_control', true));

        echo esc_attr($global_notice);
        $notice = "<div class='sp_global_notice'>$global_notice</div>";
        $notice1 = "<div class='sp_global_notice'>$global_notice1</div>";
        $notice2 = "<div class='sp_global_notice'>$global_notice2</div>";

        return $notice . $notice1 . $notice2 . $content;
    }
}

new DynamicBlock_Post_Types();

add_action('init', 'dynamicblock_products_save');
function dynamicblock_products_save() {
     if(empty(get_transient( 'dynamicblock_products_list'))) {
         global $wpdb;
         $result = $wpdb->get_results( "SELECT ID , post_title FROM `wp_posts` WHERE `post_type` = 'product' and `post_status` = 'publish' ORDER BY `post_title` ASC;");
        $all_product_List = $result;
        $special_query_results= array();
        $j=0;

        if(isset($all_product_List)){
            foreach ($all_product_List as $list){
                $special_query_results[$j]['id']=$list->ID;
                $special_query_results[$j]['title']=$list->post_title;
                $j++;
            }
        }
        set_transient( 'dynamicblock_products_list', $special_query_results, 3600 );
    }
}

function dynamicblock_get_all_products() {
     return get_transient( 'dynamicblock_products_list');
}