<?php
defined('ABSPATH') || exit;

/**
 * Class DynamicBlock_Display_Rules
 */
class DynamicBlock_Display_Rules
{
    public function __construct()
    {
    }

    /**
     * @param $type
     *
     * @return mixed|void
     */
    public static function when_to_display_rule_types($type = '')
    {
        $types = array(
            __('Global', DYNAMICBLOCK_TEXT)     => array(
                'show_always' => __('Show Always', DYNAMICBLOCK_TEXT),
            ),
            __("Product", DYNAMICBLOCK_TEXT)    => apply_filters('dynamicblock_when_to_display_rule_type_product',
                array(
                    'products'          => __('Products', DYNAMICBLOCK_TEXT),
                    'product_type'      => __("Product Type", DYNAMICBLOCK_TEXT),
                    'product_category'  => __("Product Category", DYNAMICBLOCK_TEXT),
                    'product_attribute' => __("Product Attributes", DYNAMICBLOCK_TEXT),
                    'product_tag'       => __("Product Tag", DYNAMICBLOCK_TEXT),
                    'product_price'     => __("Product Price", DYNAMICBLOCK_TEXT),
                    'pro_version'     => __("Product Set", DYNAMICBLOCK_TEXT) . ' - ' . __('Update to PRO version', DYNAMICBLOCK_TEXT),
                )),
            __('Stock', DYNAMICBLOCK_TEXT)      => apply_filters('dynamicblock_when_to_display_rule_type_stock', array(
                'pro_version' => __('Update to PRO version', DYNAMICBLOCK_TEXT),
            )),
            __('Sales', DYNAMICBLOCK_TEXT)      => apply_filters('dynamicblock_when_to_display_rule_type_sales', array(
                'sale_date'   => __('Sale Date', DYNAMICBLOCK_TEXT),
                'sale_status' => __('Sale Status', DYNAMICBLOCK_TEXT),
            )),
            __("Cart", DYNAMICBLOCK_TEXT)       => apply_filters('dynamicblock_when_to_display_rule_type_cart', array(
                'pro_version' => __('Update to PRO version', DYNAMICBLOCK_TEXT),
            )),
            __("User Role", DYNAMICBLOCK_TEXT) => apply_filters('dynamicblock_when_to_display_rule_type_membership',
                array(
                    'pro_version' => __('Update to PRO version', DYNAMICBLOCK_TEXT),
                )),
            __("Checkout", DYNAMICBLOCK_TEXT)   => apply_filters('dynamicblock_when_to_display_rule_type_checkout',
                array(
                    'pro_version' => __('Update to PRO version', DYNAMICBLOCK_TEXT),
                )),

            __("Browser & Device", DYNAMICBLOCK_TEXT) => apply_filters('dynamicblock_what_to_display_rule_type_browser_and_device',
                array(
                    'pro_version' => __('Update to PRO version', DYNAMICBLOCK_TEXT),
                )),

            __("Screen Size", DYNAMICBLOCK_TEXT) => apply_filters('dynamicblock_what_to_display_rule_type_screen_size',
                array(
                    'pro_version' => __('Update to PRO version', DYNAMICBLOCK_TEXT),
                )),

            __("Metadata", DYNAMICBLOCK_TEXT) => apply_filters('dynamicblock_what_to_display_rule_type_metadata',
                array(
                    'pro_version' => __('Update to PRO version', DYNAMICBLOCK_TEXT),
                )),
            __("Date & Time", DYNAMICBLOCK_TEXT) => apply_filters('dynamicblock_what_to_display_rule_type_date_and_time',
                array(
                    'schedule'          => __('Schedule', DYNAMICBLOCK_TEXT),
                )),
            __("URL Source", DYNAMICBLOCK_TEXT) => apply_filters('dynamicblock_what_to_display_rule_type_url_source',
                array(
                    'url_path'          => __('Url Path', DYNAMICBLOCK_TEXT),
                   // 'referral_source'   => __('Referral source', DYNAMICBLOCK_TEXT), // 12-10-2023 temporary commented
                    'query_string'      => __('Query String', DYNAMICBLOCK_TEXT),
                ) ),
        );

        if (in_array('yith-woocommerce-brands-add-on/init.php',
                apply_filters('active_plugins', get_option('active_plugins'))) ||
            in_array('yith-woocommerce-brands-add-on-premium/init.php',
                apply_filters('active_plugins', get_option('active_plugins')))
        ) {
            $types["YITH Brands Add-On"] = array("brands" => "Brands");
        }

//        if($type) {
//            $types = array($type => $types[$type]);
//        }

        return apply_filters('dynamicblock_when_to_display_rule_types', $types);
    }

    /**
     * @param $type
     *
     * @return mixed|void
     */
    public static function what_to_display_rule_types($type = '')
    {

        $types = array(
            __("Product", DYNAMICBLOCK_TEXT)    => apply_filters('dynamicblock_what_to_display_rule_type_product',
                array(
                    'products'          => __('Products', DYNAMICBLOCK_TEXT),
                    'product_type'      => __("Product Type", DYNAMICBLOCK_TEXT),
                    'product_category'  => __("Product Category", DYNAMICBLOCK_TEXT),
                    'product_attribute' => __("Product Attributes", DYNAMICBLOCK_TEXT),
                    'product_tag'       => __("Product Tag", DYNAMICBLOCK_TEXT),
                    'product_price'     => __("Product Price", DYNAMICBLOCK_TEXT),
                    'pro_version'     => __("Product Set", DYNAMICBLOCK_TEXT) . ' - ' . __('Update to PRO version', DYNAMICBLOCK_TEXT),
                )),
            __('Stock', DYNAMICBLOCK_TEXT)      => apply_filters('dynamicblock_what_to_display_rule_type_stock', array(
                'pro_version' => __('Update to PRO version', DYNAMICBLOCK_TEXT),
            )),
            __("User Role", DYNAMICBLOCK_TEXT) => apply_filters('dynamicblock_what_to_display_rule_type_membership',
                array(
                    'pro_version' => __('Update to PRO version', DYNAMICBLOCK_TEXT),
                )),
            __("Store", DYNAMICBLOCK_TEXT)      => apply_filters('dynamicblock_what_to_display_rule_type_store', array(
                'pro_version' => __('Update to PRO version', DYNAMICBLOCK_TEXT),
            )),
        );
        if (in_array('yith-woocommerce-brands-add-on/init.php',
                apply_filters('active_plugins', get_option('active_plugins'))) ||
            in_array('yith-woocommerce-brands-add-on-premium/init.php',
                apply_filters('active_plugins', get_option('active_plugins')))
        ) {
            $types["YITH Brands Add-On "] = array("brands" => "Brands");
        }

//        if($type) {
//            $types = (array)$types[$type];
//        }

        return apply_filters('dynamicblock_what_to_display_rule_types', $types);
    }
}