<?php

defined('ABSPATH') || exit;

class Dynamicblock_Conditional_Content_Rule_browser extends Dynamicblock_Conditional_Content_Rule_Base {

    public function __construct() {
        parent::__construct( 'browser' );
    }

    public function get_possible_rule_operators() {
        $operators = array(
            'in' => __( "is", 'DYNAMICBLOCK_TEXT' ),
            'notin' => __( "is not", 'DYNAMICBLOCK_TEXT' ),
        );

        return $operators;
    }

    public function get_condition_input_type() {
        return 'Chosen_Select';
    }
    public function get_possible_rule_values() {

        $options = array(
            'chrome' 			=> __( 'Chrome', 'DYNAMICBLOCK_TEXT' ),
            'firefox' 			=> __( 'Firefox', 'DYNAMICBLOCK_TEXT' ),
            'microsoft_edge' 	=> __( 'Microsoft Edge', 'DYNAMICBLOCK_TEXT' ),
            'internet_explorer' => __( 'Internet Explorer', 'DYNAMICBLOCK_TEXT' ),
            'opera' 			=> __( 'Opera', 'DYNAMICBLOCK_TEXT' ),
            'safari' 			=> __( 'Safari', 'DYNAMICBLOCK_TEXT' ),
            'samsung_internet' 	=> __( 'Samsung Internet', 'DYNAMICBLOCK_TEXT' ),
        );
		
		return $options;
    }
    public function is_match( $rule_data, $arguments = null ) {
        $result = false;
        $user_browser = $this->detectBrowser();
        
        if ( $user_browser && isset( $rule_data['condition'] ) && isset( $rule_data['operator'] ) ) 
        {
            $in = in_array($user_browser, $rule_data['condition']);
            $result = $rule_data['operator'] == 'in' ? $in : !$in;
        }

        return $this->return_is_match( $result, $rule_data, $arguments );
    }

    public function detectBrowser()
	{
	    $user_agent = $_SERVER['HTTP_USER_AGENT'];
	    $browser = "N/A";

	    $browsers = [
            '/msie/i'           => 'internet_explorer',
            '/firefox/i'        => 'firefox',
            '/safari/i'         => 'safari',
            '/chrome/i'         => 'chrome',
            '/edg/i'            => 'microsoft_edge',
            '/opr/i'            => 'opera',
            '/samsungBrowser/i' => 'samsung_internet',
        ];

        foreach ($browsers as $regex => $value) {
	        if (preg_match($regex, $user_agent)) {
	            $browser = $value;
	        }
	    }
	    return $browser;
	}


}

class Dynamicblock_Conditional_Content_Rule_platform extends Dynamicblock_Conditional_Content_Rule_Base {

    public function __construct() {
        parent::__construct( 'platform' );
    }

    public function get_possible_rule_operators() {
        $operators = array(
            'in' => __( "is", 'DYNAMICBLOCK_TEXT' ),
            'notin' => __( "is not", 'DYNAMICBLOCK_TEXT' ),
        );

        return $operators;
    }

    public function get_condition_input_type() {
        return 'Chosen_Select';
    }
    public function get_possible_rule_values() {

        $options = array(
            'android' 	=> __( 'Android', 'DYNAMICBLOCK_TEXT' ),
            'ios' 		=> __( 'iOS (iPhone and iPad)', 'DYNAMICBLOCK_TEXT' ),
            'linux' 	=> __( 'Linux', 'DYNAMICBLOCK_TEXT' ),
            'macintosh' => __( 'Macintosh', 'DYNAMICBLOCK_TEXT' ),
            'windows' 	=> __( 'Windows', 'DYNAMICBLOCK_TEXT' ),
        );
		
		return $options;
    }
    public function is_match( $rule_data, $arguments = null ) {
        $result = false;
        $product = wc_get_product( get_the_ID() );

        $user_platform = $this->detectPlatform();
        
        if ( $user_platform && isset( $rule_data['condition'] ) && isset( $rule_data['operator'] ) ) 
        {
            $in = in_array($user_platform, $rule_data['condition']);
            $result = $rule_data['operator'] == 'in' ? $in : !$in;
        }
		
        return $this->return_is_match( $result, $rule_data, $arguments );
    }

    public function detectPlatform()
    {
        $platform = false;
        $user_agent = getenv("HTTP_USER_AGENT");
        if(strpos($user_agent, "Win") !== FALSE)
            $platform = "windows";
        elseif(strpos($user_agent, "Macintos") !== FALSE)
            $platform = "macintosh";
        elseif(strpos($user_agent, "Mac") !== FALSE)
            $platform = "ios";
        elseif(strpos($user_agent, "Android") !== FALSE)
            $platform = "android";
        elseif(strpos($user_agent, "Linux") !== FALSE)
            $platform = "linux";
        return $platform;
    }

}


class Dynamicblock_Conditional_Content_Rule_device_type extends Dynamicblock_Conditional_Content_Rule_Base {

    public function __construct() {
        parent::__construct( 'device_type' );
    }

    public function get_possible_rule_operators() {
        $operators = array(
            'in' => __( "is", 'DYNAMICBLOCK_TEXT' ),
            'notin' => __( "is not", 'DYNAMICBLOCK_TEXT' ),
        );
        return $operators;
    }

    public function get_condition_input_type() {
        return 'Chosen_Select';
    }
    public function get_possible_rule_values() {

        $options = array(
            'mobile' 	=> __( 'Is Mobile', 'DYNAMICBLOCK_TEXT' ),
            'robot' 	=> __( 'Is Robot', 'DYNAMICBLOCK_TEXT' ),
            'other' 	=> __( 'Is Desktop (Other)', 'DYNAMICBLOCK_TEXT' ),
        );
		return $options;
    }
    public function is_match( $rule_data, $arguments = null ) {
        $result = false;
        $user_device =  $this->detectDevice();

        if ( $user_device && isset( $rule_data['condition'] ) && isset( $rule_data['operator'] ) ) 
        {
            $in = in_array($user_device, $rule_data['condition']);
            $result = $rule_data['operator'] == 'in' ? $in : !$in;
        }
        
        return $this->return_is_match( $result, $rule_data, $arguments );

    }

    public function detectDevice()
    {
        $user_agent = getenv("HTTP_USER_AGENT");
        $is_bot = false;

        // Bots list 
        $bot_list = array(
            'Googlebot', 'Baiduspider', 'ia_archiver',
            'R6_FeedFetcher', 'NetcraftSurveyAgent',
            'Sogou web spider', 'bingbot', 'Yahoo! Slurp',
            'facebookexternalhit', 'PrintfulBot', 'msnbot',
            'Twitterbot', 'UnwindFetchor', 'urlresolver'
        );

        foreach($bot_list as $bl) 
        {
            if( stripos( $user_agent, $bl ) !== false ) $is_bot = true;
        } 
              
        if(  is_numeric( strpos( strtolower($user_agent) , "mobile")) ){
            return "mobile";
        }elseif( $is_bot == true ){
            return "robot";
        }else{
            return "other";
        }
    }

}