<?php
defined('ABSPATH') || exit;

class Dynamicblock_Conditional_Content_Rule_Cart_Total_Quantity extends Dynamicblock_Conditional_Content_Rule_Base {

	public function __construct() {
		parent::__construct( 'cart_total_quantity' );
	}

	public function get_possible_rule_operators() {
		$operators = array(
			'==' => __( "is equal to", 'Dynamicblock_Conditional_Content' ),
			'!=' => __( "is not equal to", 'Dynamicblock_Conditional_Content' ),
			'>'  => __( "is greater than", 'Dynamicblock_Conditional_Content' ),
			'<'  => __( "is less than", 'Dynamicblock_Conditional_Content' ),
			'>=' => __( "is greater or equal to", 'Dynamicblock_Conditional_Content' ),
			'<=' => __( "is less or equal to", 'Dynamicblock_Conditional_Content' )
		);

		return $operators;
	}

	public function get_condition_input_type() {
		return 'Text';
	}

	public function is_match( $rule_data, $arguments = null ) {
		$result = false;
        $total_quantity =  WC()->cart->cart_contents_count;
		$value = (float) $rule_data['condition'];

		$rule_data['operator'] = htmlspecialchars_decode($rule_data['operator']);
		switch ( $rule_data['operator'] ) {
			case '==' :
				$result = $total_quantity == $value;
				break;
			case '!=' :
				$result = $total_quantity != $value;
				break;
			case '>' :
				$result = $total_quantity > $value;
				break;
			case '<' :
				$result = $total_quantity < $value;
				break;
			case '>=' :
				$result = $total_quantity >= $value;
				break;
			case '<=' :
				$result = $total_quantity <= $value;
				break;
			default:
				$result = false;
				break;
		}

		return $this->return_is_match( $result, $rule_data, $arguments );
	}

}

class Dynamicblock_Conditional_Content_Rule_Cart_Total_Value_Amount extends Dynamicblock_Conditional_Content_Rule_Base {

	public function __construct() {
		parent::__construct( 'cart_total_value_amount' );
	}

	public function get_possible_rule_operators() {
		$operators = array(
			'==' => __( "is equal to", 'Dynamicblock_Conditional_Content' ),
			'!=' => __( "is not equal to", 'Dynamicblock_Conditional_Content' ),
			'>'  => __( "is greater than", 'Dynamicblock_Conditional_Content' ),
			'<'  => __( "is less than", 'Dynamicblock_Conditional_Content' ),
			'>=' => __( "is greater or equal to", 'Dynamicblock_Conditional_Content' ),
			'<=' => __( "is less or equal to", 'Dynamicblock_Conditional_Content' )
		);

		return $operators;
	}

	public function get_condition_input_type() {
		return 'Text';
	}

	public function is_match( $rule_data, $arguments = null ) {


		$result = false;

		if ( ! wc_prices_include_tax() ) {
			$price = WC()->cart->get_cart_contents_total();
		} else {
			$price = WC()->cart->get_cart_contents_total() + WC()->cart->get_cart_tax();
		}

		$value = (float) $rule_data['condition'];
		$rule_data['operator'] = htmlspecialchars_decode($rule_data['operator']);
		switch ( $rule_data['operator'] ) {
			case '==' :
				$result = $price == $value;
				break;
			case '!=' :
				$result = $price != $value;
				break;
			case '>' :
				$result = $price > $value;
				break;
			case '<' :
				$result = $price < $value;
				break;
			case '>=' :
				$result = $price >= $value;
				break;
			case '<=' :
				$result = $price <= $value;
				break;
			default:
				$result = false;
				break;
		}

		return $this->return_is_match( $result, $rule_data, $arguments );
	}

}

class Dynamicblock_Conditional_Content_Rule_Cart_Product extends Dynamicblock_Conditional_Content_Rule_Base {

	public function __construct() {
		parent::__construct( 'cart_product' );
	}

	public function get_possible_rule_operators() {

		$operators = array(
			'<'  => __( "contains less than", 'Dynamicblock_Conditional_Content' ),
			'>'  => __( "contains at least", 'Dynamicblock_Conditional_Content' ),
			'==' => __( "contains exactly", 'Dynamicblock_Conditional_Content' ),
		);

		return $operators;
	}

	public function get_condition_input_type() {
		return 'Cart_Product_Select';
	}

	public function is_match( $rule_data, $arguments = null ) {
		$result        = false;
		$cart_contents = WC()->cart->get_cart();

		$products = $rule_data['condition']['product'];
		$quantity = $rule_data['condition']['qty'];
		$type     = $rule_data['operator'];

		$found_quantity = 0;

		if (isset($products) && !empty($products) &&   $cart_contents && count( $cart_contents ) ) {
			foreach ( $cart_contents as $cart_item_key => $cart_item ) {
				if ( in_array( '0', $products ) ) {
					$found_quantity += $cart_item['quantity'];
				} elseif ( in_array( $cart_item['product_id'], $products ) || ( isset( $cart_item['variation_id'] ) && in_array( $cart_item['variation_id'], $products ) ) ) {
					$found_quantity += $cart_item['quantity'];
				}
			}
		}

		switch ( $type ) {
			case '<' :
				$result = $quantity >= $found_quantity;
				break;
			case '>' :
				$result = $quantity <= $found_quantity;
				break;
			case '==' :
				$result = $quantity == $found_quantity;
				break;
			default :
				$result = false;
				break;
		}

		return $this->return_is_match( $result, $rule_data, $arguments );
	}

}

class Dynamicblock_Conditional_Content_Rule_Cart_Category extends Dynamicblock_Conditional_Content_Rule_Base {

	public function __construct() {
		parent::__construct( 'cart_category' );
	}

	public function get_possible_rule_operators() {

		$operators = array(
			'<'  => __( "contains less than", 'Dynamicblock_Conditional_Content' ),
			'>'  => __( "contains at least", 'Dynamicblock_Conditional_Content' ),
			'==' => __( "contains exactly", 'Dynamicblock_Conditional_Content' ),
		);

		return $operators;
	}

	public function get_possible_rule_values() {
		$result = array();

		$terms = DynamicBlock_Admin::Dynamicblock_Conditional_Content_get_all_product_categories();
		if ( $terms && ! is_wp_error( $terms ) ) {
			foreach ( $terms as $term ) {
				$result[ $term->term_id ] = $term->name;
			}
		}

		return $result;
	}

	public function get_condition_input_type() {
		return 'Cart_Category_Select';
	}

	public function is_match( $rule_data, $arguments = null ) {


		$result        = false;
		$cart_contents = WC()->cart->get_cart();

		$categories = $rule_data['condition']['categories'];
		$quantity   = $rule_data['condition']['qty'];
		$type       = $rule_data['operator'];

		$found_quantity = 0;

		if ( $cart_contents && count( $cart_contents ) ) {
			foreach ( $cart_contents as $cart_item_key => $cart_item ) {
				$product = $cart_item['data'];
				if ( $product->is_type( 'variation' ) ) {
					$product = wc_get_product( $product->get_parent_id() );
				}
				$terms = $product->get_category_ids();
                foreach ($terms as $term){
                   $_cat = get_term($term);
                   if($_cat->parent) {
                       $terms[] = $_cat->parent;
                   }
                }
				if ( $terms && ! is_wp_error( $terms ) && count( array_intersect((array)$terms, (array)$categories ) ) > 0 ) {
					$found_quantity += $cart_item['quantity'];
				}
			}
		}
		switch ( $type ) {
			case '<' :
				$result = $quantity > $found_quantity;
				break;
			case '>' :
				$result = $quantity <= $found_quantity;
				break;
			case '==' :
				$result = $quantity == $found_quantity;
				break;
			default :
				$result = false;
				break;
		}

		return $this->return_is_match( $result, $rule_data, $arguments );
	}

}

class Dynamicblock_Conditional_Content_Rule_Cart_Line_Item_Product extends Dynamicblock_Conditional_Content_Rule_Base {

	public function __construct() {
		parent::__construct( 'cart_line_item_product' );
	}

	public function get_possible_rule_operators() {
		$operators = array(
			'in'    => __( "in", 'Dynamicblock_Conditional_Content' ),
			'notin' => __( "not in", 'Dynamicblock_Conditional_Content' ),
		);

		return $operators;
	}

	public function get_condition_input_type() {
		return 'Product_Select';
	}

	public function is_match( $rule_data, $arguments = null ) {


		$result  = false;
        $cart_contents = WC()->cart->get_cart();
        $products = $rule_data['condition'];
        $in = false;
        if (isset($products) && !empty($products) ) {
            foreach ( $cart_contents as $cart_item_key => $cart_item ) {
                if ( $cart_item && isset( $rule_data['condition'] ) && isset( $rule_data['operator'] ) ) {
                    if(in_array( $cart_item['product_id'], $rule_data['condition'] )){
                        $in = true;
                        break;
                    }
                }
            }
        }

        if($in && $rule_data['operator'] == 'in'){
            $result = true;
        }else if(!$in && $rule_data['operator'] != 'in'){
            $result = true;
        }
		return $this->return_is_match( $result, $rule_data, $arguments );
	}

}

class Dynamicblock_Conditional_Content_Rule_Cart_Line_Item_Quantity extends Dynamicblock_Conditional_Content_Rule_Base {

	public function __construct() {
		parent::__construct( 'cart_line_item_quantity' );
	}

	public function get_possible_rule_operators() {
		$operators = array(
			'==' => __( "is equal to", 'Dynamicblock_Conditional_Content' ),
			'!=' => __( "is not equal to", 'Dynamicblock_Conditional_Content' ),
			'>'  => __( "is greater than", 'Dynamicblock_Conditional_Content' ),
			'<'  => __( "is less than", 'Dynamicblock_Conditional_Content' ),
			'>=' => __( "is greater or equal to", 'Dynamicblock_Conditional_Content' ),
			'<=' => __( "is less or equal to", 'Dynamicblock_Conditional_Content' )
		);

		return $operators;
	}

	public function get_condition_input_type() {
		return 'Text';
	}


	public function is_match( $rule_data, $arguments = null ) {
		$result         = false;
		$quantity       = $rule_data['condition']; //The quantity input.
		$type           = $rule_data['operator'];
		$found_quantity = 0;
        $cart_contents = WC()->cart->get_cart();
        $in = false;
        if (isset($quantity) && !empty($quantity) ) {
            foreach ( $cart_contents as $cart_item_key => $cart_item ) {
                if ( $cart_item && isset( $rule_data['condition'] ) && isset( $rule_data['operator'] ) ) {
                    if($cart_item['product_id'] == get_the_ID()){
                        $found_quantity = $cart_item['quantity'];
                    }
                }
            }
        }

        $rule_data['operator'] = htmlspecialchars_decode($rule_data['operator']);
		switch ( $rule_data['operator'] ) {
			case '==' :
				$result = $found_quantity == $quantity;
				break;
			case '!=' :
				$result = $found_quantity != $quantity;
				break;
			case '>' :
				$result = $found_quantity > $quantity;
				break;
			case '<' :
				$result = $found_quantity < $quantity;
				break;
			case '>=' :
				$result = $found_quantity >= $quantity;
				break;
			case '<=' :
				$result = $found_quantity <= $quantity;
				break;
			default:
				$result = false;
				break;
		}

		return $this->return_is_match( $result, $rule_data, $arguments );
	}

}
