<?php
defined('ABSPATH') || exit;

class Dynamicblock_Conditional_Content_Rule_Checkout_Shipping_Cost_Items extends Dynamicblock_Conditional_Content_Rule_Base {

    public function __construct() {
        parent::__construct( 'checkout_shipping_cost_items' );
    }

    public function get_possible_rule_operators() {
        $operators = array(
            '==' => __( "is equal to", 'DYNAMICBLOCK_TEXT' ),
            '!=' => __( "is not equal to", 'DYNAMICBLOCK_TEXT' ),
            '>'  => __( "is greater than", 'DYNAMICBLOCK_TEXT' ),
            '<'  => __( "is less than", 'DYNAMICBLOCK_TEXT' ),
            '>=' => __( "is greater or equal to", 'DYNAMICBLOCK_TEXT' ),
            '<=' => __( "is less or equal to", 'DYNAMICBLOCK_TEXT' )
        );

        return $operators;
    }

    public function get_condition_input_type() {
        return 'Text';
    }
	
	//EDIT MCESUS
	public function is_match( $rule_data, $arguments = null ) {

		$result = false;
		$price = WC()->cart->shipping_total;
		$value = (float) $rule_data['condition'];
        $rule_data['operator'] = htmlspecialchars_decode($rule_data['operator']);

		switch ( $rule_data['operator'] ) {
			case '==' :
				$result = $price == $value;
				break;
			case '!=' :
				$result = $price != $value;
				break;
			case '>' :
				$result = $price > $value;
				break;
			case '<' :
				$result = $price < $value;
				break;
			case '>=' :
				$result = $price >= $value;
				break;
			case '<=' :
				$result = $price <= $value;
				break;
			default:
				$result = false;
				break;
		}

		return $this->return_is_match( $result, $rule_data, $arguments );
	}
	
    /*public function is_match( $rule_data, $arguments = null ) {
        $result         = false;
        $quantity       = $rule_data['condition']; 		
        $type           = $rule_data['operator'];
        $found_quantity = 0;
		
		
        if ( ! empty( $arguments ) && isset( $arguments[0] ) && isset( $arguments[0]['quantity'] ) ) {
            $found_quantity = $arguments[0]['quantity'];
        }
		
        switch ( $rule_data['operator'] ) {
            case '==' :
                $result = $found_quantity == $quantity;
                break;
            case '!=' :
                $result = $found_quantity != $quantity;
                break;
            case '>' :
                $result = $found_quantity > $quantity;
                break;
            case '<' :
                $result = $found_quantity < $quantity;
                break;
            case '>=' :
                $result = $found_quantity >= $quantity;
                break;
            case '<=' :
                $result = $found_quantity <= $quantity;
                break;
            default:
                $result = false;
                break;
        }
        return $this->return_is_match( $result, $rule_data, $arguments );
    }*/

}

class Dynamicblock_Conditional_Content_Rule_Checkout_Email_Field extends Dynamicblock_Conditional_Content_Rule_Base {

    public function __construct() {
        parent::__construct( 'Checkout_email_field' );
    }

    public function get_possible_rule_operators() {
        $operators = array(
            '=='  => __( "contains", 'DYNAMICBLOCK_TEXT' ),
            '!='  => __( "not contains", 'DYNAMICBLOCK_TEXT' ),
        );

        return $operators;
    }

    public function get_condition_input_type() {
        return 'Text_Area';
    }

    public function is_match( $rule_data, $arguments = null ) {
        $result         = false;
        $_rule_value         = $rule_data['condition']; 
        $type           = $rule_data['operator'];

        $user_id = get_current_user_id();
        $customer_val = '';
        if($user_id) {
            $customer = new WC_Customer( $user_id );
            $customer_val = $customer->get_email();
        }
        $rule_data['operator'] = htmlspecialchars_decode($rule_data['operator']);
        switch ( $rule_data['operator'] ) {
            case '==' :
                if(strpos($customer_val,$_rule_value)!== false){
                    $result = 1;
                }else{
                    $result = false;
                }

                break;
            case '!=' :
                if(strpos($customer_val,$_rule_value)!== false){
                    $result = false;
                }else{
                    $result = 1;
                }
                break;
            default:
                $result = false;
                break;
        }
        return $this->return_is_match( $result, $rule_data, $arguments );
    }

}

class Dynamicblock_Conditional_Content_Rule_Checkout_Payment_Method extends Dynamicblock_Conditional_Content_Rule_Base {

    public function __construct() {
        parent::__construct( 'checkout_payment_method' );
    }

    public function get_possible_rule_operators() {
        $operators = array(
            'in'    => __( "in", 'DYNAMICBLOCK_TEXT' ),
            'notin' => __( "not in", 'DYNAMICBLOCK_TEXT' ),
        );

        return $operators;
    }

    public function get_condition_input_type() {
        return 'Chosen-Select';

    }

    public function get_possible_rule_values() {
        $gateways = WC()->payment_gateways->get_available_payment_gateways();
        $payment_method = array();
        foreach ($gateways  as $key => $gateway) {
            $payment_method[$key]=$gateway->title;
        }
        return $payment_method;
    }

    public function is_match( $rule_data, $arguments = null ) {
        $result  = false;
        $payment_methods = $this->get_possible_rule_values();
        if ( $payment_methods && isset( $rule_data['condition'] ) && isset( $rule_data['operator'] ) ) {
            foreach($payment_methods as $key => $payment_method) {
                $in     = in_array( $key, $rule_data['condition'] );
                $result = $rule_data['operator'] == 'in' ? $in : ! $in;
            }
        }

        return $this->return_is_match( $result, $rule_data, $arguments );
    }

}

class Dynamicblock_Conditional_Content_Rule_Checkout_Billing_Country_Field extends Dynamicblock_Conditional_Content_Rule_Base {

    public function __construct() {
        parent::__construct( 'checkout_billing_country_field' );
    }

    public function get_possible_rule_operators() {

        $operators = array(
            'in' => __( "in", 'DYNAMICBLOCK_TEXT' ),
            'notin' => __( "in not", 'DYNAMICBLOCK_TEXT' ),
        );

        return $operators;
    }

    public function get_possible_rule_values() {
        return WC()->countries->get_allowed_countries();
    }

    public function get_condition_input_type() {
        return 'Chosen-Select';
    }

    public function is_match( $rule_data, $arguments = null ) {

        $result = false;
        if ( isset( $rule_data['condition'] ) && isset( $rule_data['operator'] ) ) {

            $location = WC_Geolocation::geolocate_ip();
            $country = '';
            if ( empty( $location['country'] ) ) {
                $location = wc_format_country_state_string( apply_filters( 'woocommerce_customer_default_location', get_option( 'woocommerce_default_country' ) ) );
                $country = $location['country'];
            }

            $user_id = get_current_user_id();
            if($user_id) {
                $customer = new WC_Customer( $user_id );
                $country = $customer->get_billing_country();
            }
            
            switch ( $rule_data['operator'] ) {
                case 'in' :
                    $result = in_array($country, $rule_data['condition']);
                    break;
                case 'notin' :
                    $result = !in_array($country, $rule_data['condition']);
                    break;
                default:
                    $result = false;
                    break;
            }
        }
        
        return $this->return_is_match($result, $rule_data, $arguments);
    }

}

class Dynamicblock_Conditional_Content_Rule_Checkout_Billing_City_Field extends Dynamicblock_Conditional_Content_Rule_Base {

    public function __construct() {
        parent::__construct( 'checkout_billing_city_field' );
    }

    public function get_possible_rule_operators() {
        $operators = array(
            'in'    => __( "in", 'DYNAMICBLOCK_TEXT' ),
            'notin' => __( "not in", 'DYNAMICBLOCK_TEXT' ),
        );

        return $operators;
    }

    public function get_condition_input_type() {
        return 'Text_Area';
    }

    public function is_match( $rule_data, $arguments = null ) {
        $result  = false;
        if ( isset( $rule_data['condition'] ) && isset( $rule_data['operator'] ) ) {

            $user_id = get_current_user_id();
            if($user_id) {
                $customer = new WC_Customer( $user_id );
                $city = $customer->get_billing_city();
            }
            
            switch ( $rule_data['operator'] ) {
                case 'in' :
                    $result = strtolower($city) == strtolower($rule_data['condition']);
                    break;
                case 'notin' :
                    $result = strtolower($city) != strtolower($rule_data['condition']);
                    break;
                default:
                    $result = false;
                    break;
            }
        }
        
        return $this->return_is_match( $result, $rule_data, $arguments );
    }

}

class Dynamicblock_Conditional_Content_Rule_Checkout_Billing_Region_Province_Field extends Dynamicblock_Conditional_Content_Rule_Base {

    public function __construct() {
        parent::__construct( 'checkout_billing_region_province_field' );
    }

    public function get_possible_rule_operators() {
        $operators = array(
            'in'    => __( "in", 'DYNAMICBLOCK_TEXT' ),
            'notin' => __( "not in", 'DYNAMICBLOCK_TEXT' ),
        );

        return $operators;
    }

    public function get_condition_input_type() {
        return 'Text_Area';
    }

    public function is_match( $rule_data, $arguments = null ) {

        $result  = false;
        $user_id = get_current_user_id();
        $region  = '';
        if($user_id) {
            $customer = new WC_Customer( $user_id );
            $region = $customer->get_billing_state();
        }

        switch ( $rule_data['operator'] ) {
            case 'in' :
                $result = strtolower($region) == strtolower($rule_data['condition']);
                break;
            case 'notin' :
                $result = strtolower($region) != strtolower($rule_data['condition']);
                break;
            default:
                $result = false;
                break;
        }

        return $this->return_is_match( $result, $rule_data, $arguments );
    }

}

class Dynamicblock_Conditional_Content_Rule_Checkout_Billing_Zip_Field extends Dynamicblock_Conditional_Content_Rule_Base {

    public function __construct() {
        parent::__construct( 'checkout_billing_zip_field' );
    }

    public function get_possible_rule_operators() {
        $operators = array(
            'in'    => __( "in", 'DYNAMICBLOCK_TEXT' ),
            'notin' => __( "not in", 'DYNAMICBLOCK_TEXT' ),
        );

        return $operators;
    }

    public function get_condition_input_type() {
        return 'Number';
    }

    public function is_match( $rule_data, $arguments = null ) {
        $result  = false;
        $user_id = get_current_user_id();
        $zip = '';
        if($user_id) {
            $customer = new WC_Customer( $user_id );
            $zip = $customer->get_billing_postcode();
        }
        
        switch ( $rule_data['operator'] ) {
            case 'in' :
                $result = strtolower($zip) == strtolower($rule_data['condition']);
                break;
            case 'notin' :
                $result = strtolower($zip) != strtolower($rule_data['condition']);
                break;
            default:
                $result = false;
                break;
        }

        return $this->return_is_match( $result, $rule_data, $arguments );
    }

}

class Dynamicblock_Conditional_Content_Rule_Checkout_Billing_Phone extends Dynamicblock_Conditional_Content_Rule_Base {

    public function __construct() {
        parent::__construct( 'checkout_billing_phone' );
    }

    public function get_possible_rule_operators() {
        $operators = array(
            '=='  => __( "contains", 'DYNAMICBLOCK_TEXT' ),
            '!='  => __( "not contains", 'DYNAMICBLOCK_TEXT' ),
        );

        return $operators;
    }

    public function get_condition_input_type() {
        return 'Number';
    }


    public function is_match( $rule_data, $arguments = null ) {
        $result         = false;
        $quantity       = $rule_data['condition']; 
        $type           = $rule_data['operator'];
        $user_id = get_current_user_id();
        $phone = '';
        if($user_id) {
            $customer = new WC_Customer( $user_id );
            $phone = $customer->get_billing_phone();
        }
        
        switch ( $rule_data['operator'] ) {
            case '==' :
                $result = strtolower($phone) == strtolower($rule_data['condition']);
                break;
            case '!=' :
                $result = strtolower($phone) != strtolower($rule_data['condition']);
                break;
            default:
                $result = false;
                break;
        }

        return $this->return_is_match( $result, $rule_data, $arguments );
    }

}

class Dynamicblock_Conditional_Content_Rule_Checkout_Shipping_Country_Field extends Dynamicblock_Conditional_Content_Rule_Base {

    public function __construct() {
        parent::__construct( 'checkout_billing_country_field' );
    }

    public function get_possible_rule_operators() {

        $operators = array(
            '==' => __( "in", 'DYNAMICBLOCK_TEXT' ),
            '!=' => __( "in not", 'DYNAMICBLOCK_TEXT' ),
        );

        return $operators;
    }

    public function get_possible_rule_values() {
        return WC()->countries->get_allowed_countries();
    }

    public function get_condition_input_type() {
        return 'Chosen-Select';
    }

    public function is_match( $rule_data, $arguments = null ) {
        $result = false;
        if ( isset( $rule_data['condition'] ) && isset( $rule_data['operator'] ) ) {

            $location = WC_Geolocation::geolocate_ip();
            $country = '';
            if ( empty( $location['country'] ) ) {
                $location = wc_format_country_state_string( apply_filters( 'woocommerce_customer_default_location', get_option( 'woocommerce_default_country' ) ) );
                $country = $location['country'];
            }

            $user_id = get_current_user_id();
            if($user_id) {
                $customer = new WC_Customer( $user_id );
                $country = $customer->get_shipping_country();
            }
            
            switch ( $rule_data['operator'] ) {
                case '==' :
                    $result = in_array($country, $rule_data['condition']);
                    break;
                case '!=' :
                    $result = !in_array($country, $rule_data['condition']);
                    break;
                default:
                    $result = false;
                    break;
            }
        }
        return $this->return_is_match($result, $rule_data, $arguments);
    }

}

class Dynamicblock_Conditional_Content_Rule_Checkout_shipping_city_field extends Dynamicblock_Conditional_Content_Rule_Base {

    public function __construct() {
        parent::__construct( 'Checkout_shipping_city_field' );
    }

    public function get_possible_rule_operators() {
        $operators = array(
            'in'    => __( "in", 'DYNAMICBLOCK_TEXT' ),
            'notin' => __( "not in", 'DYNAMICBLOCK_TEXT' ),
        );

        return $operators;
    }

    public function get_condition_input_type() {
        return 'Text_Area';
    }

    public function is_match( $rule_data, $arguments = null ) {
        $result  = false;
        if ( isset( $rule_data['condition'] ) && isset( $rule_data['operator'] ) ) {

            $user_id = get_current_user_id();
            if($user_id) {
                $customer = new WC_Customer( $user_id );
                $city = $customer->get_shipping_city();
            }
            
            switch ( $rule_data['operator'] ) {
                case 'in' :
                    $result = strtolower($city) == strtolower($rule_data['condition']);
                    break;
                case 'notin' :
                    $result = strtolower($city) != strtolower($rule_data['condition']);
                    break;
                default:
                    $result = false;
                    break;
            }
        }

        return $this->return_is_match( $result, $rule_data, $arguments );
    }

}

class Dynamicblock_Conditional_Content_Rule_Checkout_Shipping_Region_Province_Field extends Dynamicblock_Conditional_Content_Rule_Base {

    public function __construct() {
        parent::__construct( 'checkout_shipping_region_province_field' );
    }

    public function get_possible_rule_operators() {
        $operators = array(
            'in'    => __( "in", 'DYNAMICBLOCK_TEXT' ),
            'notin' => __( "not in", 'DYNAMICBLOCK_TEXT' ),
        );

        return $operators;
    }

    public function get_condition_input_type() {
        return 'Text_Area';
    }

    public function is_match( $rule_data, $arguments = null ) {
        $result  = false;
        $user_id = get_current_user_id();
        $region  = '';
        if($user_id) {
            $customer = new WC_Customer( $user_id );
            $region = $customer->get_shipping_state();
        }
        
        switch ( $rule_data['operator'] ) {
            case 'in' :
                $result = strtolower($region) == strtolower($rule_data['condition']);
                break;
            case 'notin' :
                $result = strtolower($region) != strtolower($rule_data['condition']);
                break;
            default:
                $result = false;
                break;
        }

        return $this->return_is_match( $result, $rule_data, $arguments );
    }

}

class Dynamicblock_Conditional_Content_Rule_Checkout_Shipping_Zip_Field extends Dynamicblock_Conditional_Content_Rule_Base {

    public function __construct() {
        parent::__construct( 'checkout_shipping_zip_field' );
    }

    public function get_possible_rule_operators() {
        $operators = array(
            'in'    => __( "in", 'DYNAMICBLOCK_TEXT' ),
            'notin' => __( "not in", 'DYNAMICBLOCK_TEXT' ),
        );

        return $operators;
    }

    public function get_condition_input_type() {
        return 'Number';
    }

    public function is_match( $rule_data, $arguments = null ) {
        $result  = false;
        $zip = '';
        $user_id = get_current_user_id();
        if(isset($user_id)) {
            $customer = new WC_Customer( $user_id );
            $zip = $customer->get_shipping_postcode();
        }
        
        switch ( $rule_data['operator'] ) {
            case 'in' :
                $result = strtolower($zip) == strtolower($rule_data['condition']);
                break;
            case 'notin' :
                $result = strtolower($zip) != strtolower($rule_data['condition']);
                break;
            default:
                $result = false;
                break;
        }

        return $this->return_is_match( $result, $rule_data, $arguments );
    }

}

class Dynamicblock_Conditional_Content_Rule_Checkout_Shipping_Phone extends Dynamicblock_Conditional_Content_Rule_Base {

    public function __construct() {
        parent::__construct( 'checkout_shipping_phone' );
    }

    public function get_possible_rule_operators() {
        $operators = array(
            '=='  => __( "contains", 'DYNAMICBLOCK_TEXT' ),
            '!='  => __( "not contains", 'DYNAMICBLOCK_TEXT' ),
        );

        return $operators;
    }

    public function get_condition_input_type() {
        return 'Number';
    }


    public function is_match( $rule_data, $arguments = null ) {
        $result         = false;
        $quantity       = $rule_data['condition']; 
        $type           = $rule_data['operator'];
        $user_id = get_current_user_id();
        $phone = '';
        if($user_id) {
            $customer = new WC_Customer( $user_id );
            $phone = $customer->get_billing_phone();
        }
        
        switch ( $rule_data['operator'] ) {
            case '==' :
                $result = strtolower($phone) == strtolower($rule_data['condition']);
                break;
            case '!=' :
                $result = strtolower($phone) != strtolower($rule_data['condition']);
                break;
            default:
                $result = false;
                break;
        }

        return $this->return_is_match( $result, $rule_data, $arguments );
    }

}
