<?php

defined('ABSPATH') || exit;

class Dynamicblock_Conditional_Content_Rule_User  extends Dynamicblock_Conditional_Content_Rule_Base {

    public function __construct() {
        parent::__construct( 'user' );
    }

    public function get_possible_rule_operators() {
        $operators = array(
            'in'    => __( "is", 'Dynamicblock_Conditional_Content' ),
            'notin' => __( "is not", 'Dynamicblock_Conditional_Content' ),
        );

        return $operators;
    }

    public function get_possible_rule_values() {
        $result = array();
        global $wpdb;
        $users = $wpdb->get_results('SELECT ID,display_name FROM `wp_users`');
        if ( $users ) {
            foreach ( $users as $user ) {
                $result[ $user->ID ] = $user->display_name;
            }
        }

        return $result;
    }

    public function get_condition_input_type() {
        return 'Chosen_Select';
    }

    public function is_match( $rule_data, $arguments = null ) {
        $result = in_array( get_current_user_id(), $rule_data['condition'] );
        $result = $rule_data['operator'] == 'in' ? $result : ! $result;

        return $this->return_is_match( $result, $rule_data, $arguments );
    }
}

class Dynamicblock_Conditional_Content_Rule_Role extends Dynamicblock_Conditional_Content_Rule_Base {

    public function __construct() {
        parent::__construct( 'role' );
    }

    public function get_possible_rule_operators() {
        $operators = array(
            'in'    => __( "is", 'Dynamicblock_Conditional_Content' ),
            'notin' => __( "is not", 'Dynamicblock_Conditional_Content' ),
        );

        return $operators;
    }

    public function get_possible_rule_values() {
        $result = array();

        $editable_roles = get_editable_roles();

        if ( $editable_roles ) {
            foreach ( $editable_roles as $role => $details ) {
                $name            = translate_user_role( $details['name'] );
                $result[ $role ] = $name;
            }
        }

        return $result;
    }

    public function get_condition_input_type() {
        return 'Chosen_Select';
    }

    public function is_match( $rule_data, $arguments = null ) {
        $result = false;
        if ( $rule_data['condition'] && is_array( $rule_data['condition'] ) ) {
            foreach ( $rule_data['condition'] as $role ) {
                $result |= current_user_can( $role );
            }
        }

        $result = $rule_data['operator'] == 'in' ? $result : ! $result;

        return $this->return_is_match( $result, $rule_data, $arguments );
    }

    public function sort_attribute_taxonomies( $taxa, $taxb ) {
        return strcmp( $taxa->attribute_name, $taxb->attribute_name );
    }

}

class Dynamicblock_Conditional_Content_Rule_Authentication extends Dynamicblock_Conditional_Content_Rule_Base {

    public function __construct() {
        parent::__construct( 'user' );
    }

    public function get_possible_rule_operators() {
        $operators = array(
            '==' => __( "is", 'Dynamicblock_Conditional_Content' ),
            '!=' => __( "is not", 'Dynamicblock_Conditional_Content' ),
        );

        return $operators;
    }

    public function get_possible_rule_values() {
        $result = array(
            'logged-out' => 'Logged Out',
            'logged-in'  => 'Logged In'
        );

        return $result;
    }

    public function get_condition_input_type() {
        return 'Select';
    }

    public function is_match( $rule_data, $arguments = null ) {
        $result = false;
        if ( $rule_data['condition'] ) {

            if ($rule_data['condition'] == 'logged-in') {
                if ( $rule_data['operator'] == '==' ) {
                    $result = is_user_logged_in();
                } else if ( $rule_data['operator'] == '!=' ) {
                    $result = ! is_user_logged_in();
                }
            } elseif ($rule_data['condition'] == 'logged-out') {
                if ( $rule_data['operator'] == '==' ) {
                    $result = !is_user_logged_in();
                } else if ( $rule_data['operator'] == '!=' ) {
                    $result = is_user_logged_in();
                }
            }
        }

        return $this->return_is_match( $result, $rule_data, $arguments );
    }
}
