<?php

defined('ABSPATH') || exit;

class Dynamicblock_Conditional_Content_Rule_Products extends Dynamicblock_Conditional_Content_Rule_Base {

    public function __construct() {
        parent::__construct( 'products' );
    }

    public function get_possible_rule_operators() {
        $operators = array(
            'in' => __( "is", 'DYNAMICBLOCK_TEXT' ),
            'notin' => __( "is not", 'DYNAMICBLOCK_TEXT' ),
        );

        return $operators;
    }

    public function get_condition_input_type() {
        return 'Product_Select';
    }

    public function is_match( $rule_data, $arguments = null ) {
        $result = false;
        $product = wc_get_product( get_the_ID() );
        if ( $product && isset( $rule_data['condition'] ) && isset( $rule_data['operator'] ) ) {
            $in = in_array( $product->get_id(), $rule_data['condition'] );
            $result = $rule_data['operator'] == 'in' ? $in : !$in;
        }

        return $this->return_is_match( $result, $rule_data, $arguments );
    }

}

class Dynamicblock_Conditional_Content_Rule_Product_Set extends Dynamicblock_Conditional_Content_Rule_Base {

    public function __construct() {
        parent::__construct( 'product_set' );
    }

    public function get_possible_rule_operators() {
        $operators = array(
            'in' => __( "is", 'DYNAMICBLOCK_TEXT' ),
            'notin' => __( "is not", 'DYNAMICBLOCK_TEXT' ),
        );

        return $operators;
    }

    public function get_condition_input_type() {
        return 'Select';
    }
    public function get_possible_rule_values() {

        $options = array(
            'featured' => __( 'Featured', 'DYNAMICBLOCK_TEXT' ),
            'best_selling' => __( 'Best Selling', 'DYNAMICBLOCK_TEXT' ),
            'top_rated' => __( 'Top rated', 'DYNAMICBLOCK_TEXT' ),
            'recent' => __( 'Recent', 'DYNAMICBLOCK_TEXT' ),
            'on_sale' => __( 'On Sale', 'DYNAMICBLOCK_TEXT' ),
        );

        return $options;
    }
    public function is_match( $rule_data, $arguments = null ) {
        $result = false;
        $product = wc_get_product( get_the_ID() );

        if ( $product && isset( $rule_data['condition'] ) && isset( $rule_data['operator'] ) ) {

        	$type = array();

			if ( $product->is_featured() ) {
        		$type[] = 'featured';
			}

			if ( $product->is_on_sale() ) {
        		$type[] = 'on_sale';
			}

			if ( $product->get_average_rating() > 4 ) {
        		$type[] = 'top_rated';
			}

			if ( strtotime( $product->get_date_created() ) > strtotime('-3 month') ) {
        		$type[] = 'recent';
			}

			if ( $product->get_total_sales() > 10 && strtotime( $product->get_date_created() ) > strtotime('-365 day') ) {
        		$type[] = 'best_selling';
			}

			$in = count( array_intersect( $type, $rule_data['condition'] ) ) > 0;

			$result = $rule_data['operator'] == 'in' ? $in : !$in;

        }

        return $this->return_is_match( $result, $rule_data, $arguments );
    }

}

class Dynamicblock_Conditional_Content_Rule_Product_Type extends Dynamicblock_Conditional_Content_Rule_Base {

	public function __construct() {
		parent::__construct( 'product_type' );
	}

	public function get_possible_rule_operators() {
		$operators = array(
		    'in' => __( "is", 'DYNAMICBLOCK_TEXT' ),
		    'notin' => __( "is not", 'DYNAMICBLOCK_TEXT' ),
		);

		return $operators;
	}

	public function get_possible_rule_values() {
		$result = array();

		$terms = DynamicBlock_Admin::Dynamicblock_Conditional_Content_get_product_types();
		if ( $terms && !is_wp_error( $terms ) ) {
			foreach ( $terms as $term ) {
				$result[$term->term_id] = $term->name;
			}
		}

		return $result;
	}

	public function get_condition_input_type() {
		return 'Chosen_Select';
	}

	public function is_match( $rule_data, $arguments = null ) {
		
		$result = false;
		$product = wc_get_product( get_the_ID() );





		if ( $product && isset( $rule_data['condition'] ) && isset( $rule_data['operator'] ) ) {
			$product_types = wp_get_post_terms( $product->get_id(), 'product_type', array('fields' => 'ids') );
			$in = count( array_intersect( $product_types, $rule_data['condition'] ) ) > 0;
			$result = $rule_data['operator'] == 'in' ? $in : !$in;
		}

		return $this->return_is_match( $result, $rule_data, $arguments );
	}

}

class Dynamicblock_Conditional_Content_Rule_Product_Category extends Dynamicblock_Conditional_Content_Rule_Base {

	public function __construct() {
		parent::__construct( 'product_category' );
	}

	public function get_possible_rule_operators() {
		$operators = array(
		    'in' => __( "in", 'DYNAMICBLOCK_TEXT' ),
		    'notin' => __( "not in", 'DYNAMICBLOCK_TEXT' ),
		);

		return $operators;
	}

    public function get_possible_rule_values() {
        $result = array();

        $terms = DynamicBlock_Admin::Dynamicblock_conditional_content_get_all_product_categories();
        if ( $terms && !is_wp_error( $terms ) ) {
            foreach ( $terms as $term ) {
                $result[$term->term_id] = $term->name;
            }
        }

        return $result;
    }

    public function get_condition_input_type() {
        return 'Chosen_Select';
    }

    public function is_match( $rule_data, $arguments = null ) {
        $product = wc_get_product( get_the_ID() );
        $result = false;
        if( is_single()){
            if ( isset($product) && $product && isset( $rule_data['condition'] ) && isset( $rule_data['operator'] ) ) {
                $terms = $product->get_category_ids();
                $in = count( array_intersect( $terms, $rule_data['condition'] ) ) > 0;
                $result = $rule_data['operator'] == 'in' ? $in : !$in ;
            }
        }else if(is_archive()){
            $category = get_queried_object();
            if(isset($category)){
                $cat_id = isset($category->term_id) ? $category->term_id : '';
                if(in_array( $cat_id,$rule_data['condition'])){
                    $result = $rule_data['operator'] == 'in' ? 1 : 0 ;
                }else{
                    $result = $rule_data['operator'] == 'in' ? 0 : 1 ;
                }
            }
        }
        /*if ( isset($product) && $product && isset( $rule_data['condition'] ) && isset( $rule_data['operator'] ) ) {
            $terms = $product->get_category_ids();
            $in = count( array_intersect( $terms, $rule_data['condition'] ) ) > 0;
            $result = $rule_data['operator'] == 'in' ? $in : !$in ;
        }*/

        return $this->return_is_match( $result, $rule_data, $arguments );
    }

}

class Dynamicblock_Conditional_Content_Rule_Product_Attribute extends Dynamicblock_Conditional_Content_Rule_Base {

	public function __construct() {
		parent::__construct( 'product_attribute' );
	}

	public function get_possible_rule_operators() {
		$operators = array(
		    'in' => __( "has", 'DYNAMICBLOCK_TEXT' ),
		    'notin' => __( "does not have", 'DYNAMICBLOCK_TEXT' ),
		);

		return $operators;
	}

	public function get_possible_rule_values() {
		global $woocommerce;

		$result = array();

		$attribute_taxonomies = Dynamicblock_Content_Compatibility::dynamicblock_get_attribute_taxonomies();

		if ( $attribute_taxonomies ) {
			foreach ( $attribute_taxonomies as $tax ) {
				$attribute_taxonomy_name = Dynamicblock_Content_Compatibility::dynamicblock_attribute_taxonomy_name( $tax->attribute_name );
				if ( taxonomy_exists( $attribute_taxonomy_name ) ) {
					$terms = get_terms( $attribute_taxonomy_name, 'orderby=name&hide_empty=0' );
					if ( $terms ) {
						foreach ( $terms as $term ) {
							$result[$attribute_taxonomy_name . '|' . $term->term_id] = $tax->attribute_name . ': ' . $term->name;
						}
					}
				}
			}
		}

		return $result;
	}

	public function get_condition_input_type() {
		return 'Chosen_Select';
	}

	public function sort_attribute_taxonomies( $taxa, $taxb ) {
		return strcmp( $taxa->attribute_name, $taxb->attribute_name );
	}

	public function is_match( $rule_data, $arguments = null ) {
		$product = wc_get_product( get_the_ID() );
		$result = false;

		if ( $product && isset( $rule_data['condition'] ) && isset( $rule_data['operator'] ) ) {

			foreach ( $rule_data['condition'] as $condition ) {

				$term_data = explode( '|', $condition );

				$attribute_taxonomy_name = $term_data[0];
				$term_id = $term_data[1];

				$post_terms = wp_get_post_terms( $product->get_id(), $attribute_taxonomy_name, array('fields' => 'ids') );
				$in = in_array( $term_id, $post_terms );
				$result = $rule_data['operator'] == 'in' ? $in : !$in;
			}
		}

		return $this->return_is_match( $result, $rule_data, $arguments );
	}

}

class Dynamicblock_Conditional_Content_Rule_Product_Tag extends Dynamicblock_Conditional_Content_Rule_Base {

	public function __construct() {
		parent::__construct( 'product_tag' );
	}

	public function get_possible_rule_operators() {
		$operators = array(
			'in' => __( "in", 'DYNAMICBLOCK_TEXT' ),
			'notin' => __( "not in", 'DYNAMICBLOCK_TEXT' ),
		);

		return $operators;
	}

	public function get_possible_rule_values() {
		$result = array();

		$terms = DynamicBlock_Admin::Dynamicblock_Conditional_Content_get_all_product_tags();
		if ( $terms && !is_wp_error( $terms ) ) {
			foreach ( $terms as $term ) {
				$result[$term->term_id] = $term->name;
			}
		}

		return $result;
	}

	public function get_condition_input_type() {
		return 'Chosen_Select';
	}

	public function is_match( $rule_data, $arguments = null ) {
		$product = wc_get_product( get_the_ID() );
		$result = false;

		if ( $product && isset( $rule_data['condition'] ) && isset( $rule_data['operator'] ) ) {
			$terms = $product->get_tag_ids();
			$in = count( array_intersect( $terms, $rule_data['condition'] ) ) > 0;
			$result = $rule_data['operator'] == 'in' ? $in : !$in;
		}

		return $this->return_is_match( $result, $rule_data, $arguments );
	}

}

class Dynamicblock_Conditional_Content_Rule_Product_Price extends Dynamicblock_Conditional_Content_Rule_Base {

	public function __construct() {
		parent::__construct( 'product_price' );
	}

	public function get_possible_rule_operators() {
		$operators = array(
		    '==' => __( "is equal to", 'DYNAMICBLOCK_TEXT' ),
		    '!=' => __( "is not equal to", 'DYNAMICBLOCK_TEXT' ),
		    '>' => __( "is greater than", 'DYNAMICBLOCK_TEXT' ),
		    '<' => __( "is less than", 'DYNAMICBLOCK_TEXT' ),
		    '>=' => __( "is greater or equal to", 'DYNAMICBLOCK_TEXT' ),
		    '<=' => __( "is less or equal to", 'DYNAMICBLOCK_TEXT' )
		);
		return $operators;
	}

	public function get_condition_input_type() {
		return 'Price_Select';
	}

	public function is_match( $rule_data, $arguments = null ) {

		$result = false;
		$product = wc_get_product( get_the_ID() );
		if ( $product && isset( $rule_data['condition'] ) && isset( $rule_data['operator'] ) ) {
            
            $price = $product->get_price();
            $value = (float) htmlspecialchars_decode($rule_data['condition']);
            $rule_data['operator'] = htmlspecialchars_decode($rule_data['operator']);

			switch ( $rule_data['operator'] ) {
				case '==' :
					$result = $price == $value;
					break;
				case '!=' :
					$result = $price != $value;
					break;
				case '>' :
					$result = $price > $value;
					break;
				case '<' :
					$result = $price < $value;
					break;
				case '>=' :
					$result = $price >= $value;
					break;
				case '<=' :
					$result = $price <= $value;
					break;
				default:
					$result = false;
					break;
			}
		}

		return $this->return_is_match( $result, $rule_data, $arguments );
	}

}
