<?php
defined('ABSPATH') || exit;

class Dynamicblock_Conditional_Content_Rule_Sale_Status extends Dynamicblock_Conditional_Content_Rule_Base {

    public function __construct() {
        parent::__construct( 'sale_status' );
    }

    public function get_possible_rule_operators() {

        $operators = array(
            '==' => __( "is", 'Dynamicblock_Conditional_Content' ),
            '!=' => __( "is not", 'Dynamicblock_Conditional_Content' ),
        );

        return $operators;
    }

    public function get_possible_rule_values() {
        $options = array(
            '0' => __( 'Not On Sale', 'Dynamicblock_Conditional_Content' ),
            '1' => __( 'On Sale', 'Dynamicblock_Conditional_Content' )
        );

        return $options;
    }

    public function get_condition_input_type() {
        return 'Select';
    }

    public function is_match( $rule_data, $arguments = null ) {
        global $post;
        $result = false;
        $product = wc_get_product( get_the_ID() );
        $rule_data['operator'] = htmlspecialchars_decode($rule_data['operator']);
        if ( $product && isset( $rule_data['condition'] ) && isset( $rule_data['operator'] ) ) {
            $in = $product->is_on_sale();
            if ( $rule_data['operator'] == '==' ) {
                $result = $rule_data['condition'] == 1 ? $in : !$in;
            }

            if ( $rule_data['operator'] == '!=' ) {
                $result = !($rule_data['condition'] == 1 ? $in : !$in);
            }
        }

        return $this->return_is_match( $result, $rule_data, $arguments );
    }

}

class Dynamicblock_Conditional_Content_Rule_Sale_Date extends Dynamicblock_Conditional_Content_Rule_Base {

    public function __construct() {
        parent::__construct( 'sale_date' );
    }

    public function get_possible_rule_operators() {
        $operators = array(
            '<=' => __( "starts", 'Dynamicblock_Conditional_Content' ),
            '>=' => __( "ends", 'Dynamicblock_Conditional_Content' )
        );
        return $operators;
    }

    public function get_condition_input_type() {
        return 'Date';
    }

    public function is_match( $rule_data, $arguments = null ) {
        global $post;
        $product = wc_get_product( get_the_ID() );
        $result = false;
        if ( $product && isset( $rule_data['condition'] ) && isset( $rule_data['operator'] ) ) {

            $start_date = get_post_meta( $product->get_id(), '_sale_price_dates_from', true );
            $start_date =  ($start_date) ? strtotime(date("Y-m-d",$start_date)) :'';
            $end_date = get_post_meta( $product->get_id(), '_sale_price_dates_to', true );
            $end_date = ($end_date) ? strtotime(date("Y-m-d",$end_date)) : '';
            $date = strtotime($rule_data['condition']);
            switch ( $rule_data['operator'] ) {
                case '<=' :
                    if ( $start_date ) {
                        $result = $date <= ( $start_date );
                    }
                    break;
                case '>=' :
                    if ( $end_date ) {
                        $result = $date >= ( $end_date );
                    }
                    break;
                default:
                    $result = false;
                    break;
            }
        }

        return $this->return_is_match( $result, $rule_data, $arguments );
    }

}
