<?php
defined('ABSPATH') || exit;

class Dynamicblock_Conditional_Content_Rule_Schedule_Date extends Dynamicblock_Conditional_Content_Rule_Base {

	public function __construct() {
		parent::__construct( 'schedule_date' );
	}

	public function get_possible_rule_operators() {
		$operators = array(
			'>=' => __( "starts", 'DYNAMICBLOCK_TEXT' ),
			'<=' => __( "ends", 'DYNAMICBLOCK_TEXT' )
		);

		return $operators;
	}

	public function get_condition_input_type() {
		return 'Date';
	}

	public function is_match( $rule_data, $arguments = null ) {
		$result  = false;
		if ( isset( $rule_data['condition'] ) && isset( $rule_data['operator'] ) ) {

			$date = strtotime( $rule_data['condition'] );

			switch ( $rule_data['operator'] ) {
				case '>=' :
					$result = strtotime( date( 'Y-m-d' ) ) >= $date;
					break;
				case '<=' :
					$result = strtotime( date( 'Y-m-d' ) ) <= $date;
					break;
				default:
					$result = false;
					break;
			}
		}

		return $this->return_is_match( $result, $rule_data, $arguments );
	}

}


class Dynamicblock_Conditional_Content_Rule_Schedule_Day extends Dynamicblock_Conditional_Content_Rule_Base {

	public function __construct() {
		parent::__construct( 'schedule_day' );
	}

	public function get_possible_rule_operators() {
		$operators = array(
			'==' => __( "is", 'DYNAMICBLOCK_TEXT' ),
			'!=' => __( "is not", 'DYNAMICBLOCK_TEXT' )
		);

		return $operators;
	}

	public function get_possible_rule_values() {

		$options = array(
			'Sunday' => __( 'Sunday', 'DYNAMICBLOCK_TEXT' ),
			'Monday' => __( 'Monday', 'DYNAMICBLOCK_TEXT' ),
			'Tuesday' => __( 'Tuesday', 'DYNAMICBLOCK_TEXT' ),
			'Wednesday' => __( 'Wednesday', 'DYNAMICBLOCK_TEXT' ),
			'Thursday' => __( 'Thursday', 'DYNAMICBLOCK_TEXT' ),
			'Friday' => __( 'Friday', 'DYNAMICBLOCK_TEXT' ),
			'Saturday' => __( 'Saturday', 'DYNAMICBLOCK_TEXT' )
		);

		return $options;
	}


	public function get_condition_input_type() {
		return 'Chosen_Select';
	}

	public function is_match( $rule_data, $arguments = null ) {
		$result = false;
		if ( isset( $rule_data['condition'] ) && isset( $rule_data['operator'] ) ) {

            $date = array();
            foreach ($rule_data['condition'] as $day){
                $date[] = $day;
            }
            switch ( $rule_data['operator'] ) {
                case '==' :
                    $result = in_array(date( 'l' ),$date);
                    break;
                case '!=' :
                    $result = !in_array(date( 'l' ),$date);
                    break;
                default:
                    $result = false;
                    break;
            }
		}

		return $this->return_is_match( $result, $rule_data, $arguments );
	}

}


class Dynamicblock_Conditional_Content_Rule_Schedule_Time extends Dynamicblock_Conditional_Content_Rule_Base {

	public function __construct() {
		parent::__construct( 'schedule_time' );
	}

	public function get_possible_rule_operators() {
		$operators = array(
			'==' => __( "is equal to", 'DYNAMICBLOCK_TEXT' ),
			'!=' => __( "is not equal to", 'DYNAMICBLOCK_TEXT' ),
			'>'  => __( "is greater than", 'DYNAMICBLOCK_TEXT' ),
			'<'  => __( "is less than", 'DYNAMICBLOCK_TEXT' ),
			'>=' => __( "is greater or equal to", 'DYNAMICBLOCK_TEXT' ),
			'<=' => __( "is less or equal to", 'DYNAMICBLOCK_TEXT' )
		);

		return $operators;
	}

	public function get_condition_input_type() {
		return 'Time_Picker';
	}

	public function is_match( $rule_data, $arguments = null ) {
		$result = false;
		if ( isset( $rule_data['condition'] ) && isset( $rule_data['operator'] ) ) {
			$time = strtotime(date("h:i",strtotime( $rule_data['condition'] )));
			$now  = strtotime( date( "h:i" ) );
			switch ( $rule_data['operator'] ) {
				case '==' :
					$result = $time == $now;
					break;
				case '!=' :
					$result = $time != $now;
					break;
				case '>' :
					$result = $time > $now;
					break;
				case '<' :
					$result = $now < $time;
					break;
				case '>=' :
					$result = $time >= $now;
					break;
				case '<=' :
					$result = $now <= $time;
					break;
				default:
					$result = false;
					break;
			}
		}

		return $this->return_is_match( $result, $rule_data, $arguments );
	}

}