<?php
defined('ABSPATH') || exit;

class Dynamicblock_Conditional_Content_Rule_Stock_Status extends Dynamicblock_Conditional_Content_Rule_Base {

	public function __construct() {
		parent::__construct( 'stock_status' );
	}

	public function get_possible_rule_operators() {
		$operators = array(
		    '==' => __( "is", 'DYNAMICBLOCK_TEXT' ),
		    '!=' => __( "is not", 'DYNAMICBLOCK_TEXT' ),
		);
		return $operators;
	}

	public function get_possible_rule_values() {
		$options = array(
		    '0' => __( 'Out of Stock', 'DYNAMICBLOCK_TEXT' ),
		    '1' => __( 'In Stock', 'DYNAMICBLOCK_TEXT' )
		);

		return $options;
	}

	public function get_condition_input_type() {
		return 'Select';
	}

	public function is_match( $rule_data, $arguments = null ) {

		$result = false;
		$product = wc_get_product( get_the_ID() );
		$rule_data['operator'] = htmlspecialchars_decode($rule_data['operator']);
		if ( $product && isset( $rule_data['condition'] ) && isset( $rule_data['operator'] ) ) {
			$in = $product->is_in_stock();
			if ( $rule_data['operator'] == '==' ) {
				$result = $rule_data['condition'] == 1 ? $in : !$in;
			}

			if ( $rule_data['operator'] == '!=' ) {
				$result = !($rule_data['condition'] == 1 ? $in : !$in);
			}
		}

		return $this->return_is_match( $result, $rule_data, $arguments );
	}

}

class Dynamicblock_Conditional_Content_Rule_Stock_Level extends Dynamicblock_Conditional_Content_Rule_Base {

	public function __construct() {
		parent::__construct( 'stock_level' );
	}

	public function get_possible_rule_operators() {
		$operators = array(
		    '==' => __( "is equal to", 'DYNAMICBLOCK_TEXT' ),
		    '!=' => __( "is not equal to", 'DYNAMICBLOCK_TEXT' ),
		    '>' => __( "is greater than", 'DYNAMICBLOCK_TEXT' ),
		    '<' => __( "is less than", 'DYNAMICBLOCK_TEXT' ),
		    '>=' => __( "is greater or equal to", 'DYNAMICBLOCK_TEXT' ),
		    '<=' => __( "is less or equal to", 'DYNAMICBLOCK_TEXT' )
		);
		return $operators;
	}

	public function get_condition_input_type() {
		return 'Text';
	}

	public function is_match( $rule_data, $arguments = null ) {
		global $post;
		$result = false;
		$product = wc_get_product( get_the_ID() );
		if ( $product && isset( $rule_data['condition'] ) && isset( $rule_data['operator'] ) ) {
			$stock = $product->get_stock_quantity();
			$value = (float) $rule_data['condition'];
			$rule_data['operator'] = htmlspecialchars_decode($rule_data['operator']);
			switch ( $rule_data['operator'] ) {
				case '==' :
					$result = $stock == $value;
					break;
				case '!=' :
					$result = $stock != $value;
					break;
				case '>' :
					$result = $stock > $value;
					break;
				case '<' :
					$result = $stock < $value;
					break;
				case '>=' :
					$result = $stock >= $value;
					break;
				case '<=' :
					$result = $stock <= $value;
					break;
				default:
					$result = false;
					break;
			}
		}

		return $this->return_is_match( $result, $rule_data, $arguments );
	}

}
