<?php
defined('ABSPATH') || exit;

class Dynamicblock_Conditional_Content_Rule_Store_Order_Count extends Dynamicblock_Conditional_Content_Rule_Base {

	public function __construct() {
		parent::__construct( 'store_order_count' );
	}

	public function get_possible_rule_operators() {
		$operators = array(
			'==' => __( "is equal to", 'Dynamicblock_Conditional_Content' ),
			'!=' => __( "is not equal to", 'Dynamicblock_Conditional_Content' ),
			'>' => __( "is greater than", 'Dynamicblock_Conditional_Content' ),
			'<' => __( "is less than", 'Dynamicblock_Conditional_Content' ),
			'>=' => __( "is greater or equal to", 'Dynamicblock_Conditional_Content' ),
			'<=' => __( "is less or equal to", 'Dynamicblock_Conditional_Content' )
		);
		return $operators;
	}

	public function get_condition_input_type() {
		return 'Store_Order_Count';
	}

	public function is_match( $rule_data, $arguments = null ) {
		$result = false;
		if ( isset( $rule_data['condition'] ) && isset( $rule_data['operator'] ) ) {
			$value = $rule_data['condition']['qty'];
			
			$count = $this->get_order_count();
			$rule_data['operator'] = htmlspecialchars_decode($rule_data['operator']);
			switch ( $rule_data['operator'] ) {
				case '==' :
					$result = $count == $value;
					break;
				case '!=' :
					$result = $count != $value;
					break;
				case '>' :
					$result = $count > $value;
					break;
				case '<' :
					$result = $count < $value;
					break;
				case '>=' :
					$result = $count >= $value;
					break;
				case '<=' :
					$result = $count <= $value;
					break;
				default:
					$result = false;
					break;
			}
		}

		return $this->return_is_match( $result, $rule_data, $arguments );
	}

	private function get_order_count() {
		return count(wc_get_orders( array(
			'return' => 'ids',
			'limit' => -1,
		)));
	}

}

class Dynamicblock_Conditional_Content_Rule_Store_Order_Status extends Dynamicblock_Conditional_Content_Rule_Base {

	public function __construct() {
		parent::__construct( 'store_order_status' );
	}

    public function get_possible_rule_operators() {
        $operators = array(
            '==' => __( "is", 'DYNAMICBLOCK_TEXT' ),
            '!=' => __( "is not", 'DYNAMICBLOCK_TEXT' ),
        );
        return $operators;
    }

	public function get_condition_input_type() {
		return 'Order_Status';
	}

    public function is_match( $rule_data, $arguments = null ) {

        $result = false;
        $rule_data['operator'] = htmlspecialchars_decode($rule_data['operator']);
        $product = wc_get_product( get_the_ID() );
        $product_id = get_the_ID();

        $query = new WC_Order_Query();
        $orders = $query->get_orders();
        $post_in = array();
        foreach ($orders as $single_order) {
            $order_data = $single_order->get_data();
            $status = explode('wc-', $rule_data['condition']['status']);
            if ($order_data['status'] == $status[1]) {
                foreach ($single_order->get_items() as $item_key => $item_values):
                    $post_in[] = $item_values->get_product_id();
                endforeach;
            }
        }

        if(in_array($product_id, $post_in) && $rule_data['operator'] == '=='){
            $result = true;
        }else if(! in_array( $product_id,$post_in)  && $rule_data['operator'] == '!=' ){
            $result = true;
        }
        return $this->return_is_match( $result, $rule_data, $arguments );
    }

}