<?php
/**
 * Plugin Name: Dynamic Blocks - Supercharge Sales & Engage Your Audience
 *
 * Plugin URI: https://www.dynamicblock.ai/
 * Description: Dynamic Blocks for WordPress, WooCommerce & Gutenberg is the comprehensive solution that allows you to expand sales, increase revenue, and capture visitors' attention, encouraging them to make more substantial purchases and spend more.
 *
 * Version: 1.1.5
 * Author: Gotour
 * Author URI: https://www.dynamicblock.ai/
 * Text Domain: dynamicblock-builder
 */

defined('ABSPATH') || exit;

// Define MS_PLUGIN_FILE.
if (! defined("DYNAMICBLOCK_PLUGIN_FILE")) {
    define("DYNAMICBLOCK_PLUGIN_FILE", __FILE__);
}

define("DYNAMICBLOCK_PLUGIN_URL", plugin_dir_url(__FILE__));
define("DYNAMICBLOCK_VERSION", "1.1.5");
define("DYNAMICBLOCK_TEMPLATE_PATH", untrailingslashit(plugin_dir_path(__FILE__)) . "/templates/");
define("DYNAMICBLOCK_PLUGIN_PATH",
    untrailingslashit(plugins_url(basename(plugin_dir_path(__FILE__)), basename(__FILE__))));
define("DYNAMICBLOCK_MAIN_FILE", __FILE__);
define("DYNAMICBLOCK_ABSPATH", dirname(__FILE__) . "/");
define("DYNAMICBLOCK_TEXT", "dynamicblock-builder");
define("DYNAMICBLOCK_MAIN_FILE_NAME", basename(__FILE__));
define("DYNAMICBLOCK_MAIN_DIR_NAME", basename(dirname(__FILE__)));

// Is localhost
$whitelist   = array('127.0.0.1', '::1');
$isLocalhost = isset($_SERVER['REMOTE_ADDR']) && in_array($_SERVER['REMOTE_ADDR'], $whitelist);
if (! defined('IS_LOCALHOST')) {
    define("IS_LOCALHOST", $isLocalhost);
}

// Init
add_action('plugins_loaded', function () {
    global $dynamicblock_user_is_pro;
    $dynamicblock_user_is_pro = false;

    // Include the main WooCommerce class.
    if (! class_exists('DynamicBlock_Main', false)) {
        if (in_array('yith-woocommerce-brands-add-on/init.php',
            apply_filters('active_plugins', get_option('active_plugins')))) {
            define("DYNAMICBLOCK_YITH_WOOCOMMERCE_PRODUCT_BRAND", "yith_product_brand");
        } else if (in_array('yith-woocommerce-brands-add-on-premium/init.php',
            apply_filters('active_plugins', get_option('active_plugins')))) {
            define("DYNAMICBLOCK_YITH_WOOCOMMERCE_PRODUCT_BRAND", "product_brand");
        }

        if(!defined('DYNAMICBLOCK_YITH_WOOCOMMERCE_PRODUCT_BRAND')){
            define("DYNAMICBLOCK_YITH_WOOCOMMERCE_PRODUCT_BRAND", "");
        }

        include_once DYNAMICBLOCK_ABSPATH . '/includes/class-dynamicblock.php';
        new DynamicBlock_Main;
    }

    class DynamicBlock_Plugin_Activator
    {
        public function __construct()
        {
            register_activation_hook(__FILE__, array($this, 'activate'));
        }

        public function activate()
        {
            if (in_array('woocommerce/woocommerce.php',
                apply_filters('active_plugins', get_option('active_plugins')))) {
            } else {
                deactivate_plugins(plugin_basename(__FILE__));
                wp_die(__('Please activate or add WooCommerce.', 'dynamicblock-builder'), 'Plugin dependency check',
                    array('back_link' => true));
            }
        }
    }

    new DynamicBlock_Plugin_Activator();
}, 100);

/*-----------------------------------------Gutenberg block register here--------------------------*/

// Register block category
add_filter( 'block_categories_all', 'filter_block_categories_when_post_provided', 10, 2 );
function filter_block_categories_when_post_provided( $block_categories, $editor_context ) {

    if ( ! empty( $editor_context->post ) ) {
        array_unshift(
            $block_categories,
            array(
                'slug'  => 'dynamic-blocks',
                'title' => __( 'Dynamic Blocks', DYNAMICBLOCK_TEXT ),
                'icon'  => 'products',
            )
        );
    }
    return $block_categories;
}


function create_block_dynamicblocks_block_init() {
    register_block_type( __DIR__ . '/build' );
}
add_action( 'init', 'create_block_dynamicblocks_block_init' );


// Create custom rest points
add_action('rest_api_init', function () {
    register_rest_route( 'dynamicblock/v1', 'display_rule_types',array(
                'methods'  => 'GET',
                'callback' => 'get_latest_posts_by_category',
                'permission_callback' => function() { return true; }
    ));

     // all_display_rule_types
    register_rest_route( 'dynamicblock/v1', 'all_display_rule_types',array(
        'methods'  => 'GET',
        'callback' => 'get_all_display_rule_types',
        'permission_callback' => function() { return true; }
    ));

      register_rest_route( 'dynamicblock/v1', 'all_when_to_display_rule',array(
        'methods'  => 'GET',
        'callback' => 'get_all_when_to_display_rule',
        'permission_callback' => function() { return true; }
    ));

    // Product Filter rest route
    register_rest_route( 'dynamicblock/v1', 'product-filter',array(
            'methods'  => 'POST',
            'callback' => 'db_product_filter_callback',
            'permission_callback' =>"__return_true"
    ));
});


function get_latest_posts_by_category()
{
    include DYNAMICBLOCK_ABSPATH . 'admin/class/class-dynamicblock-content-compatibility.php';
    if ( ! function_exists( 'get_editable_roles' ) ) {
        require_once ABSPATH . 'wp-admin/includes/user.php';
    }
    if(!class_exists('DynamicBlock_Display_Rules') || !class_exists('DynamicBlock_Admin')) return ;

    $display_condition = new DynamicBlock_Display_Rules();
    $DynamicBlock_Admin = new DynamicBlock_Admin();

    if(!isset($_GET['rule_type']) ||  !$_GET['rule_type']){
        return $display_condition->what_to_display_rule_types();
    }

    $response = array();
    $DynamicBlock_Admin =  $DynamicBlock_Admin->dynamicblock_woocommerce_conditional_content_get_rule_object($_GET['rule_type']);

    if (! empty($DynamicBlock_Admin))
    {
        $response["values"]               = $DynamicBlock_Admin->get_possible_rule_values();
        $response["operators"]            = $DynamicBlock_Admin->get_possible_rule_operators();
        $response["condition_input_type"] = $DynamicBlock_Admin->get_condition_input_type();

        if($_GET['rule_type'] == "products")
        {
            $productsData = array_column(dynamicblock_get_all_products(),  'title', 'id');
            $response["values"]  = $productsData;
        }else if($_GET['rule_type'] == "store_order_status"){
            if(function_exists("wc_get_order_statuses")){
                $sts = wc_get_order_statuses();
                $response["values"]  = $sts;
            }
        }
    }
    return $response;
}

function db_product_filter_callback()
{
    $attributes = json_decode(file_get_contents('php://input'), true);

    $single_fields = array(
        'Price_Select',
        'Text',
        'Store_Order_Count',
        'Order_Status',
        'Select',
    );

    $rule_and_condition =  json_decode($attributes['rule_and_condition']);
    $dataset = isset($attributes['dataset'])?  array_column( $attributes['dataset'], 'value' ) : array();

    $DynamicBlock_Admin = new DynamicBlock_Admin();

    $what_to_display_query_builder = array();


    if(!empty($rule_and_condition)){
        foreach ($rule_and_condition as $key => $main_each_rule_and_condition)
        {
            $transform_data = array();
            foreach ($main_each_rule_and_condition->data_filter as $key => $each_rule_and_condition)
            {
                $ruleVal =json_decode( $each_rule_and_condition->values, true );
                $each_rule_and_condition_values = array();

                if(is_array($ruleVal))
                {
                    foreach ( array_column($ruleVal , 'value') as $value ) {
                        $each_rule_and_condition_values[] = $value;
                    }
                }

                $Dynamic_block = $DynamicBlock_Admin->dynamicblock_woocommerce_conditional_content_get_rule_object($each_rule_and_condition->display_rule_types);

                $condition_input_type = (method_exists($Dynamic_block, "get_condition_input_type")) ? $Dynamic_block->get_condition_input_type() : "";

                if(!empty($condition_input_type && !empty($each_rule_and_condition_values)))
                {
                    $tagfields = ( in_array($condition_input_type, $single_fields) ? $each_rule_and_condition_values[0] : $each_rule_and_condition_values );
                }else{
                    $tagfields = array();
                }
                $transform_data[] = array(
                            'dataset' => $each_rule_and_condition->display_rule_types,
                            'condition' => $each_rule_and_condition->operators,
                            'tagfields' => $tagfields,
                            'child_rule_type' => ( ($each_rule_and_condition->child_rule_type == "0") ? 'and' : 'or')
                        );
            }
            $what_to_display_query_builder[] = array(
                'rules-expression' =>  ( ($main_each_rule_and_condition->rule_type == "0") ? 'and' : 'or'),
                'dynamicblock_query' => $transform_data,
            );
        }
    }

    $dynamicBlockArgs = array(
        'limit' => isset( $attributes['limit']) && !empty( $attributes['limit'] )  ? $attributes['limit'] : 5,
        'posts_per_page' =>  isset( $attributes['limit']) && !empty( $attributes['limit'] )  ? $attributes['limit'] : 5,
        'columns' => '4',
        'orderby' => isset($attributes['order_by']) ? $attributes['order_by'] : "date",
        'order' => isset($attributes['mode']) ? $attributes['mode'] : "asc",
        'dynamicblock_rules' => array(
            'block_info' => array(
                'post_note' => '',
                'post_block_type' => 'related-products',
                'block_title' => isset($attributes['block_title']) ? $attributes['block_title'] : "",
            ),
            'what_to_display' => array(
                'datasets' => array(
                    'products' => $dataset,
                    'order_by' => array(
                        isset($attributes['order_by']) ? $attributes['order_by'] : "date",
                    ),
                    'order' => isset($attributes['mode']) ? $attributes['mode'] : "asc",
                    'limit' => isset( $attributes['limit']) && !empty( $attributes['limit'] )  ? $attributes['limit'] : 5,
                ),
                'post_query_rules' => $what_to_display_query_builder,
            ),
            'when_to_display' => array(
                'post_query_rules' => array(
                    'dynamicblock_1684411251' => array(
                        'rules-expression' => 'and',
                        'dynamicblock_query' => array(
                            'dynamicblock_3368822502' => array(
                                'dataset' => 'show_always',
                                'condition' => '==',
                            ),
                        ),
                    ),
                ),
            ),
            'how_to_display' => array(
                'type-setting' => array(
                    'show_product_excerpt' => isset($attributes['excerptToggle'])? $attributes['excerptToggle'] : false,
                    'enable-carousel' => isset($attributes['carouselToggle']) ? $attributes['carouselToggle'] : false,
                    'display-add-to-cart-button' => isset($attributes['addToCartBtnToggle']) ? $attributes['addToCartBtnToggle'] : "",
                    'autoPlay' => (isset($attributes['autoplayToggle']) && $attributes['autoplayToggle'] == true) ? 'enable_autoplay' : 'no_autoplay',
                ),
            ),
            'settings' => array(
                'custom-css-class' => isset($attributes['gbcustomCssClass']) ? $attributes['gbcustomCssClass'] : "",
                'custom-css' =>  isset($attributes['gbcustomCss']) ? $attributes['gbcustomCss'] : "",
                'custom-js' => isset($attributes['gbcustomJs']) ? $attributes['gbcustomJs'] : "",
                'utm-source' => isset($attributes['gbutmSource']) ? $attributes['gbutmSource'] : "",
                'utm-source' => isset($attributes['gbutmSource']) ? $attributes['gbutmSource'] : "",
                'utm-medium' => isset($attributes['gbutmMedium']) ? $attributes['gbutmMedium'] : "",
                'utm-medium' => isset($attributes['gbutmMedium']) ? $attributes['gbutmMedium'] : "",
                'utm-campaign' => isset($attributes['gbutmCampaign']) ? $attributes['gbutmCampaign'] : "",
            ),
        ),
        'skin_type' => (isset($attributes['carouselToggle']) && $attributes['carouselToggle'] == true) ? 'carousel' : 'grid',
        'enable_excerpt' => (isset($attributes['excerptToggle']) && $attributes['excerptToggle'] == true) ? 'enable_excerpt' : 'no_excerpt',
        'enable_cart_button' => (isset($attributes['addToCartBtnToggle']) && $attributes['addToCartBtnToggle'] == true) ? 'enable_cart_button' : 'no_cart_button',
        'enable_autoplay' => (isset($attributes['autoplayToggle']) && $attributes['autoplayToggle'] == true) ? 'enable_autoplay' : 'no_autoplay',
        'enable_wishlist' => (isset($attributes['enable_wishlist']) && $attributes['enable_wishlist'] == true) ? 'enable_wishlist' : 'no_wishlist',
    );


    $last_query_product = DynamicBlock_Conditional_Content_Display:: dynamicblock_what_to_display_result_product($dynamicBlockArgs);


    $response = array();

    $args = array(
        'numberposts' =>  isset( $attributes['limit']) && !empty( $attributes['limit'] )  ? $attributes['limit'] : 5,
        'post_type'   => 'product',
        'post_status' => 'publish',
        'post__in' => $last_query_product ? $last_query_product : array(0),
    );

    if(isset($attributes['order_by']) && !empty($attributes['order_by'])){
        $args['orderby'] = $attributes['order_by'];
    }

    if(isset($attributes['mode']) && !empty($attributes['mode'])){
        $args['order'] = $attributes['mode'];
    }

    if(isset($attributes['limit']) && !empty($attributes['limit'])){
        $args['numberposts'] = $attributes['limit'];
    }

    $allProducts = get_posts( $args );

    if(count($allProducts) > 0)
    {
        foreach($allProducts as $prd)
        {
            $prodId = $prd->ID;

            $product = wc_get_product( $prodId );
            $product->get_price_html();

            $prdd = (array) $prd;
            $img_array = array( "feature_image" => get_the_post_thumbnail_url($prodId) );
            $prdd['feature_image'] = get_the_post_thumbnail_url($prodId) ;
            $prdd['price_html'] = json_encode($product->get_price_html());
            $response[] =  $prdd;
        }
    }
    return $response;
}


add_filter("woocommerce_product_related_products_heading",function( $heading ){
    $post = get_post();

    if ( has_blocks( $post->post_content ) ) {
        $blocks = parse_blocks( $post->post_content );
        if ( $blocks[0]['blockName'] === 'create-block/dynamicblocks' ) {

            if(isset($blocks[0]['attrs']['block_title'])){
                return  $blocks[0]['attrs']['block_title'];

            }
        }
    };
    return $heading;
});


function get_all_display_rule_types()
{
    require_once DYNAMICBLOCK_ABSPATH . 'admin/class/class-dynamicblock-content-compatibility.php';
    if ( ! function_exists( 'get_editable_roles' ) ) {
        require_once ABSPATH . 'wp-admin/includes/user.php';
    }
    if(!class_exists('DynamicBlock_Display_Rules') || !class_exists('DynamicBlock_Admin')) return ;

    $display_condition = new DynamicBlock_Display_Rules();

    $display_rule_types = $display_condition->what_to_display_rule_types();

    $DataResponse = array();
    foreach( $display_rule_types as $dataset => $datasetVals)
    {
        foreach( $datasetVals as $rulek => $rulev)
        {
            $DataResponse[$dataset][$rulek] = array($rulek => $rulev);
            $DynamicBlock_Admin = new DynamicBlock_Admin();
            $DynamicBlock_AdminRt =  $DynamicBlock_Admin->dynamicblock_woocommerce_conditional_content_get_rule_object($rulek);
             $response = array();
            if (! empty($DynamicBlock_AdminRt))
            {
                $response["values"]               = $DynamicBlock_AdminRt->get_possible_rule_values();
                $response["operators"]            = $DynamicBlock_AdminRt->get_possible_rule_operators();
                $response["condition_input_type"] = $DynamicBlock_AdminRt->get_condition_input_type();

                if($rulek == "products")
                {
                    $productsData = array_column(dynamicblock_get_all_products(),  'title', 'id');
                    $response["values"]  = $productsData;
                }else if($rulek == "store_order_status"){
                    if(function_exists("wc_get_order_statuses"))
                    {
                        $sts = wc_get_order_statuses();
                        $response["values"]  = $sts;
                    }
                }
                $DataResponse[$dataset][$rulek]['rule_types'] = $response;
            }
        }
    }
    return $DataResponse;
}

function get_all_when_to_display_rule()
{
    require_once DYNAMICBLOCK_ABSPATH . 'admin/class/class-dynamicblock-content-compatibility.php';
    require_once DYNAMICBLOCK_ABSPATH . 'admin/class/inputs/html-always.php';
    if ( ! function_exists( 'get_editable_roles' ) ) {
        require_once ABSPATH . 'wp-admin/includes/user.php';
    }
    if(!class_exists('DynamicBlock_Display_Rules') || !class_exists('DynamicBlock_Admin')) return ;

    $display_condition = new DynamicBlock_Display_Rules();

    $display_rule_types = $display_condition->when_to_display_rule_types();

    $permitedDataset = array("Global","Membership", "User Role","Date & Time","Browser & Device","Screen Size","URL Source");

    $DataResponse = array();
    foreach( $display_rule_types as $dataset => $datasetVals)
    {
        if(in_array($dataset, $permitedDataset) ) :
            foreach( $datasetVals as $rulek => $rulev)
            {
                $DataResponse[$dataset][$rulek] = array($rulek => $rulev);
                $DynamicBlock_Admin = new DynamicBlock_Admin();
                $DynamicBlock_AdminRt =  $DynamicBlock_Admin->dynamicblock_woocommerce_conditional_content_get_rule_object($rulek);
                $response = array();
                if (! empty($DynamicBlock_AdminRt))
                {
                    $response["values"]               = $DynamicBlock_AdminRt->get_possible_rule_values();
                    $response["operators"]            = $DynamicBlock_AdminRt->get_possible_rule_operators();
                    $response["condition_input_type"] = $DynamicBlock_AdminRt->get_condition_input_type();

                    if($rulek == "products")
                    {
                        $productsData = array_column(dynamicblock_get_all_products(),  'title', 'id');
                        $response["values"]  = $productsData;
                    }else if($rulek == "store_order_status"){
                        if(function_exists("wc_get_order_statuses"))
                        {
                            $sts = wc_get_order_statuses();
                            $response["values"]  = $sts;
                        }
                    }else if($rulek == "show_always"){

                        $_html_val = new DynamicBlock_Conditional_Content_Input_Html_Always();
                        ob_start();
                        echo $_html_val->render("");
                        $response["values"]  = ob_get_clean();
                    }
                    $DataResponse[$dataset][$rulek]['rule_types'] = $response;
                }
            }
        endif;
    }
    return $DataResponse;
}


// function for validate when to display conditions for butenbuerg blocks
function gb_get_when_to_display_result($when_post_query_rules, $current_block_type = '')
{
    global $all_over_last_result;
    $all_over_last_result = 0;
    $default_value        = 0;

    if (empty($when_post_query_rules)) {
        $when_post_query_rules[] = array(
            'rules-expression'   => 'or',
            'dynamicblock_query' => array(
                '0' => array(
                    'dataset'   => 'show_always',
                    'condition' => '==',
                ),
            ),
        );
    }

    $_array_result = array();
    $last_count    = 0;
    if (! empty($when_post_query_rules)) {
        foreach ($when_post_query_rules as $group_rule_key => $when_post_query_rule) {
            $group_condition = $when_post_query_rule['rules-expression'];
            if ($group_condition == 'or') {
                $last_count++;
            }
            if (! $default_value) {
                $group_condition = '';
            }
            if (isset($when_post_query_rule['dynamicblock_query'])) {
                $current_group_last_result = 'new';
                foreach ($when_post_query_rule['dynamicblock_query'] as $rules_key => $dynamicblock_query_single) {
                    if (isset($dynamicblock_query_single['dataset']) && isset($dynamicblock_query_single['condition']) && isset($dynamicblock_query_single['tagfields'])) {

                        $rule        = array(
                            'operator'  => $dynamicblock_query_single['condition'],
                            'condition' => $dynamicblock_query_single['tagfields'],
                            'rule_type' => $dynamicblock_query_single['dataset'],
                        );
                        $rule_object = dynamicblock_conditional_content_get_rule_object($dynamicblock_query_single['dataset']);



                        $result      = $rule_object->is_match($rule);
                        if ($current_group_last_result || ! $result) {
                            $current_group_last_result = $result;
                        }
                        if ($result == 1) {
                            $_when_display_result = true;
                        } else {
                            $result = 0;
                        }
                    } else {
                    }
                    if (isset($dynamicblock_query_single['dataset']) && $dynamicblock_query_single['dataset'] == 'show_always') {
                        $_when_display_result = true;
                        $result               = 1;
                    }
                    $_array_result[$last_count][] = $result;
                }
            }
            $default_value++;
        }
    }
    $last_group_result  = 'new';
    $result_array_count = count($_array_result);
    foreach ($_array_result as $key => $single_group_result) {
        $_result = '';
        if (in_array('0', $single_group_result)) {
            $_result = '0';
        } else {
            $_result = 'true';
        }
        if ($last_group_result == 'new') {
            $last_group_result = $_result;
        } else {
            if ($last_group_result || $result) {
                $last_group_result = 'true';
            } else {
                $last_group_result = '0';
            }
        }
    }
    return $last_group_result;

}
/*-----------------------------------------Gutenberg block register here--------------------------*/
