<?php

/**
 * DynamicBlock Main Class
 */
defined('ABSPATH') || exit;

class DynamicBlock_Main
{
    public function __construct()
    {
        $this->includes();
        $this->init_hooks();
    }

    public function init_hooks()
    {
        add_action('template_redirect', array($this, 'wc_track_product_view'), 20);
        add_filter('dynamicblock_conditional_content_get_locations', array(&$this, 'dynamicblock_default_locations'), 0);
        add_action('admin_enqueue_scripts', array($this, 'dynamicblock_admin_theme_style'));
        add_action('login_enqueue_scripts', array($this, 'dynamicblock_admin_theme_style'));
    }

    public function includes()
    {
        /**
         * Before Extra includes
         */
        do_action('dynamicblock_main_before_includes');

        include_once DYNAMICBLOCK_ABSPATH . 'admin/class/class-dynamicblock-post-type.php';
        include_once DYNAMICBLOCK_ABSPATH . 'admin/class/class-dynamicblock-admin.php';
        include_once DYNAMICBLOCK_ABSPATH . 'admin/class/display-rules/all_rules_array.php';
        include_once DYNAMICBLOCK_ABSPATH . 'public/class/class-dynamicblock-public.php';
        include_once DYNAMICBLOCK_ABSPATH . 'admin/class/display-rules/base.php';
        include_once DYNAMICBLOCK_ABSPATH . 'admin/class/display-rules/products.php';
        include_once DYNAMICBLOCK_ABSPATH . 'admin/class/display-rules/schedule.php';
        include_once DYNAMICBLOCK_ABSPATH . 'admin/class/display-rules/stock.php';
        include_once DYNAMICBLOCK_ABSPATH . 'admin/class/display-rules/store.php';
        include_once DYNAMICBLOCK_ABSPATH . 'admin/class/display-rules/sales.php';
        include_once DYNAMICBLOCK_ABSPATH . 'admin/class/display-rules/membership.php';
        include_once DYNAMICBLOCK_ABSPATH . 'admin/class/display-rules/product_meta.php';
        include_once DYNAMICBLOCK_ABSPATH . 'admin/class/display-rules/cart.php';
        include_once DYNAMICBLOCK_ABSPATH . 'admin/class/display-rules/global.php';
        include_once DYNAMICBLOCK_ABSPATH . 'admin/class/display-rules/checkout.php';
        include_once DYNAMICBLOCK_ABSPATH . 'admin/class/display-rules/yith_brands_add-on.php';
        include_once DYNAMICBLOCK_ABSPATH . 'admin/class/display-rules/browser_and_device.php';
        include_once DYNAMICBLOCK_ABSPATH . 'admin/class/display-rules/screen_size.php';
        include_once DYNAMICBLOCK_ABSPATH . 'admin/class/display-rules/metadata.php';
        include_once DYNAMICBLOCK_ABSPATH . 'admin/class/display-rules/date_and_time.php';
        include_once DYNAMICBLOCK_ABSPATH . 'admin/class/display-rules/url_source.php';

        include_once DYNAMICBLOCK_ABSPATH . '/includes/class-related-product.php';

        /**
         * After Extra includes
         */
        do_action('dynamicblock_main_after_includes');

        if (is_admin() || defined('DOING_AJAX')) {

            // Include the admin interface builder
            include DYNAMICBLOCK_ABSPATH . 'admin/class/display-rules/rules_input_builder.php';
            include DYNAMICBLOCK_ABSPATH . 'admin/class/class-dynamicblock-content-compatibility.php';
            // Admin Inputs
            include DYNAMICBLOCK_ABSPATH . 'admin/class/inputs/html-always.php';
            include DYNAMICBLOCK_ABSPATH . 'admin/class/inputs/text.php';
            include DYNAMICBLOCK_ABSPATH . 'admin/class/inputs/date.php';
            include DYNAMICBLOCK_ABSPATH . 'admin/class/inputs/select.php';
            include DYNAMICBLOCK_ABSPATH . 'admin/class/inputs/product-select.php';
            include DYNAMICBLOCK_ABSPATH . 'admin/class/inputs/cart-product-select.php';
            include DYNAMICBLOCK_ABSPATH . 'admin/class/inputs/cart-category-select.php';
            include DYNAMICBLOCK_ABSPATH . 'admin/class/inputs/product-meta-select.php';
            include DYNAMICBLOCK_ABSPATH . 'admin/class/inputs/price-select.php';
            include DYNAMICBLOCK_ABSPATH . 'admin/class/inputs/chosen-select.php';
            include DYNAMICBLOCK_ABSPATH . 'admin/class/inputs/order-status.php';
            include DYNAMICBLOCK_ABSPATH . 'admin/class/inputs/store_order_count.php';
            include DYNAMICBLOCK_ABSPATH . 'admin/class/inputs/time_picker.php';
            include DYNAMICBLOCK_ABSPATH . 'admin/class/inputs/checkout-product-select.php';
            include DYNAMICBLOCK_ABSPATH . 'admin/class/inputs/text_area.php';
            include DYNAMICBLOCK_ABSPATH . 'admin/class/inputs/text_area_default.php';
            include DYNAMICBLOCK_ABSPATH . 'admin/class/inputs/number.php';
            include DYNAMICBLOCK_ABSPATH . 'admin/class/inputs/payment_method_select.php';
            include DYNAMICBLOCK_ABSPATH . 'admin/class/inputs/checkout-category-select.php';
            include DYNAMICBLOCK_ABSPATH . 'admin/class/inputs/schedule_date_time.php';
        }
    }

    public function dynamicblock_admin_theme_style()
    {
        if (isset($_GET['post_type']) && sanitize_text_field($_GET['post_type']) == 'dynamicblock-block' || isset($_GET['page']) && sanitize_text_field($_GET['page']) == 'global_setting' || isset($_GET['page']) && sanitize_text_field($_GET['page']) == 'dynamicblock-dashboard' || isset($_GET['post']) && get_post_type(sanitize_text_field($_GET['post'])) == 'dynamicblock-block') {
            ?>
            <style>.update-nag, .updated, .error, .is-dismissible {
                display: none;
              }</style>
            <?php
        }
    }

    public static function add_action($action, $method, $priority = 10)
    {
        add_action($action, $method, $priority);
    }

    /**
     * Get other templates (e.g. product attributes) passing attributes and including the file.
     *
     * @param string $template_name Template name.
     * @param array  $args          Arguments. (default: array).
     * @param string $template_path Template path. (default: '').
     * @param string $default_path  Default path. (default: '').
     */
    static function dynamicblock_get_template($template_name, $args = array(), $template_path = '', $default_path = '')
    {
        $template = self::dynamicblock_locate_template($template_name, $template_path, $default_path);
        // Allow 3rd party plugin filter template file from their plugin.

        $filter_template = apply_filters('wc_get_template', $template, $template_name, $args, $template_path,
            $default_path);
        if ($filter_template !== $template) {
            if (! file_exists($filter_template)) {
                /* translators: %s template */
                wc_doing_it_wrong(__FUNCTION__,
                    sprintf(__('%s does not exist.', 'woocommerce'), '<code>' . $filter_template . '</code>'), '2.1');

                return;
            }
            $template = $filter_template;
        }

        $action_args = array(
            'template_name' => $template_name,
            'template_path' => $template_path,
            'located'       => $template,
            'args'          => $args,
        );

        if (! empty($args) && is_array($args)) {
            if (isset($args['action_args'])) {
                wc_doing_it_wrong(
                    __FUNCTION__,
                    __('action_args should not be overwritten when calling wc_get_template.', 'woocommerce'),
                    '3.6.0'
                );
                unset($args['action_args']);
            }
            extract($args); // @codingStandardsIgnoreLine
        }
        do_action('dynamicblock_before_template_part', $action_args['template_name'], $action_args['template_path'],
            $action_args['located'], $action_args['args']);
        include $action_args['located'];
        do_action('dynamicblock_after_template_part', $action_args['template_name'], $action_args['template_path'],
            $action_args['located'], $action_args['args']);
    }

    /**
     * Locate a template and return the path for inclusion.
     *
     * This is the load order:
     *
     * yourtheme/$template_path/$template_name
     * yourtheme/$template_name
     * $default_path/$template_name
     *
     * @param string $template_name Template name.
     * @param string $template_path Template path. (default: '').
     *
     * @return string
     */

    static function dynamicblock_locate_template($template_name, $template_path = '')
    {
        if (! $template_path) {
            $template_path = DYNAMICBLOCK_TEMPLATE_PATH;
        }

        if (! isset($default_path)) {
            $default_path = DYNAMICBLOCK_TEMPLATE_PATH;
        }

        // Look within passed path within the theme - this is priority.
        if (false !== strpos($template_name, 'product_cat') || false !== strpos($template_name, 'product_tag')) {
            $cs_template = str_replace('_', '-', $template_name);
            $template    = locate_template(
                array(
                    trailingslashit($template_path) . $cs_template,
                    $cs_template,
                )
            );
        }

        if (empty($template)) {
            $template = locate_template(
                array(
                    trailingslashit($template_path) . $template_name,
                    $template_name,
                )
            );
        }

        // Get default template/.
        if (! $template) {
            if (empty($cs_template)) {
                $template = $default_path . $template_name;
            } else {
                $template = $default_path . $cs_template;
            }
        }

        // Return what we found.
        return apply_filters('dynamicblock_locate_template', $template, $template_name, $template_path);
    }

    /**
     * @param array $locations The current list, if any, of configured action hooks and filters.
     *
     * @return array
     */
    public static function dynamicblock_default_locations($locations = array())
    {
        $locations = array(
            'woocommerce' => array(
                'title'       => __('WooCommerce', DYNAMICBLOCK_TEXT),
                'description' => __('The single product page', DYNAMICBLOCK_TEXT),
                'hooks'       => array(
                    'woocommerce_before_main_content' => array(
                        'action'      => 'woocommerce_before_main_content',
                        'priority'    => 0,
                        'title'       => __('Before main content', DYNAMICBLOCK_TEXT),
                        'description' => '',
                    ),
                    'woocommerce_after_main_content'  => array(
                        'action'      => 'woocommerce_after_main_content',
                        'priority'    => 0,
                        'title'       => __('After main content', DYNAMICBLOCK_TEXT),
                        'description' => '',
                    ),
                ),
            ),

            'wc_product_page'        => array(
                'title'       => __('Single Product', DYNAMICBLOCK_TEXT),
                'description' => __('The single product page', DYNAMICBLOCK_TEXT),
                'hooks'       => array(
                    'woocommerce_before_single_product'         => array(
                        'action'      => 'woocommerce_before_single_product',
                        'priority'    => 10,
                        'title'       => __('Before Single product', DYNAMICBLOCK_TEXT),
                        'description' => '',
                    ),
                    'woocommerce_before_single_product_summary' => array(
                        'action'      => 'woocommerce_before_single_product_summary',
                        'priority'    => 10,
                        'title'       => __('Before Single product summary', DYNAMICBLOCK_TEXT),
                        'description' => '',
                    ),
                    'woocommerce_single_product_summary'        => array(
                        'action'      => 'woocommerce_single_product_summary',
                        'priority'    => 10,
                        'title'       => __('Single product summary', DYNAMICBLOCK_TEXT),
                        'description' => '',
                    ),
                    'woocommerce_before_add_to_cart_form'       => array(
                        'action'      => 'woocommerce_before_add_to_cart_form',
                        'priority'    => 10,
                        'title'       => __('Before Add to cart form', DYNAMICBLOCK_TEXT),
                        'description' => '',
                    ),
                    'woocommerce_before_variations_form'        => array(
                        'action'      => 'woocommerce_before_variations_form',
                        'priority'    => 10,
                        'title'       => __('Before Variations form', DYNAMICBLOCK_TEXT),
                        'description' => '',
                    ),
                    'woocommerce_before_add_to_cart_button'     => array(
                        'action'      => 'woocommerce_before_add_to_cart_button',
                        'priority'    => 10,
                        'title'       => __('Before Add to cart button', DYNAMICBLOCK_TEXT),
                        'description' => '',
                    ),
                    'woocommerce_before_single_variation'       => array(
                        'action'      => 'woocommerce_before_single_variation',
                        'priority'    => 10,
                        'title'       => __('Before Single Variation', DYNAMICBLOCK_TEXT),
                        'description' => '',
                    ),
                    'woocommerce_before_add_to_cart_quantity'   => array(
                        'action'      => 'woocommerce_before_add_to_cart_quantity',
                        'priority'    => 10,
                        'title'       => __('Before Add to cart quantity', DYNAMICBLOCK_TEXT),
                        'description' => '',
                    ),
                    'woocommerce_after_add_to_cart_quantity'    => array(
                        'action'      => 'woocommerce_after_add_to_cart_quantity',
                        'priority'    => 10,
                        'title'       => __('After Add to cart quantity', DYNAMICBLOCK_TEXT),
                        'description' => '',
                    ),
                    'woocommerce_after_single_variation'        => array(
                        'action'      => 'woocommerce_after_single_variation',
                        'priority'    => 10,
                        'title'       => __('After Single Variation', DYNAMICBLOCK_TEXT),
                        'description' => '',
                    ),
                    'woocommerce_after_add_to_cart_button'      => array(
                        'action'      => 'woocommerce_after_add_to_cart_button',
                        'priority'    => 10,
                        'title'       => __('After add to cart button', DYNAMICBLOCK_TEXT),
                        'description' => '',
                    ),
                    'woocommerce_after_variations_form'         => array(
                        'action'      => 'woocommerce_after_variations_form',
                        'priority'    => 10,
                        'title'       => __('After Variation Form', DYNAMICBLOCK_TEXT),
                        'description' => '',
                    ),
                    'woocommerce_after_add_to_cart_form'        => array(
                        'action'      => 'woocommerce_after_add_to_cart_form',
                        'priority'    => 10,
                        'title'       => __('After add to cart form', DYNAMICBLOCK_TEXT),
                        'description' => '',
                    ),
// Duplicate
//                    'woocommerce_before_single_variation'       => array(
//                        'action'      => 'woocommerce_before_single_variation',
//                        'priority'    => 10,
//                        'title'       => __('Before Single Variation', DYNAMICBLOCK_TEXT),
//                        'description' => '',
//                    ),
//                    'woocommerce_before_single_variation'       => array(
//                        'action'      => 'woocommerce_before_single_variation',
//                        'priority'    => 10,
//                        'title'       => __('Before Single Variation', DYNAMICBLOCK_TEXT),
//                        'description' => '',
//                    ),
                    'woocommerce_after_single_product_summary'  => array(
                        'action'      => 'woocommerce_after_single_product_summary',
                        'priority'    => 10,
                        'title'       => __('After single product summary', DYNAMICBLOCK_TEXT),
                        'description' => '',
                    ),

                    'woocommerce_after_single_product' => array(
                        'action'      => 'woocommerce_after_single_product',
                        'priority'    => 10,
                        'title'       => __('After single product', DYNAMICBLOCK_TEXT),
                        'description' => '',
                    ),
                ),
            ),
            'wc_cart_page'           => array(
                'title'       => __('Cart Page', DYNAMICBLOCK_TEXT),
                'description' => __('The Cart page', DYNAMICBLOCK_TEXT),
                'hooks'       => array(
                    'woocommerce_before_cart'                    => array(
                        'action'      => 'woocommerce_before_cart',
                        'priority'    => 0,
                        'title'       => __('Before Cart content', DYNAMICBLOCK_TEXT),
                        'description' => '',
                    ),
                    'woocommerce_before_cart_contents'           => array(
                        'action'      => 'woocommerce_before_cart_contents',
                        'priority'    => 0,
                        'title'       => __('Before main content', DYNAMICBLOCK_TEXT),
                        'description' => '',
                    ),
                    'woocommerce_cart_coupon'                    => array(
                        'action'      => 'woocommerce_cart_coupon',
                        'priority'    => 0,
                        'title'       => __('Before cart coupon', DYNAMICBLOCK_TEXT),
                        'description' => '',
                    ),
                    'woocommerce_after_cart_contents'            => array(
                        'action'      => 'woocommerce_after_cart_contents',
                        'priority'    => 0,
                        'title'       => __('After cart content', DYNAMICBLOCK_TEXT),
                        'description' => '',
                    ),
                    'woocommerce_after_cart_table'               => array(
                        'action'      => 'woocommerce_after_cart_table',
                        'priority'    => 0,
                        'title'       => __('After cart table', DYNAMICBLOCK_TEXT),
                        'description' => '',
                    ),
                    'woocommerce_before_cart_totals'             => array(
                        'action'      => 'woocommerce_before_cart_totals',
                        'priority'    => 0,
                        'title'       => __('before cart totals', DYNAMICBLOCK_TEXT),
                        'description' => '',
                    ),
                    'woocommerce_cart_totals_before_shipping'    => array(
                        'action'      => 'woocommerce_cart_totals_before_shipping',
                        'priority'    => 0,
                        'title'       => __('Cart totals before shipping', DYNAMICBLOCK_TEXT),
                        'description' => '',
                    ),
                    'woocommerce_before_shipping_calculator'     => array(
                        'action'      => 'woocommerce_before_shipping_calculator',
                        'priority'    => 0,
                        'title'       => __('Before Shipping calculator', DYNAMICBLOCK_TEXT),
                        'description' => '',
                    ),
                    'woocommerce_after_shipping_calculator'      => array(
                        'action'      => 'woocommerce_after_shipping_calculator',
                        'priority'    => 0,
                        'title'       => __('After shipping calculator', DYNAMICBLOCK_TEXT),
                        'description' => '',
                    ),
                    'woocommerce_cart_totals_after_shipping'     => array(
                        'action'      => 'woocommerce_cart_totals_after_shipping',
                        'priority'    => 0,
                        'title'       => __('Cart totals after shipping', DYNAMICBLOCK_TEXT),
                        'description' => '',
                    ),
                    'woocommerce_cart_totals_before_order_total' => array(
                        'action'      => 'woocommerce_cart_totals_before_order_total',
                        'priority'    => 0,
                        'title'       => __('Cart before order total', DYNAMICBLOCK_TEXT),
                        'description' => '',
                    ),
                    'woocommerce_after_cart_totals'              => array(
                        'action'      => 'woocommerce_after_cart_totals',
                        'priority'    => 0,
                        'title'       => __('After cart totals', DYNAMICBLOCK_TEXT),
                        'description' => '',
                    ),
                    'woocommerce_after_cart'                     => array(
                        'action'      => 'woocommerce_after_cart',
                        'priority'    => 0,
                        'title'       => __('After cart content', DYNAMICBLOCK_TEXT),
                        'description' => '',
                    ),
                ),
            ),
            'wc_shop_page'           => array(
                'title'       => __('Shop/Archives Page', DYNAMICBLOCK_TEXT),
                'description' => __('The Shop/Archives page', DYNAMICBLOCK_TEXT),
                'hooks'       => array(
                    'woocommerce_before_shop_loop'            => array(
                        'action'      => 'woocommerce_before_shop_loop',
                        'priority'    => 0,
                        'title'       => __('Before Shop loop', DYNAMICBLOCK_TEXT),
                        'description' => '',
                    ),
                    'woocommerce_before_shop_loop_item'       => array(
                        'action'      => 'woocommerce_before_shop_loop_item',
                        'priority'    => 0,
                        'title'       => __('Before Shop loop item', DYNAMICBLOCK_TEXT),
                        'description' => '',
                    ),
                    'woocommerce_before_shop_loop_item_title' => array(
                        'action'      => 'woocommerce_before_shop_loop_item_title',
                        'priority'    => 0,
                        'title'       => __('Before Shop loop item title', DYNAMICBLOCK_TEXT),
                        'description' => '',
                    ),
                    'woocommerce_after_shop_loop_item_title'  => array(
                        'action'      => 'woocommerce_after_shop_loop_item_title',
                        'priority'    => 0,
                        'title'       => __('After shop loop item title', DYNAMICBLOCK_TEXT),
                        'description' => '',
                    ),
                    'woocommerce_after_shop_loop_item'        => array(
                        'action'      => 'woocommerce_after_shop_loop_item',
                        'priority'    => 0,
                        'title'       => __('After shop loop item', DYNAMICBLOCK_TEXT),
                        'description' => '',
                    ),
                    'woocommerce_after_shop_loop'             => array(
                        'action'      => 'woocommerce_after_shop_loop',
                        'priority'    => 0,
                        'title'       => __('After shop loop', DYNAMICBLOCK_TEXT),
                        'description' => '',
                    ),
                ),
            ),
            'wc_checkout_page'       => array(
                'title'       => __('Checkout Page', DYNAMICBLOCK_TEXT),
                'description' => __('The Checkout page', DYNAMICBLOCK_TEXT),
                'hooks'       => array(
                    'woocommerce_before_checkout_form'              => array(
                        'action'      => 'woocommerce_before_checkout_form',
                        'priority'    => 0,
                        'title'       => __('Before checkout form', DYNAMICBLOCK_TEXT),
                        'description' => '',
                    ),
                    'woocommerce_checkout_before_customer_details'  => array(
                        'action'      => 'woocommerce_checkout_before_customer_details',
                        'priority'    => 0,
                        'title'       => __('Before customer details', DYNAMICBLOCK_TEXT),
                        'description' => '',
                    ),
                    'woocommerce_before_checkout_billing_form'      => array(
                        'action'      => 'woocommerce_before_checkout_billing_form',
                        'priority'    => 0,
                        'title'       => __('Before Billing form', DYNAMICBLOCK_TEXT),
                        'description' => '',
                    ),
                    'woocommerce_after_checkout_billing_form'       => array(
                        'action'      => 'woocommerce_after_checkout_billing_form',
                        'priority'    => 0,
                        'title'       => __('After Billing form', DYNAMICBLOCK_TEXT),
                        'description' => '',
                    ),
                    'woocommerce_before_checkout_shipping_form'     => array(
                        'action'      => 'woocommerce_before_checkout_shipping_form',
                        'priority'    => 0,
                        'title'       => __('Before Shipping form', DYNAMICBLOCK_TEXT),
                        'description' => '',
                    ),
                    'woocommerce_after_checkout_shipping_form'      => array(
                        'action'      => 'woocommerce_after_checkout_shipping_form',
                        'priority'    => 0,
                        'title'       => __('After Shipping form', DYNAMICBLOCK_TEXT),
                        'description' => '',
                    ),
                    'woocommerce_before_order_notes'                => array(
                        'action'      => 'woocommerce_before_order_notes',
                        'priority'    => 0,
                        'title'       => __('Before order note', DYNAMICBLOCK_TEXT),
                        'description' => '',
                    ),
                    'woocommerce_after_order_notes'                 => array(
                        'action'      => 'woocommerce_after_order_notes',
                        'priority'    => 0,
                        'title'       => __('After order note', DYNAMICBLOCK_TEXT),
                        'description' => '',
                    ),
                    'woocommerce_checkout_after_customer_details'   => array(
                        'action'      => 'woocommerce_checkout_after_customer_details',
                        'priority'    => 0,
                        'title'       => __('After Customer Details', DYNAMICBLOCK_TEXT),
                        'description' => '',
                    ),
                    'woocommerce_checkout_before_order_review'      => array(
                        'action'      => 'woocommerce_checkout_before_order_review',
                        'priority'    => 0,
                        'title'       => __('Before order review', DYNAMICBLOCK_TEXT),
                        'description' => '',
                    ),
                    'woocommerce_review_order_before_cart_contents' => array(
                        'action'      => 'woocommerce_review_order_before_cart_contents',
                        'priority'    => 0,
                        'title'       => __('Review Order before cart contents', DYNAMICBLOCK_TEXT),
                        'description' => '',
                    ),
                    'woocommerce_review_order_after_cart_contents'  => array(
                        'action'      => 'woocommerce_review_order_after_cart_contents',
                        'priority'    => 0,
                        'title'       => __('Review Order after cart contents', DYNAMICBLOCK_TEXT),
                        'description' => '',
                    ),
                    'woocommerce_review_order_before_shipping'      => array(
                        'action'      => 'woocommerce_review_order_before_shipping',
                        'priority'    => 0,
                        'title'       => __('Review Order Before Shipping', DYNAMICBLOCK_TEXT),
                        'description' => '',
                    ),
                    'woocommerce_review_order_after_shipping'       => array(
                        'action'      => 'woocommerce_review_order_after_shipping',
                        'priority'    => 0,
                        'title'       => __('Review Order After Shipping', DYNAMICBLOCK_TEXT),
                        'description' => '',
                    ),
                    'woocommerce_review_order_before_order_total'   => array(
                        'action'      => 'woocommerce_review_order_before_order_total',
                        'priority'    => 0,
                        'title'       => __('Review Order before order total', DYNAMICBLOCK_TEXT),
                        'description' => '',
                    ),
                    'woocommerce_review_order_after_order_total'    => array(
                        'action'      => 'woocommerce_review_order_after_order_total',
                        'priority'    => 0,
                        'title'       => __('Review Order After order total', DYNAMICBLOCK_TEXT),
                        'description' => '',
                    ),
                    'woocommerce_review_order_before_payment'       => array(
                        'action'      => 'woocommerce_review_order_before_payment',
                        'priority'    => 0,
                        'title'       => __('Review order before payment', DYNAMICBLOCK_TEXT),
                        'description' => '',
                    ),
                    'woocommerce_review_order_after_payment'        => array(
                        'action'      => 'woocommerce_review_order_after_payment',
                        'priority'    => 0,
                        'title'       => __('Review order after payment', DYNAMICBLOCK_TEXT),
                        'description' => '',
                    ),
                    'woocommerce_checkout_after_order_review'       => array(
                        'action'      => 'woocommerce_checkout_after_order_review',
                        'priority'    => 0,
                        'title'       => __('After order review', DYNAMICBLOCK_TEXT),
                        'description' => '',
                    ),
                    'woocommerce_after_checkout_form'               => array(
                        'action'      => 'woocommerce_after_checkout_form',
                        'priority'    => 0,
                        'title'       => __('After checkout form', DYNAMICBLOCK_TEXT),
                        'description' => '',
                    ),
                ),
            ),
            'wc_login_register_page' => array(
                'title'       => __('Login / Register Page', DYNAMICBLOCK_TEXT),
                'description' => __('WC Login / Register Page', DYNAMICBLOCK_TEXT),
                'hooks'       => array(
                    'woocommerce_before_customer_login_form' => array(
                        'action'      => 'woocommerce_before_customer_login_form',
                        'priority'    => 0,
                        'title'       => __('Before Customer login form', DYNAMICBLOCK_TEXT),
                        'description' => '',
                    ),
                    'woocommerce_login_form_start'           => array(
                        'action'      => 'woocommerce_login_form_start',
                        'priority'    => 0,
                        'title'       => __('Login form start', DYNAMICBLOCK_TEXT),
                        'description' => '',
                    ),
                    'woocommerce_login_form_end'             => array(
                        'action'      => 'woocommerce_login_form_end',
                        'priority'    => 0,
                        'title'       => __('Login form end', DYNAMICBLOCK_TEXT),
                        'description' => '',
                    ),
                    'woocommerce_register_form_start'        => array(
                        'action'      => 'woocommerce_register_form_start',
                        'priority'    => 0,
                        'title'       => __('Register form start', DYNAMICBLOCK_TEXT),
                        'description' => '',
                    ),
                    'woocommerce_register_form_end'          => array(
                        'action'      => 'woocommerce_register_form_end',
                        'priority'    => 0,
                        'title'       => __('Register Form end', DYNAMICBLOCK_TEXT),
                        'description' => '',
                    ),
                    'woocommerce_after_customer_login_form'  => array(
                        'action'      => 'woocommerce_after_customer_login_form',
                        'priority'    => 0,
                        'title'       => __('After customer login form', DYNAMICBLOCK_TEXT),
                        'description' => '',
                    ),
                ),
            ),
            'wc_my_account_page'     => array(
                'title'       => __('My Account Page', DYNAMICBLOCK_TEXT),
                'description' => __('My Account Page', DYNAMICBLOCK_TEXT),
                'hooks'       => array(
                    'woocommerce_before_account_navigation' => array(
                        'action'      => 'woocommerce_before_account_navigation',
                        'priority'    => 0,
                        'title'       => __('Before Account navigation', DYNAMICBLOCK_TEXT),
                        'description' => '',
                    ),
                    'woocommerce_after_account_navigation'  => array(
                        'action'      => 'woocommerce_after_account_navigation',
                        'priority'    => 0,
                        'title'       => __('After account navigation', DYNAMICBLOCK_TEXT),
                        'description' => '',
                    ),
                    'woocommerce_before_edit_account_form'  => array(
                        'action'      => 'woocommerce_before_edit_account_form',
                        'priority'    => 0,
                        'title'       => __('Before edit account form', DYNAMICBLOCK_TEXT),
                        'description' => '',
                    ),
                    'woocommerce_after_edit_account_form'   => array(
                        'action'      => 'woocommerce_after_edit_account_form    ',
                        'priority'    => 0,
                        'title'       => __('After edit account form', DYNAMICBLOCK_TEXT),
                        'description' => '',
                    ),
                ),
            ),
            'wc_thank_you_page'      => array(
                'title'       => __('Thank You Page', DYNAMICBLOCK_TEXT),
                'description' => __('Thank You Page', DYNAMICBLOCK_TEXT),
                'hooks'       => array(
                    'woocommerce_before_thankyou' => array(
                        'action'      => 'woocommerce_before_thankyou    ',
                        'priority'    => 0,
                        'title'       => __('Before Thankyou', DYNAMICBLOCK_TEXT),
                        'description' => '',
                    ),
                ),
            ),
        );

        return apply_filters('DynamicBlock_Main.dynamicblock_default_locations', $locations);
    }

    /**
     * Track product views.
     */

    function wc_track_product_view()
    {
        if (! is_singular('product')) {
            return;
        }

        global $post;
        if (empty($_COOKIE['woocommerce_recently_viewed'])) { // @codingStandardsIgnoreLine.
            $viewed_products = array();
        } else {
            $viewed_products = wp_parse_id_list((array)explode('|',
                wp_unslash($_COOKIE['woocommerce_recently_viewed']))); // @codingStandardsIgnoreLine.
        }

        // Unset if already in viewed products list.
        $keys = array_flip($viewed_products);

        if (isset($keys[$post->ID])) {
            unset($viewed_products[$keys[$post->ID]]);
        }

        $viewed_products[] = $post->ID;
        if (count($viewed_products) > 15) {
            array_shift($viewed_products);
        }

        // Store for session only.
        wc_setcookie('woocommerce_recently_viewed', implode('|', $viewed_products));
    }

}
