<?php

defined( 'ABSPATH' ) || exit;

class DynamicBlock_Grouped
{
    public function __construct() {
   }

    /**
     * List grouped products.
     *
     * @param array $atts Attributes.
     * @return string
     */
    public static function dynamicblock_grouped_products_sc( $atts ) {
        if ( isset( $atts['per_page'] ) ) {
            $atts['limit'] = $atts['per_page'];
        }

        // @codingStandardsIgnoreStart
        $atts = shortcode_atts( array(
            'limit'    => '4',
            'columns'  => '4',
            'orderby'  => 'rand',
        ), $atts, 'grouped_products' );
        // @codingStandardsIgnoreEnd

        ob_start();

        // Rename arg.
        $atts['posts_per_page'] = absint( $atts['limit'] );

        self::dynamicblock_grouped_products( $atts );

        return ob_get_clean();
    }

    /**
     * List grouped products.
     *
     * @param array $atts Attributes.
     * @return string
     */
    public static function grouped_products( $atts ) {

        if ( isset( $atts['per_page'] ) ) {
            $atts['limit'] = $atts['per_page'];
        }

        // @codingStandardsIgnoreStart
        $atts = shortcode_atts( array(
            'limit'    =>  ($atts['dynamicblock_rules']['what_to_display']['datasets']['limit']) ?: '',
            'columns'  => '4',
            'orderby'  => ($atts['dynamicblock_rules']['what_to_display']['datasets']['order_by'][0]) ?: '',
            'order'     => ($atts['dynamicblock_rules']['what_to_display']['datasets']['order']) ?: '',
            'dynamicblock_rules' => $atts['dynamicblock_rules'],
            'skin_type' => $atts['skin_type'],
            'enable_excerpt' => $atts['enable_excerpt'],
            'enable_cart_button' => $atts['enable_cart_button'],
            'enable_autoplay' => $atts['enable_autoplay'],
            'enable_wishlist' => $atts['enable_wishlist'],
        ), $atts, 'grouped_products' );
        // @codingStandardsIgnoreEnd



        // Rename arg.
        $atts['posts_per_page'] = absint( $atts['limit'] );

        self::dynamicblock_grouped_products( $atts );
    }

    /**
     * Output the grouped products.
     *
     * @param array $args Provided arguments.
     */
    public static function dynamicblock_grouped_products( $args = array() ) {

        global $product;

        if ( ! $product ) {
            return;
        }

        $defaults = array(
            'posts_per_page' => 2,
            'columns'        => 2,
            'orderby'        => 'rand',
            'order'          => 'desc',
        );
        $tax_query = array();
        if(isset($args['dynamicblock_rules']['what_to_display']['post_query_rules'])) {
            foreach($args['dynamicblock_rules']['what_to_display']['post_query_rules'] as $_post_query_rules) {

                if(!empty($_post_query_rules['dynamicblock_query'])) {
                    foreach($_post_query_rules['dynamicblock_query'] as $dynamicblock_query) {

                        if($dynamicblock_query['dataset'] == 'product_type') {
                            if(empty($tax_query)) {
                                $tax_query = array(
                                    'relation' => 'OR',
                                    array(
                                        'relation' => 'AND',
                                        array(
                                            'taxonomy' => 'product_type',
                                            'field'    => 'term_id',
                                            'terms'    => $dynamicblock_query['tagfields'], //simple, grouped, variable, external
                                        ),
                                    )
                                );
                            } else {
                                $tax_query[0][] = array(
                                    'taxonomy' => 'product_type',
                                    'field'    => 'term_id',
                                    'terms'    => $dynamicblock_query['tagfields'], 
                                );
                            }
                        }
                        if($dynamicblock_query['dataset'] == 'product_category') {
                            if(empty($tax_query)) {
                                $tax_query = array(
                                    'relation' => 'OR',
                                    array(
                                        'relation' => 'AND',
                                        array(
                                            'taxonomy' => 'product_cat',
                                            'field'    => 'term_id',
                                            'terms'    => $dynamicblock_query['tagfields'], 
                                        ),
                                    )
                                );
                            } else {
                                $tax_query[0][] = array(
                                    'taxonomy' => 'product_cat',
                                    'field'    => 'term_id',
                                    'terms'    => $dynamicblock_query['tagfields'], 
                                );
                            }
                        }
                        if($dynamicblock_query['dataset'] == 'product_tag') {
                            if(empty($tax_query)) {
                                $tax_query = array(
                                    'relation' => 'OR',
                                    array(
                                        'relation' => 'AND',
                                        array(
                                            'taxonomy' => 'product_tag',
                                            'field'    => 'term_id',
                                            'terms'    => $dynamicblock_query['tagfields'], 
                                        ),
                                    )
                                );
                            } else {
                                $tax_query[0][] = array(
                                    'taxonomy' => 'product_tag',
                                    'field'    => 'term_id',
                                    'terms'    => $dynamicblock_query['tagfields'], 
                                );
                            }
                        }
                        if($dynamicblock_query['dataset'] == 'product_attribute') {
                            foreach($dynamicblock_query['tagfields'] as $tagfields) {
                                $_attribute_data = explode('|', $tagfields);
                                $tax_query[0][] = array(
                                    'taxonomy' => $_attribute_data[0],
                                    'field'    => 'term_id',
                                    'terms'    => $_attribute_data[1], 
                                );
                            }
                        }
                    }
                }
            }
        }

        $meta_query = array();
        if(isset($args['dynamicblock_rules']['what_to_display']['post_query_rules'])) {
            foreach($args['dynamicblock_rules']['what_to_display']['post_query_rules'] as $_post_query_rules) {

                if(!empty($_post_query_rules['dynamicblock_query'])) {
                    foreach($_post_query_rules['dynamicblock_query'] as $dynamicblock_query) {

                        if($dynamicblock_query['dataset'] == 'product_price') {
                            if(empty($tax_query)) {
                                $meta_query = array(
                                    'relation' => 'AND',
                                    array(
                                        'key'       => '_price',
                                        'value'     => $dynamicblock_query['tagfields'],
                                        'compare'   => '>=',
                                    ),
                                );
                            } else {
                                $meta_query[] = array(
                                    'key'       => '_price',
                                    'value'     => $dynamicblock_query['tagfields'],
                                    'compare'   => '>=',
                                );
                            }
                        }

                    }
                }
            }
        }
     
        $_args = array(
            'post_type' => 'product',
            'post_status' => 'publish',
            'posts_per_page' => $args['dynamicblock_rules']['what_to_display']['datasets']['limit'],
            'fields'        => 'ids',
            'orderby'        => (isset($args['dynamicblock_rules']['what_to_display']['datasets']['order_by']['0'])) ? $args['dynamicblock_rules']['what_to_display']['datasets']['order_by']['0'] : 'date',
            'order'          => $args['dynamicblock_rules']['what_to_display']['datasets']['order'],
            
            'category__in' => array(),
            'category__not_in' => array(),
            'post__in' => array(),
            'post__not_in' => array(),
            'tag__in' => array(),
            'tag__not_in' => array(),
            'author__in' => array(),
            'author__not_in' => array(),
        );
        if(!empty($tax_query)) {
            $_args['tax_query'] = $tax_query;
        }
        if(!empty($meta_query)) {
            $_args['meta_query'] = $meta_query;
        }
        $_products = get_posts($_args);
        $_products_arr = array();
        foreach($_products as $_product_id) {
            $_products_arr[] = wc_get_product($_product_id);
        }
        $args = wp_parse_args( $args, $defaults );

        // Get visible grouped products then sort them at random. $_products
        $args['grouped_products'] = $_products_arr;
       
        // Handle orderby.
        $args['grouped_products'] = wc_products_array_orderby( $args['grouped_products'], $args['orderby'], $args['order'] );

        // Set global loop values.
        wc_set_loop_prop( 'name', 'grouped' );
        wc_set_loop_prop( 'columns', apply_filters( 'dynamicblock_grouped_products_columns', $args['columns'] ) );
        DynamicBlock_Main::dynamicblock_get_template( 'public/grouped-products.php', $args );
    }
}