<?php
/**
 * DynamicBlock Related Product Class
 */

defined('ABSPATH') || exit;

class DynamicBlock_Related_Product
{

    public function __construct()
    {
        remove_action('woocommerce_after_single_product_summary', 'woocommerce_output_related_products', 20);
    }

    /**
     * List related products.
     *
     * @param array $atts Attributes.
     * @return string
     */
    public static function dynamicblock_related_products_sc($atts)
    {
        if (isset($atts['per_page'])) {
            $atts['limit'] = $atts['per_page'];
        }

        // @codingStandardsIgnoreStart
        $atts = shortcode_atts(array(
            'limit' => '4',
            'columns' => '4',
            'orderby' => 'rand',
        ), $atts, 'related_products');
        // @codingStandardsIgnoreEnd

        ob_start();

        // Rename arg.
        $atts['posts_per_page'] = absint($atts['limit']);

        self::dynamicblock_related_products($atts);

        return ob_get_clean();
    }

    /**
     * List related products.
     *
     * @param array $atts Attributes.
     * @return string
     */
    public static function related_products($atts)
    {


        if (isset($atts['per_page'])) {
            $atts['limit'] = $atts['per_page'];
        }
        // @codingStandardsIgnoreStart
        $atts = shortcode_atts(array(
            'limit' => ($atts['dynamicblock_rules']['what_to_display']['datasets']['limit']) ?: '',
            'columns' => '4',
            'orderby' => ($atts['dynamicblock_rules']['what_to_display']['datasets']['order_by'][0]) ?: '',
            'order' => ($atts['dynamicblock_rules']['what_to_display']['datasets']['order']) ?: '',
            'dynamicblock_rules' => $atts['dynamicblock_rules'],
            'skin_type' => $atts['skin_type'],
            'enable_excerpt' => $atts['enable_excerpt'],
            'enable_cart_button' => $atts['enable_cart_button'],
            'enable_autoplay' => $atts['enable_autoplay'],
            'enable_wishlist' => $atts['enable_wishlist'],
        ), $atts, 'related_products');
        // @codingStandardsIgnoreEnd

        // Rename arg.
        $atts['posts_per_page'] = absint($atts['limit']);

        self::dynamicblock_related_products($atts);
    }

    /**
     * Output the related products.
     *
     * @param array $args Provided arguments.
     */
    public static function dynamicblock_related_products($args = array())
    {
        $defaults = array(
            'posts_per_page' => -1,
            'orderby' => 'rand',
            'order' => 'desc',
            'fields' => 'ids',
            'post_type' => 'product',
            'post_status' => 'publish',
           
        );



        $the_query = new WP_Query( $defaults );

        $last_query_product = DynamicBlock_Conditional_Content_Display:: dynamicblock_what_to_display_result_product($args);

        $_products_arr = array();
        $_args = array(
            'post_type' => 'product',
            'post_status' => 'publish',
            'posts_per_page' => ($args['dynamicblock_rules']['what_to_display']['datasets']['limit']) ?: '5',
            'orderby' => (isset($args['dynamicblock_rules']['what_to_display']['datasets']['order_by']['0'])) ? $args['dynamicblock_rules']['what_to_display']['datasets']['order_by']['0'] : 'date',
            'order' => $args['dynamicblock_rules']['what_to_display']['datasets']['order'],
            'post__in' => ($last_query_product) ? $last_query_product : '',
        );

       

        if ($last_query_product) {
            $last_query_product = new WP_Query($_args);
        }



        if(isset($last_query_product->posts)){
            foreach ($last_query_product->posts as $_product_id) {
                $_products_arr[] = wc_get_product($_product_id);
            }
            $args = wp_parse_args($args, $defaults);
            // Get visible related products then sort them at random. $_products
            //$_products_arr = array();
            $args['related_products'] = $_products_arr;
            
            // Handle orderby.
            $args['related_products'] = wc_products_array_orderby($args['related_products'], $args['orderby'], $args['order']);

            // Set global loop values.
            wc_set_loop_prop('name', 'related');
            wc_set_loop_prop('columns', apply_filters('dynamicblock_related_products_columns', $args['columns']));
            DynamicBlock_Main::dynamicblock_get_template('public/related-products.php', $args);
        }
    }
}

add_filter('posts_where', function ($where, $query) {
    $label = $query->query['query_label'] ?? '';
    $dynamicblock_args = $query->query['dynamicblock_args'] ?? '';

    if($label === 'dynamicblock_data_filter') {
        global $wpdb;
        $where = " IN(
                SELECT r.term_taxonomy_id IN(36)
                AND post_meta.meta_key = '_stock_status'
                AND post_meta.meta_value = 'instock'
                AND post_meta_1.meta_key ='_price' 
                AND post_meta_1.meta_value = '20')
            AND(
                SELECT r.term_taxonomy_id IN(36)
                AND post_meta_1.meta_key = '_price'
                AND post_meta_1.meta_value = '20')
            OR(
                post_meta.meta_key = '_manage_stock'
                AND post_meta.meta_value = 'yes'
                AND post_meta_1.meta_key ='wp_capabilities'
                AND post_meta_1.meta_value = 'administrator'
            ) ". $where;
    }

    return $where;
}, 10, 2);

add_filter('posts_join', function ($join, $query) {
    global $wpdb;
    $dynamicblock_args = $query->query['dynamicblock_args'] ?? '';

    $label = $query->query['query_label'] ?? '';
    if($label === 'dynamicblock_data_filter') {
        $join .= "LEFT JOIN $wpdb->term_relationships r ON ($wpdb->posts.ID = r.object_id) 
         INNER JOIN $wpdb->postmeta post_meta ON $wpdb->posts.ID = post_meta.post_id 
         JOIN $wpdb->postmeta post_meta_1 ON $wpdb->posts.ID = post_meta_1.post_id 
         JOIN $wpdb->postmeta post_meta_2 ON $wpdb->posts.ID = post_meta_2.post_id ";
    }
    return $join;
},15,2);
