<?php

/**
 * The class for displaying conditional content blocks.
 * This is where the content block rules are tested and content displayed in the hook as defined in the content block.
 */
class DynamicBlock_Conditional_Content_Display
{
    private static $instance = null;
    private static $contents = array();
    private static $loop_contents = array();

    /**
     * Registers a single instance of the DynamicBlock_Conditional_Content_Display class
     */
    public static function register()
    {
        if (self::$instance == null) {
            self::$instance = new DynamicBlock_Conditional_Content_Display();
        }
    }

    /**
     * Gets the single instance of the DynamicBlock_Conditional_Content_Display class.
     *
     * @return DynamicBlock_Conditional_Content_Display
     */
    public static function instance()
    {
        self::register();

        return self::$instance;
    }

    private $locations;

    /**
     * Creates a new instance of the DynamicBlock_Conditional_Content_Display class.
     */
    public function __construct()
    {
        add_action('init', array($this, 'load_locations'), 0);
        add_action('template_redirect', array($this, 'bind_display'), 0);
        remove_action('woocommerce_after_single_product_summary', 'woocommerce_output_related_products', 20);
        add_shortcode('dynamicblock_block', array($this, 'demoashok'));
    }

    public function demoashok($attr)
    {
        $_post_obj = get_post($attr['id']);
        if (isset($_post_obj)) {
            ob_start();
            $_post_obj->shortcode = 'true';
            $_output              = $this->demo_code($_post_obj);
            $_output              = ob_get_contents();
            ob_end_clean();

            return $_output;
        } else {
            return false;
        }
    }

    public function load_locations()
    {
        $this->locations = apply_filters('dynamicblock_conditional_content_get_locations', array(), 0);
    }

    /**
     * Hooked into the template_redirect function.  Loads the configured content blocks and binds them based on
     * the rules which have been configured for the content block.
     *
     * Content is stored internally and is displayed using the __call method.
     */
    public function bind_display($_contents = array(), $current_shortcode = array())
    {
        if (is_admin()) {
            return;
        }
        if ($_contents) {
            $contents = $_contents;
        } else {
            $contents = get_posts(array(
                'post_type'   => 'dynamicblock-block',
                'post_status' => 'publish',
                'nopaging'    => true,
            ));
        }

        $dynamicblock_and_query = array();
        $dynamicblock_or_query  = array();
        if ($contents && count($contents)) {
            foreach ($contents as $content) {
                $display                        = false;
                $loop                           = false;
                $settings                       = get_post_meta($content->ID, 'where_to_display_control', true);
                $current_dynamicblock_post_data = get_post_meta($content->ID, 'dynamicblock_post', true);

                $what_post_query_rules = $current_dynamicblock_post_data['dynamicblock_post_' . $content->ID]['what_to_display']['post_query_rules'] ?? array();

                $when_post_query_rules     = $current_dynamicblock_post_data['dynamicblock_post_' . $content->ID]['when_to_display']['post_query_rules'] ?? array();
                $new_when_post_query_rules = array();
                $new_dynamicblock_query    = array();
                $current_block_type        = $current_dynamicblock_post_data['dynamicblock_post_' . $content->ID]['block_info']['post_block_type'];
//                if($content->ID != 7184) { continue;}
                foreach ($when_post_query_rules as $group_rule_key => $when_post_query_rule) {
                    $rule = $when_post_query_rule['rules-expression'];
                    if ($rule == 'or') {
                        $new_when_post_query_rules = self::build_new_when_post_query_rules($new_when_post_query_rules,
                            $new_dynamicblock_query);
                        $new_dynamicblock_query    = array();
                    }
                    if (isset($when_post_query_rule['dynamicblock_query'])) {
                        foreach ($when_post_query_rule['dynamicblock_query'] as $key => $dynamicblock_query) {
                            $dataset    = isset($dynamicblock_query['dataset']) ? $dynamicblock_query['dataset'] : '';
                            $condition  = isset($dynamicblock_query['condition']) ? $dynamicblock_query['condition'] : '';
                            $tag_fields = isset($dynamicblock_query['tagfields']) ? $dynamicblock_query['tagfields'] : '';

                            if ($dataset == 'show_always') {
                                $new_dynamicblock_query['show_always'] = array(
                                    'dataset'   => 'show_always',
                                    'condition' => '==',
                                );
                            } elseif ($dataset == 'cart_product' || $dataset == 'checkout_products') {
                                $new_dynamicblock_query[$dataset . '_' . $condition . '_' . $tag_fields['qty']]['dataset']          = $dataset;
                                $new_dynamicblock_query[$dataset . '_' . $condition . '_' . $tag_fields['qty']]['condition']        = $condition;
                                $new_dynamicblock_query[$dataset . '_' . $condition . '_' . $tag_fields['qty']]['tagfields']['qty'] = $tag_fields['qty'];
                                if (isset($tag_fields['product'])) {
                                    foreach ($tag_fields['product'] as $tag_filed) {
                                        $new_dynamicblock_query[$dataset . '_' . $condition . '_' . $tag_fields['qty']]['tagfields']['product'][] = $tag_filed;
                                    }
                                }
                                $new_dynamicblock_query[$dataset . '_' . $condition . '_' . $tag_fields['qty']]['tagfields']['product'] =
                                    @(is_array($new_dynamicblock_query[$dataset . '_' . $condition . '_' . $tag_fields['qty']]['tagfields']['product'])
                                        ? array_unique($new_dynamicblock_query[$dataset . '_' . $condition . '_' . $tag_fields['qty']]['tagfields']['product'])
                                        : ($new_dynamicblock_query[$dataset . '_' . $condition . '_' . $tag_fields['qty']]['tagfields']['product']));
                            } elseif ($dataset == 'cart_category' || $dataset == 'checkout_categories') {

                                $new_dynamicblock_query[$dataset . '_' . $condition . '_' . $tag_fields['qty']]['dataset']          = $dataset;
                                $new_dynamicblock_query[$dataset . '_' . $condition . '_' . $tag_fields['qty']]['condition']        = $condition;
                                $new_dynamicblock_query[$dataset . '_' . $condition . '_' . $tag_fields['qty']]['tagfields']['qty'] = $tag_fields['qty'];
                                if (isset($tag_fields['categories'])) {
                                    foreach ($tag_fields['categories'] as $tag_filed) {
                                        $new_dynamicblock_query[$dataset . '_' . $condition . '_' . $tag_fields['qty']]['tagfields']['categories'][] = $tag_filed;
                                    }
                                }
                                $new_dynamicblock_query[$dataset . '_' . $condition . '_' . $tag_fields['qty']]['tagfields']['categories'] =
                                    (is_array($new_dynamicblock_query[$dataset . '_' . $condition . '_' . $tag_fields['qty']]['tagfields']['categories'])
                                        ? array_unique($new_dynamicblock_query[$dataset . '_' . $condition . '_' . $tag_fields['qty']]['tagfields']['categories'])
                                        : ($new_dynamicblock_query[$dataset . '_' . $condition . '_' . $tag_fields['qty']]['tagfields']['categories']));
                            } elseif ($dataset == 'products'
                                      || $dataset == 'product_type'
                                      || $dataset == 'product_category'
                                      || $dataset == 'product_attribute'
                                      || $dataset == 'product_tag'
                                      || $dataset == 'cart_line_item_product'
                                      || $dataset == 'checkout_billing_country_field'
                                      || $dataset == 'checkout_shipping_country_field'
                                      || $dataset == 'brands'
                                      || $dataset == 'schedule_day'
                                      || $dataset == 'user'
                                      || $dataset == 'role'
                            ) {
                                if (isset($tag_fields) && is_array($tag_fields)) {
                                    $new_dynamicblock_query[$dataset . '_' . $condition]['dataset']   = $dataset;
                                    $new_dynamicblock_query[$dataset . '_' . $condition]['condition'] = $condition;
                                    foreach ($tag_fields as $tag_filed) {
                                        $new_dynamicblock_query[$dataset . '_' . $condition]['tagfields'][] = $tag_filed;
                                    }
                                }
                                $new_dynamicblock_query[$dataset . '_' . $condition]['tagfields'] =
                                    @(is_array($new_dynamicblock_query[$dataset . '_' . $condition]['tagfields'])
                                        ? array_unique($new_dynamicblock_query[$dataset . '_' . $condition]['tagfields'])
                                        : ($new_dynamicblock_query[$dataset . '_' . $condition]['tagfields']));
                            } else {
                                $new_dynamicblock_query[] = $dynamicblock_query;
                            }
                        }
                    }
                }

                $when_post_query_rules = self::build_new_when_post_query_rules($new_when_post_query_rules,
                    $new_dynamicblock_query);

                $_query          = '';
                $result          = 'start';
                $has_custom_hook = false;

                if (! empty($settings)) {
                    foreach ($settings as $key => $setting) {
                        if ($setting['wc_page'] == 'wc_custom_hook') {
                            $has_custom_hook = true;
                        }
                    }
                }

                if (isset($current_shortcode['shortcode']) == 'true') {
                    $all_over_last_result             = self::get_when_to_display_result($when_post_query_rules,
                        $current_block_type);
                    $content->where_to_display_result = ($all_over_last_result) ? 'true' : 'false';
                    $content->shortcode               = 'done';
                    $tag                              = 'display_shortcode_done_' . $content->ID;
                    //self::$contents = array();
                    self::$contents[$tag][] = $content;
                    $this->$tag($content);
                } else {
                    if (! empty($settings)) {
                        foreach ($settings as $key => $setting) {
                            $_array_result[] = array();
                            $custom_hook     = false;
                            $custom_priority = false;
                            $accepted_args   = 0;
                            if ($setting['hook'] ?? '' == 'wc_custom_hook') {
                                $custom_hook     = $setting['hook'];
                                $custom_priority = $setting['priority'];
                                $accepted_args   = isset($setting['custom_accepted_args']) ? $settings['custom_accepted_args'] : 0;
                            }

                            $all_over_last_result             = self::get_when_to_display_result($when_post_query_rules,
                                $current_block_type);
                            $display                          = apply_filters('woocommerce_conditional_content_get_should_display',
                                $display, $content);
                            $content->where_to_display_result = ($all_over_last_result) ? 'true' : 'false';

                            if ($all_over_last_result) {
                                if ($custom_hook) {
                                    $hook = apply_filters('woocommerce_conditional_content_get_output_hook', array(
                                        'action'        => $custom_hook,
                                        'priority'      => $custom_priority,
                                        'accepted_args' => $accepted_args,
                                    ), $settings, $content);
                                    if ($hook && isset($hook['action']) && isset($hook['priority'])) {
                                        $_priority = (isset($hook['priority']) && ! empty($hook['priority'])) ? $hook['priority'] : 0;
                                        $tag       = 'display_' . str_replace('-', '_',
                                                sanitize_title($hook['action'])) . '_' . $_priority . '_' . $content->ID;
                                        if ($loop) {
                                            self::$loop_contents[$tag][] = $content;
                                        } else {
                                            self::$contents[$tag][] = $content;
                                        }

                                        add_action($hook['action'], array($this, $tag), $hook['priority'],
                                            $accepted_args);
                                    }
                                } elseif (isset($this->locations[$setting['wc_page']]['hooks'][$setting['hook']])) {
                                    $hook_setting = $this->locations[$setting['wc_page']]['hooks'][$setting['hook']];
                                    $_priority    = (isset($hook['priority']) && ! empty($hook['priority'])) ? $hook['priority'] : 0;
                                    $hook         = apply_filters('woocommerce_conditional_content_get_output_hook',
                                        array(
                                            'action'        => $hook_setting['action'],
                                            'priority'      => $hook_setting['priority'],
                                            'accepted_args' => isset($hook_setting['custom_accepted_args']) ? $hook_setting['custom_accepted_args'] : 1,
                                        ), $settings, $content);

                                    if ($hook && isset($hook['action']) && isset($hook['priority'])) {
                                        $tag                    = 'display_' . str_replace('-', '_',
                                                sanitize_title($hook['action'])) . '_' . $_priority . '_' . $content->ID;
                                        self::$contents[$tag][] = $content;
                                        $accepted_args          = isset($hook_setting['custom_accepted_args']) ? $hook_setting['custom_accepted_args'] : 0;
                                        add_action($hook['action'], array(
                                            $this,
                                            $tag,
                                        ), $hook['priority'], $accepted_args);
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
        wp_reset_query();
    }

    /**
     * Display or retrieve conditional content.  This is used with the template tag woocommerce_conditional_content()
     *
     * @param int  $content_id Optional. The content to process.
     * @param bool $echo       Optional, default to true.Whether to display or return.
     *
     * @return null|string Null if no content rules match. String if $echo parameter is false and content rules match.
     * @since 1.0.0
     *
     */
    public function build_new_when_post_query_rules($new_when_post_query_rules, $new_dynamicblock_query)
    {
        $new_when_post_query_rules[rand()] = array(
            'rules-expression'   => 'or',
            'dynamicblock_query' => $new_dynamicblock_query,
        );

        return $new_when_post_query_rules;
    }

    private function get_when_to_display_result($when_post_query_rules, $current_block_type = '')
    {
        global $all_over_last_result;
        $all_over_last_result = 0;
        $default_value        = 0;

        if (empty($when_post_query_rules)) {
            $when_post_query_rules[] = array(
                'rules-expression'   => 'or',
                'dynamicblock_query' => array(
                    '0' => array(
                        'dataset'   => 'show_always',
                        'condition' => '==',
                    ),
                ),
            );
        }

        $_array_result = array();
        $last_count    = 0;
        if (! empty($when_post_query_rules)) {
            foreach ($when_post_query_rules as $group_rule_key => $when_post_query_rule) {
                $group_condition = $when_post_query_rule['rules-expression'];
                if ($group_condition == 'or') {
                    $last_count++;
                }
                if (! $default_value) {
                    $group_condition = '';
                }
                if (isset($when_post_query_rule['dynamicblock_query'])) {
                    $current_group_last_result = 'new';
                    foreach ($when_post_query_rule['dynamicblock_query'] as $rules_key => $dynamicblock_query_single) {
                        if (isset($dynamicblock_query_single['dataset']) && isset($dynamicblock_query_single['condition']) && isset($dynamicblock_query_single['tagfields'])) {

                            $rule        = array(
                                'operator'  => $dynamicblock_query_single['condition'],
                                'condition' => $dynamicblock_query_single['tagfields'],
                                'rule_type' => $dynamicblock_query_single['dataset'],
                            );
                            $rule_object = dynamicblock_conditional_content_get_rule_object($dynamicblock_query_single['dataset']);
                            $result      = $rule_object->is_match($rule);
                            if ($current_group_last_result || ! $result) {
                                $current_group_last_result = $result;
                            }
                            if ($result == 1) {
                                $_when_display_result = true;
                            } else {
                                $result = 0;
                            }
                        } else {
                        }
                        if (isset($dynamicblock_query_single['dataset']) && $dynamicblock_query_single['dataset'] == 'show_always') {
                            $_when_display_result = true;
                            $result               = 1;
                        }
                        $_array_result[$last_count][] = $result;
                    }
                }
                $default_value++;
            }
        }
        $last_group_result  = 'new';
        $result_array_count = count($_array_result);
        foreach ($_array_result as $key => $single_group_result) {
            $_result = '';
            if (in_array('0', $single_group_result)) {
                $_result = '0';
            } else {
                $_result = 'true';
            }
            if ($last_group_result == 'new') {
                $last_group_result = $_result;
            } else {
                if ($last_group_result || $result) {
                    $last_group_result = 'true';
                } else {
                    $last_group_result = '0';
                }
            }
        }

        return $last_group_result;
    }

    public function template_display($content_id = 0, $echo = true)
    {
        if ($content_id) {
            $contents = get_posts(array(
                'p'           => $content_id,
                'post_type'   => 'wccc',
                'post_status' => 'publish',
                'nopaging'    => true,
            ));
        } else {
            $contents = get_posts(array('post_type' => 'wccc', 'post_status' => 'publish', 'nopaging' => true));
        }

        $display = false;

        if ($contents && count($contents)) {

            foreach ($contents as $content) {
                $settings = get_post_meta($content->ID, '_wccc_settings', true);

                if ($settings['location'] == 'single-product' && ! is_product()) {
                    //We can skip checking if the rule matches.
                    continue;
                }
                $groups = get_post_meta($content->ID, 'wccc_rule', true);

                if ($groups && count($groups)) {
                    foreach ($groups as $group_id => $group) {
                        $result = 'start';

                        foreach ($group as $rule_id => $rule) {
                            $rule_object = dynamicblock_conditional_content_get_rule_object($rule['rule_type']);
                            $result      = ($result != 'start' ? ($result & $rule_object->is_match($rule)) : $rule_object->is_match($rule));
                        }

                        if ($result) {
                            $display = true;
                        }
                    }
                }

                $display = apply_filters('dynamicblock_get_should_display', $display, $content);
                if ($display) {

                    $content_result = apply_filters('dynamicblock_the_content',
                        apply_filters('the_content', $content->post_content));
                    if ($echo) {
                        wc_get_template('contentblock.php', array(
                            'content' => $content,
                        ), 'woocommerce-conditional-content', WC_Conditional_Content::plugin_path() . '/templates/');
                    }

                    return $content_result;
                }
            }
        }
    }

    /*
     * Helper function to render the content which is bound to each hook.
     */
    public function __call($name, $arguments)
    {
        global $all_over_last_result;
        global $current_shortcode;
        $rebind_filter = false;
        if (has_filter('the_content', 'sharing_display')) {
            $rebind_filter = true;
            remove_filter('the_content', 'sharing_display', 19);
        }
        if (isset($arguments)) {
            if (isset($arguments[0]->shortcode) && $arguments[0]->shortcode == 'true') {
                $current_shortcode = array('block_id' => $arguments[0]->ID, 'shortcode' => $arguments[0]->shortcode);
                self::bind_display($arguments, $current_shortcode);

                return;
            }
        }

        if (isset(self::$contents[$name])) {
            foreach (self::$contents[$name] as $content) {
                if ($content->where_to_display_result != 'false') {
                    $_block_obj          = get_post_meta($content->ID, 'dynamicblock_post', true);
                    $current_post        = get_post($content->ID);
                    $post_title          = isset($current_post->post_title) ? $current_post->post_title : '';
                    $_dynamicblock_rules = (isset($_block_obj['dynamicblock_post_' . $content->ID])) ? $_block_obj['dynamicblock_post_' . $content->ID] : 'no_data_found';

                    /*  echo "<pre>";
                      print_r($_dynamicblock_rules);
                      die;*/
                    $_skin_type          = (isset($_block_obj['dynamicblock_post_' . $content->ID]['how_to_display']['type-setting']['enable-carousel'])) ? 'carousel' : 'grid';
                    $_enable_excerpt     = (isset($_block_obj['dynamicblock_post_' . $content->ID]['how_to_display']['type-setting']['show_product_excerpt'])) ? 'enable_excerpt' : 'no_excerpt';
                    $_enable_cart_button = (isset($_block_obj['dynamicblock_post_' . $content->ID]['how_to_display']['type-setting']['display-add-to-cart-button'])) ? 'enable_cart_button' : 'no_cart_button';
                    $_enable_autoplay    = (isset($_block_obj['dynamicblock_post_' . $content->ID]['how_to_display']['type-setting']['autoPlay'])) ? 'enable_autoplay' : 'no_autoplay';
                    $_enable_wishlist    = (isset($_block_obj['dynamicblock_post_' . $content->ID]['how_to_display']['type-setting']['display-wishlist-button'])) ? 'enable_wishlist' : 'no_wishlist';
                    $post_block_type     = (isset($_block_obj['dynamicblock_post_' . $content->ID]['block_info']['post_block_type']) && ! empty($_block_obj['dynamicblock_post_' . $content->ID]['block_info']['post_block_type'])) ? $_block_obj['dynamicblock_post_' . $content->ID]['block_info']['post_block_type'] : 'html';

                    $is_active = false;
                    if (class_exists('DynamicBlocksBuilderActivator')) {
                        $_activator = new DynamicBlocksBuilderActivator();
                        $is_active  = $_activator->is_active();
                    }

                    switch ($post_block_type) {
                        // Pro
                        case "accessories":
                            if ($is_active && class_exists('DynamicBlockAccessories')) {
                                $_accessories_products = new DynamicBlockAccessories;
                                $_accessories_products->accessories_products(array(
                                        'dynamicblock_rules' => $_dynamicblock_rules,
                                        'skin_type'          => $_skin_type,
                                        'enable_excerpt'     => $_enable_excerpt,
                                        'enable_cart_button' => $_enable_cart_button,
                                        'enable_autoplay'    => $_enable_autoplay,
                                        'enable_wishlist'    => $_enable_wishlist,
                                        'current_post_title' => $post_title,
                                    )
                                );
                            }
                            break;
                        // Pro
                        case "bundle-pack":
                            if ($is_active && class_exists('DynamicBlockBundle')) {
                                $_bundle_products = new DynamicBlockBundle;
                                $_bundle_products->bundle_products(array(
                                        'dynamicblock_rules' => $_dynamicblock_rules,
                                        'skin_type'          => $_skin_type,
                                        'enable_excerpt'     => $_enable_excerpt,
                                        'enable_cart_button' => $_enable_cart_button,
                                        'enable_autoplay'    => $_enable_autoplay,
                                        'enable_wishlist'    => $_enable_wishlist,
                                        'current_post_title' => $post_title,
                                    )
                                );
                            }
                            break;
                        // Free
                        case "related-products":
                            // Related products
                            //if($is_active){
                            $_related_products = new DynamicBlock_Related_Product;
                            $_related_products->related_products(array(
                                    'dynamicblock_rules' => $_dynamicblock_rules,
                                    'skin_type'          => $_skin_type,
                                    'enable_excerpt'     => $_enable_excerpt,
                                    'enable_cart_button' => $_enable_cart_button,
                                    'enable_wishlist'    => $_enable_wishlist,
                                    'enable_autoplay'    => $_enable_autoplay,
                                    'current_post_title' => $post_title,
                                )
                            );
                            //}
                            break;
                        default:
                            // Html contents
                            wc_get_template('public/html-content.php', array('content' => $content),
                                DYNAMICBLOCK_MAIN_DIR_NAME, DYNAMICBLOCK_TEMPLATE_PATH);
                    }
                }
            }
        }
        if ($rebind_filter) {
            add_filter('the_content', 'sharing_display', 19);
        }
    }

    public function match_groups($content, $arguments = null)
    {
        $display = false;

        $where_to_display  = get_post_meta($content->ID, 'where_to_display_control', true);
        $dynamicblock_post = get_post_meta($content->ID, 'dynamicblock_post', true);
        $_content_key      = 'dynamicblock_post_' . $content->ID;

        if ($dynamicblock_post[$_content_key]['when_to_display']['post_query_rules'] && count($dynamicblock_post[$_content_key]['when_to_display']['post_query_rules'])) {
            foreach ($dynamicblock_post[$_content_key]['when_to_display']['post_query_rules'] as $group_id => $group) {
                $result = null;
                foreach ($group['dynamicblock_query'] as $rule_id => $rule) {
                    $rule_object = dynamicblock_conditional_content_get_rule_object($rule['dataset']);
                    if (is_object($rule_object)) {
                        $rule_new = array(
                            'rule_type' => $rule['dataset'],
                            'operator'  => $rule['condition'],
                            'condition' => $rule['tagfields'],
                        );
                        $match    = $rule_object->is_match($rule_new, $arguments);
                        $result   = ($result !== null ? ($result & $match) : $match);
                    }
                }
                if ($result) {
                    $display = true;
                }
            }
        } else {
            $display = true; //Always display the content if no rules have been configured.
        }

        return $display;
    }

    public static function dynamicblock_what_to_display_result_product($args)
    {

        global $wpdb;

        $get_parent_variation_sale_product_ids = array();

        $tax_query          = array();




        $meta_query         = array();
        $date_query         = array();
        $post_in            = array();
        $post_not_in        = array();
        $last_query_product = array();
        $_args              = array();
        $new_custom_array   = array();

        if (isset($args['dynamicblock_rules']['what_to_display']['post_query_rules'])) {
            $i = 0;
            foreach ($args['dynamicblock_rules']['what_to_display']['post_query_rules'] as $key => $_post_query_rules) {

                if (! empty($_post_query_rules['dynamicblock_query'])) {
                    $relation = ($_post_query_rules['rules-expression']);
                    if ($relation == 'or' && $i != 0) {
                        $new_custom_array[] = $_args;
                        $_args              = array();
                    }
                    if (! $_args) {
                        $_rule_tax_query = array();
                        $meta_query      = array();
                        $post_in         = array();
                        $post_not_in     = array();
                        $author_in       = array();
                        $author_not_in   = array();

                        // if (empty($_rule_tax_query)) {
                        //     $_rule_tax_query = array(
                        //         'relation' => 'and',
                        //         array(
                        //             'taxonomy' => 'product_visibility',
                        //             'field'    => 'term_id',
                        //             // 'operator' => ($dynamicblock_query['condition'] == 'in') ? 'IN' : 'NOT IN',
                        //             'terms'    => 'featured',
                        //         ),
                        //     );
                        // } else {
                        //     $_rule_tax_query[] = array(
                        //         'taxonomy' => 'product_visibility',
                        //         'field'    => 'term_id',
                        //         // 'operator' => ($dynamicblock_query['condition'] == 'in') ? 'IN' : 'NOT IN',
                        //         'terms'    => 'featured',
                        //     );
                        // }
                    }


                    foreach ($_post_query_rules['dynamicblock_query'] as $rule_key => $dynamicblock_query) {

                        if (isset($dynamicblock_query['tagfields'])) {
                            if (isset($dynamicblock_query) && ! empty($dynamicblock_query['dataset']) && ! empty($dynamicblock_query['condition']) && ($dynamicblock_query['tagfields'] != null || $dynamicblock_query['tagfields'] == 0)) {

                                if ($dynamicblock_query['dataset'] == 'products') {
                                    if ($dynamicblock_query['condition'] == 'in') {
                                        $post_in = ($post_in) ? self::run_relation_query($args,
                                            $dynamicblock_query['tagfields'], $post_in,
                                            'or') : $dynamicblock_query['tagfields'];
                                    } else {
                                        $post_not_in = ($post_not_in) ? self::run_relation_query($args,
                                            $dynamicblock_query['tagfields'], $post_not_in,
                                            'or') : $dynamicblock_query['tagfields'];
                                    }
                                }

                                /*ex membership start - now USER ROLE*/
                                if ($dynamicblock_query['dataset'] == 'user') {
                                    foreach ($dynamicblock_query['tagfields'] as $id) {
                                        $ids[] = $id;
                                        if ($dynamicblock_query['condition'] == 'in') {
                                            $author_in[] = $id;
                                        } else {
                                            $author_not_in[] = $id;
                                        }
                                    }
                                }

                                if ($dynamicblock_query['dataset'] == 'role') {
                                    $ids        = array();
                                    $single_ids = array();
                                    foreach ($dynamicblock_query['tagfields'] as $tagfields) {
                                        $single_ids = get_users(array('role' => $tagfields, 'fields' => 'ID'));
                                        foreach ($single_ids as $id) {
                                            $ids[] = $id;
                                            if ($dynamicblock_query['condition'] == 'in') {
                                                $author_in[] = $id;
                                            } else {
                                                $author_not_in[] = $id;
                                            }
                                        }
                                    }
                                }

                                /*tax_query start*/
                                if ($dynamicblock_query['dataset'] == 'product_type') {
                                    if (empty($_rule_tax_query)) {
                                        $_rule_tax_query = array(
                                            'relation' => 'and',
                                            array(
                                                'taxonomy' => 'product_type',
                                                'field'    => 'term_id',
                                                'operator' => ($dynamicblock_query['condition'] == 'in') ? 'IN' : 'NOT IN',
                                                'terms'    => $dynamicblock_query['tagfields'],
                                                //simple, grouped, variable, external
                                            ),
                                        );
                                    } else {
                                        $_rule_tax_query[] = array(
                                            'taxonomy' => 'product_type',
                                            'field'    => 'term_id',
                                            'operator' => ($dynamicblock_query['condition'] == 'in') ? 'IN' : 'NOT IN',
                                            'terms'    => $dynamicblock_query['tagfields'],
                                            //simple, grouped, variable, external
                                        );
                                    }
                                }

                                if ($dynamicblock_query['dataset'] == 'product_category') {
                                    if (empty($_rule_tax_query)) {
                                        $_rule_tax_query = array(
                                            'relation' => 'AND',
                                            array(
                                                'taxonomy' => 'product_cat',
                                                'field'    => 'term_id',
                                                'terms'    => $dynamicblock_query['tagfields'],
                                                'operator' => ($dynamicblock_query['condition'] == 'in') ? 'IN' : 'NOT IN',
                                            ),
                                        );
                                    } else {
                                        $_rule_tax_query[] = array(
                                            'taxonomy' => 'product_cat',
                                            'field'    => 'term_id',
                                            'terms'    => $dynamicblock_query['tagfields'],
                                            'operator' => ($dynamicblock_query['condition'] == 'in') ? 'IN' : 'NOT IN',
                                        );
                                    }
                                }

                                if ($dynamicblock_query['dataset'] == 'brands') {
                                    if (empty($_rule_tax_query)) {
                                        $_rule_tax_query = array(
                                            'relation' => 'AND',
                                            array(
                                                'taxonomy' => DYNAMICBLOCK_YITH_WOOCOMMERCE_PRODUCT_BRAND,
                                                'field'    => 'term_id',
                                                'terms'    => $dynamicblock_query['tagfields'],
                                                'operator' => ($dynamicblock_query['condition'] == 'in') ? 'IN' : 'NOT IN',
                                            ),
                                        );
                                    } else {
                                        $_rule_tax_query[] = array(
                                            'taxonomy' => DYNAMICBLOCK_YITH_WOOCOMMERCE_PRODUCT_BRAND,
                                            'field'    => 'term_id',
                                            'terms'    => $dynamicblock_query['tagfields'],
                                            'operator' => ($dynamicblock_query['condition'] == 'in') ? 'IN' : 'NOT IN',
                                        );
                                    }
                                }

                                if ($dynamicblock_query['dataset'] == 'product_tag') {
                                    if (empty($_rule_tax_query)) {
                                        $_rule_tax_query = array(
                                            'relation' => 'AND',
                                            array(
                                                'taxonomy' => 'product_tag',
                                                'field'    => 'term_id',
                                                'terms'    => $dynamicblock_query['tagfields'],
                                                'operator' => ($dynamicblock_query['condition'] == 'in') ? 'IN' : 'NOT IN',
                                            ),
                                        );
                                    } else {
                                        $_rule_tax_query[] = array(
                                            'taxonomy' => 'product_tag',
                                            'field'    => 'term_id',
                                            'terms'    => $dynamicblock_query['tagfields'],
                                            'operator' => ($dynamicblock_query['condition'] == 'in') ? 'IN' : 'NOT IN',
                                        );
                                    }
                                }

                                if ($dynamicblock_query['dataset'] == 'product_attribute') {

                                    foreach ($dynamicblock_query['tagfields'] as $tagfields) {
                                        $_attribute_data = explode('|', $tagfields);
                                        if (empty($_rule_tax_query)) {
                                            $_rule_tax_query = array(
                                                'relation' => 'AND',
                                                array(
                                                    'taxonomy' => $_attribute_data[0],
                                                    'field'    => 'term_id',
                                                    'operator' => ($dynamicblock_query['condition'] == 'in') ? 'IN' : 'NOT IN',
                                                    'terms'    => $_attribute_data[1],
                                                ),
                                            );
                                        } else {
                                            $_rule_tax_query[] = array(
                                                'taxonomy' => $_attribute_data[0],
                                                'field'    => 'term_id',
                                                'operator' => ($dynamicblock_query['condition'] == 'in') ? 'IN' : 'NOT IN',
                                                'terms'    => $_attribute_data[1],
                                            );
                                        }
                                    }
                                }

                                /*meta_query start*/

                                if ($dynamicblock_query['dataset'] == 'product_meta') {

                                    $type = '';
                                    if (is_numeric($dynamicblock_query['tagfields']['meta-value'])) {
                                        $type = 'numeric';
                                    }

                                    if (empty($meta_query)) {
                                        $meta_query = array(
                                            'relation' => 'AND',
                                            array(
                                                'key'     => $dynamicblock_query['tagfields']['meta-key'],
                                                'value'   => 1,
                                                'compare' => '>=',
                                                'type'    => $type,
                                            ),
                                            array(
                                                'key'     => $dynamicblock_query['tagfields']['meta-key'],
                                                'value'   => $dynamicblock_query['tagfields']['meta-value'],
                                                'compare' => htmlspecialchars_decode($dynamicblock_query['condition']),
                                                'type'    => $type,
                                            ),
                                        );
                                    } else {
                                        $meta_query[] = array(
                                            'key'     => $dynamicblock_query['tagfields']['meta-key'],
                                            'value'   => $dynamicblock_query['tagfields']['meta-value'],
                                            'compare' => $dynamicblock_query['condition'],
                                            'type'    => $type,
                                        );
                                        $meta_query[] = array(
                                            'key'     => $dynamicblock_query['tagfields']['meta-key'],
                                            'value'   => 1,
                                            'compare' => '>=',
                                            'type'    => $type,
                                        );
                                    }
                                }

                                if ($dynamicblock_query['dataset'] == 'product_price') {
                                    if (empty($meta_query)) {
                                        $meta_query = array(
                                            'relation' => 'AND',
                                            array(
                                                    array(
                                                        'relation' => 'OR',
                                                        array(
                                                            'key'     => '_price',
                                                            'value'   => $dynamicblock_query['tagfields'],
                                                            'compare' => htmlspecialchars_decode($dynamicblock_query['condition']),
                                                            'type'    => 'NUMERIC',
                                                        ),
                                                        array(
                                                            'key'     => '_sale_price',
                                                            'value'   => $dynamicblock_query['tagfields'],
                                                            'compare' => htmlspecialchars_decode($dynamicblock_query['condition']),
                                                            'type'    => 'NUMERIC',
                                                        ),
                                                    ),
                                            ),
                                        );
                                    } else {
                                        $meta_query[] = array(
                                                        'relation' => 'OR',
                                                        array(
                                                            'key'     => '_price',
                                                            'value'   => $dynamicblock_query['tagfields'],
                                                            'compare' => htmlspecialchars_decode($dynamicblock_query['condition']),
                                                            'type'    => 'NUMERIC',
                                                        ),
                                                        array(
                                                            'key'     => '_sale_price',
                                                            'value'   => $dynamicblock_query['tagfields'],
                                                            'compare' => htmlspecialchars_decode($dynamicblock_query['condition']),
                                                            'type'    => 'NUMERIC',
                                                        ),
                                                    );

                                    }
                                }

                                if ($dynamicblock_query['dataset'] == 'stock_status') {
                                    if (empty($meta_query)) {
                                        $meta_query = array(
                                            'relation' => 'AND',
                                            array(
                                                'key'     => '_stock_status',
                                                'value'   => ($dynamicblock_query['tagfields']) ? 'instock' : 'outofstock',
                                                'compare' => htmlspecialchars_decode($dynamicblock_query['condition']),
                                            ),
                                        );
                                    } else {
                                        $meta_query[] = array(
                                            'key'     => '_stock_status',
                                            'value'   => ($dynamicblock_query['tagfields']) ? 'instock' : 'outofstock',
                                            'compare' => htmlspecialchars_decode($dynamicblock_query['condition']),
                                        );
                                    }
                                }


                                if ($dynamicblock_query['dataset'] == 'stock_level') {
                                    if (empty($meta_query)) {
                                        $meta_query = array(
                                            'relation' => 'AND',
                                            array(
                                                'key'     => '_stock',
                                                'type'    => 'numeric',
                                                'value'   => $dynamicblock_query['tagfields'],
                                                'compare' => htmlspecialchars_decode($dynamicblock_query['condition']),
                                            ),
                                        );
                                    } else {
                                        $meta_query[] = array(
                                            'key'     => '_stock',
                                            'type'    => 'numeric',
                                            'value'   => $dynamicblock_query['tagfields'],
                                            'compare' => htmlspecialchars_decode($dynamicblock_query['condition']),
                                        );
                                    }
                                }

                                /*product_set start*/
                                if ($dynamicblock_query['dataset'] == 'product_set') {

                                    $_product_set_loop_count = 0;

                                    if(!is_array($dynamicblock_query['tagfields'])){
                                        $dynamicblock_query['tagfields'] = array($dynamicblock_query['tagfields']);
                                    }

                                    foreach ($dynamicblock_query['tagfields'] as $tagfields) {

                                        if ( $tagfields == "featured" ) {

                                            if (empty($_rule_tax_query)) {
                                                $_rule_tax_query = array(
                                                    'relation' => 'and',
                                                    array(
                                                        'taxonomy' => 'product_visibility',
                                                        'field'    => 'name',
                                                        'operator' => ($dynamicblock_query['condition'] == 'in') ? 'IN' : 'NOT IN',
                                                        'terms'    => 'featured',
                                                    ),
                                                );
                                            } else {
                                                $_rule_tax_query[] = array(
                                                    'taxonomy' => 'product_visibility',
                                                    'field'    => 'name',
                                                    'operator' => ($dynamicblock_query['condition'] == 'in') ? 'IN' : 'NOT IN',
                                                    'terms'    => 'featured',
                                                );
                                            }                                            
                                        }

                                        if ($tagfields == 'best_selling') {
                                            if (empty($meta_query)) {
                                                $meta_query = array(
                                                    'relation' => 'AND',
                                                    array(
                                                        'key'     => 'total_sales',
                                                        'compare' => ($dynamicblock_query['condition'] == 'in') ? '>' : '<=',
                                                        'value'   => 10,
                                                    ),
                                                );
                                            } else {
                                                $meta_query[] = array(
                                                    'key'     => 'total_sales',
                                                    'compare' => ($dynamicblock_query['condition'] == 'in') ? '>' : '<=',
                                                    'value'   => 10,
                                                );
                                            }

                                            $date_query[] = array(
                                                // ($dynamicblock_query['condition'] == 'in') ? 'after' : 'before' => date('Y-m-d', strtotime('-365 day')),
                                                'after' => date('Y-m-d', strtotime('-365 day')),
                                            );
                                        }

                                        if ($tagfields == 'top_rated') {
                                            if (empty($meta_query)) {
                                                $meta_query = array(
                                                    'relation' => 'AND',
                                                    array(
                                                        'key'     => '_wc_average_rating',
                                                        'compare' => ($dynamicblock_query['condition'] == 'in') ? '>' : '<=',
                                                        'value'   => 4,
                                                    ),
                                                );
                                            } else {
                                                $meta_query[] = array(
                                                    'key'     => '_wc_average_rating',
                                                    'compare' => ($dynamicblock_query['condition'] == 'in') ? '>' : '<=',
                                                    'value'   => 4,
                                                );
                                            }
                                        }

                                        if ($tagfields == 'recent') {

                                            $date_query[] = array(
                                                ($dynamicblock_query['condition'] == 'in') ? 'after' : 'before' => date('Y-m-d', strtotime('-3 month')),
                                            );

                                        }

                                        if ($tagfields == 'on_sale') {

                                            $get_parent_variation_sale_product_ids_sql = 'SELECT DISTINCT('.$wpdb->prefix.'posts.post_parent) AS post_id FROM '.$wpdb->prefix.'posts JOIN '.$wpdb->prefix.'postmeta ON '.$wpdb->prefix.'posts.ID = '.$wpdb->prefix.'postmeta.post_id WHERE '.$wpdb->prefix.'posts.post_type = "product_variation" AND '.$wpdb->prefix.'posts.post_status = "publish" AND ( '.$wpdb->prefix.'postmeta.meta_key = "_sale_price" AND '.$wpdb->prefix.'postmeta.meta_value >= 0 )';

                                            $get_parent_variation_sale_product_ids_db = $wpdb->get_results( $wpdb->prepare( $get_parent_variation_sale_product_ids_sql ) );

                                            if( !empty( $get_parent_variation_sale_product_ids_db ) ) {
                                                foreach ($get_parent_variation_sale_product_ids_db as $value) {
                                                    if ( $dynamicblock_query['condition'] == 'in' ) {
                                                        $get_parent_variation_sale_product_ids[] = $value->post_id;
                                                    }
                                                    else{
                                                        $post_not_in[] = $value->post_id;
                                                    }
                                                }
                                            }

                                            if (empty($meta_query)) {
                                                $meta_query = array(
                                                    'relation' => 'AND',
                                                    array(
                                                        'key'     => '_sale_price',
                                                        'compare' => ($dynamicblock_query['condition'] == 'in') ? 'EXISTS' : 'NOT EXISTS',
                                                    ),
                                                );
                                            } else {
                                                $meta_query[] = array(
                                                    'key'     => '_sale_price',
                                                    'compare' => ($dynamicblock_query['condition'] == 'in') ? 'EXISTS' : 'NOT EXISTS',
                                                );
                                            }
                                        }

                                        $_product_set_loop_count++;
                                    }
                                }

                                /*store start*/
                                if ($dynamicblock_query['dataset'] == 'store_order_count') {
                                    if (empty($meta_query)) {
                                        if(htmlspecialchars_decode($dynamicblock_query['condition']) == '<' || htmlspecialchars_decode($dynamicblock_query['condition']) == '<=') {
                                            $meta_query = array(
                                                'relation' => 'AND',
                                                array(
                                                    'key'     => 'total_sales',
                                                    'type'    => 'numeric',
                                                    'value'   => $dynamicblock_query['tagfields']['qty'],
                                                    'compare' => htmlspecialchars_decode($dynamicblock_query['condition']),
                                                ),
                                                array(
                                                    'key'     => 'total_sales',
                                                    'type'    => 'numeric',
                                                    'value'   => 0,
                                                    'compare' => '>',
                                                ),
                                            );
                                        }else{
                                            $meta_query = array(
                                            'relation' => 'AND',
                                                array(
                                                    'key'     => 'total_sales',
                                                    'type'    => 'numeric',
                                                    'value'   => $dynamicblock_query['tagfields']['qty'],
                                                    'compare' => htmlspecialchars_decode($dynamicblock_query['condition']),
                                                ),
                                            );    
                                        }
                                    } 
                                    else 
                                    {
                                                if(htmlspecialchars_decode($dynamicblock_query['condition']) == '<' || htmlspecialchars_decode($dynamicblock_query['condition']) == '<=') 
                                                {    
                                                    $meta_query[] = array(
                                                                        'relation' => 'AND',
                                                                        array(
                                                                            'key'     => 'total_sales',
                                                                            'type'    => 'numeric',
                                                                            'value'   => $dynamicblock_query['tagfields']['qty'],
                                                                            'compare' => htmlspecialchars_decode($dynamicblock_query['condition']),
                                                                        ),
                                                                        array(
                                                                            'key'     => 'total_sales',
                                                                            'type'    => 'numeric',
                                                                            'value'   => 0,
                                                                            'compare' => '>',
                                                                        ),
                                                                    );
                                                }else{    
                                                    $meta_query[] = array(
                                                        'key'     => 'total_sales',
                                                        'type'    => 'numeric',
                                                        'value'   => $dynamicblock_query['tagfields']['qty'],
                                                        'compare' => htmlspecialchars_decode($dynamicblock_query['condition']),
                                                    );
                                                }
                                    }
                                }
                                if ($dynamicblock_query['dataset'] == 'store_order_status') {
                                    $query         = new WC_Order_Query();
                                    $_current_list = array();
                                    $orders        = $query->get_orders();
                                    foreach ($orders as $single_order) {
                                        $order_data = $single_order->get_data();
                                        $tagfields  = $dynamicblock_query['tagfields']['status'];
                                        $status     = explode('wc-', $tagfields);

                                        if ($order_data['status'] == $status[1]) {
                                            foreach ($single_order->get_items() as $item_key => $item_values):
                                                if ($dynamicblock_query['condition'] == '==') {
                                                    $post_in[]       = $item_values->get_product_id();
                                                    $_current_list[] = $item_values->get_product_id();
                                                } else {
                                                    $_current_list[] = $item_values->get_product_id();
                                                    $post_not_in[]   = $item_values->get_product_id();
                                                }
                                            endforeach;
                                        }
                                    }
                                    if (empty($_current_list)) {
                                        $_rule_tax_query = array();
                                        $meta_query      = array();
                                        $post_in         = array();
                                        $post_not_in     = array();
                                        $author_in       = array();
                                        $author_not_in   = array();
                                    }
                                }
                            }
                        }
                    }

                    if ($_rule_tax_query || $meta_query || $post_in || $post_not_in || $author_in || $author_not_in || $date_query) {

                        if ($i == 0) {
                            $_args              = self::merge_query_args($args, $_rule_tax_query, $meta_query, $post_in, $post_not_in, array_unique($author_in), array_unique($author_not_in) , $date_query);
                            $last_query_product = self::get_products_by_args($_args);
                        }
                        if ($i != 0) {
                            $_args                 = self::merge_query_args($args, $_rule_tax_query, $meta_query, $post_in, $post_not_in, array_unique($author_in), array_unique($author_not_in) , $date_query);
                            $current_group_product = self::get_products_by_args($_args);
                            $last_query_product    = self::run_relation_query($_args, $current_group_product,
                                $last_query_product, $relation);
                        }
                    }
                }
                $i++;
            }
        }
        $new_custom_array[] = $_args;

        $last_query_product = array_merge( $get_parent_variation_sale_product_ids , self::get_new_bind_product($new_custom_array));



        return $last_query_product;
    }

    public static function get_new_bind_product($new_custom_array)
    {
        $last_posts = array();
        foreach ($new_custom_array as $custom_array) {
            $new_tax_query = array();
            if (isset($custom_array['tax_query'])) {
                $i = 0;
                foreach ($custom_array['tax_query'] as $tax_query) {
                    $taxonomy         = isset($tax_query['taxonomy']) ? $tax_query['taxonomy'] : '';
                    $operator         = isset($tax_query['operator']) ? $tax_query['operator'] : '';
                    $field            = isset($tax_query['field']) ? $tax_query['field'] : '';
                    $terms            = isset($tax_query['terms']) ? $tax_query['terms'] : '';
                    $explode_taxonomy = explode("_", $taxonomy);

                    if ($i != 0 && $terms) {
                        $new_tax_query[$taxonomy . '_' . $operator]['taxonomy'] = $taxonomy;
                        $new_tax_query[$taxonomy . '_' . $operator]['field']    = $field;
                        $new_tax_query[$taxonomy . '_' . $operator]['operator'] = $operator;
                        if ($taxonomy == 'product_cat' || $taxonomy == 'product_tag' || $taxonomy == 'product_type' || $taxonomy == DYNAMICBLOCK_YITH_WOOCOMMERCE_PRODUCT_BRAND) {
                            if (is_array($terms)) {
                                foreach ($terms as $term) {
                                    $new_tax_query[$taxonomy . '_' . $operator]['terms'][] = $term;
                                }
                            }
                        } else if ($explode_taxonomy[0] == 'pa' || $taxonomy == 'product_visibility') {
                            $new_tax_query[$taxonomy . '_' . $operator]['terms'][] = $terms;
                        }
                    }
                    $i++;
                }
            }

            if (isset($custom_array['meta_query'])) {
                $i = 0;
                foreach ($custom_array['meta_query'] as $meta_query) {
                    $key = isset($meta_query['key']) ? $meta_query['key'] : '';
                    if ($i != 0) {
                        if ($key == 'total_sales') {
                            $order_by['total_sales'] = 'DESC';
                        } elseif ($key == '_wc_average_rating') {
                            $order_by['_wc_average_rating'] = 'DESC';
                        }
                    }
                    $i++;
                }
            }

            if (isset($custom_array) && ! empty($custom_array)) {
                $new_tax_query['relation'] = 'AND';
                $custom_array['tax_query'] = $new_tax_query;
                $custom_array['orderby']   = array();
                $custom_array['orderby']   = isset($order_by) ? $order_by : $custom_array['orderby'];
                $the_query                 = new WP_Query($custom_array);
                $posts                     = $the_query->posts;
                $last_posts                = self::run_relation_query($the_query, $posts, $last_posts, 'or');
            }
        }

        return $last_posts;
    }

    public static function run_relation_query($_args, $current_group_product, $last_query_product, $relation)
    {
        $result = '';

        if ($relation == 'or') {
            $result = (array_unique(array_merge($last_query_product, $current_group_product)));

            return $result;
        }
        if ($relation == 'and') {
            $result = $current_group_product;

            return $result;
        }
    }

    public static function get_products_by_args($_args = array())
    {
        $current_group_product = get_posts($_args);

        return $current_group_product;
    }

    public static function merge_query_args(
        $args,
        $tax_query,
        $meta_query,
        $post_in,
        $post_not_in,
        $author_in = array(),
        $author_not_in = array(),
        $date_query = array()
    ) {
        $_args = array(
            'post_type'      => 'product',
            'post_status'    => 'publish',
            'posts_per_page' => -1,
            'fields'         => 'ids',
            'orderby'        => (isset($args['dynamicblock_rules']['what_to_display']['datasets']['order_by']['0'])) ? $args['dynamicblock_rules']['what_to_display']['datasets']['order_by']['0'] : 'date',
            'order'          => isset($args['dynamicblock_rules']['what_to_display']['datasets']['order']) ? $args['dynamicblock_rules']['what_to_display']['datasets']['order'] : 'ASC',
            'post__in'       => ($post_in) ? $post_in : '',
            'post__not_in'   => ($post_not_in) ? $post_not_in : '',
            'author__in'     => $author_in,
            'author__not_in' => $author_not_in,
            'date_query' => $date_query,
        );
        if (! empty($tax_query)) {
            $_args['tax_query'] = $tax_query;
        }
        if (! empty($meta_query)) {
            $_args['meta_query'] = $meta_query;
        }

        return $_args;
    }

    public static function get_new_custom_product($new_custom_array)
    {
        $last_posts          = array();
        $new_cat_in_list     = array();
        $new_cat_not_in_list = array();

        $new_product_type_in_list     = array();
        $new_product_type_not_in_list = array();

        $new_product_tag_in_list     = array();
        $new_product_tag_not_in_list = array();

        $new_pa_color_in_list     = array();
        $new_pa_color_not_in_list = array();

        $new_brand_in_list     = array();
        $new_brand_not_in_list = array();

        foreach ($new_custom_array as $key => $custom_array) {

            $new_cat_in_list     = array();
            $new_cat_not_in_list = array();

            $new_product_type_in_list     = array();
            $new_product_type_not_in_list = array();

            $new_product_tag_in_list     = array();
            $new_product_tag_not_in_list = array();

            $new_pa_color_in_list     = array();
            $new_pa_color_not_in_list = array();

            $new_pa_size_in_list     = array();
            $new_pa_size_not_in_list = array();

            $new_brand_in_list     = array();
            $new_brand_not_in_list = array();
            $new_tax_query         = array();
            $i                     = 0;

            if (isset($custom_array['tax_query'])) {
                foreach ($custom_array['tax_query'] as $tax_query) {

                    if ($i != 0) {
                        if ($tax_query['taxonomy'] == 'product_cat' && $tax_query['operator'] == 'IN') {
                            foreach ($tax_query['terms'] as $terms) {
                                $new_cat_in_list[] = $terms;
                            }
                        }
                        if ($tax_query['taxonomy'] == 'product_cat' && $tax_query['operator'] == 'NOT IN') {
                            foreach ($tax_query['terms'] as $terms) {
                                $new_cat_not_in_list[] = $terms;
                            }
                        }
                        if ($tax_query['taxonomy'] == 'product_type' && $tax_query['operator'] == 'IN') {
                            foreach ($tax_query['terms'] as $terms) {
                                $new_product_type_in_list[] = $terms;
                            }
                        }
                        if ($tax_query['taxonomy'] == 'product_type' && $tax_query['operator'] == 'NOT IN') {
                            foreach ($tax_query['terms'] as $terms) {
                                $new_product_type_not_in_list[] = $terms;
                            }
                        }
                        if ($tax_query['taxonomy'] == 'product_tag' && $tax_query['operator'] == 'IN') {
                            foreach ($tax_query['terms'] as $terms) {
                                $new_product_tag_in_list[] = $terms;
                            }
                        }
                        if ($tax_query['taxonomy'] == 'product_tag' && $tax_query['operator'] == 'NOT IN') {
                            foreach ($tax_query['terms'] as $terms) {
                                $new_product_tag_not_in_list[] = $terms;
                            }
                        }
                        if ($tax_query['taxonomy'] == 'pa_color' && $tax_query['operator'] == 'IN') {
                            $new_pa_color_in_list[] = $tax_query['terms'];
                        }
                        if ($tax_query['taxonomy'] == 'pa_color' && $tax_query['operator'] == 'NOT IN') {
                            $new_pa_color_not_in_list[] = $tax_query['terms'];
                        }

                        if ($tax_query['taxonomy'] == 'pa_size' && $tax_query['operator'] == 'IN') {
                            $new_pa_size_in_list[] = $tax_query['terms'];
                        }
                        if ($tax_query['taxonomy'] == 'pa_size' && $tax_query['operator'] == 'NOT IN') {
                            $new_pa_size_not_in_list[] = $tax_query['terms'];
                        }
                        if ($tax_query['taxonomy'] == DYNAMICBLOCK_YITH_WOOCOMMERCE_PRODUCT_BRAND && $tax_query['operator'] == 'IN') {
                            foreach ($tax_query['terms'] as $terms) {
                                $new_brand_in_list[] = $terms;
                            }
                        }
                        if ($tax_query['taxonomy'] == DYNAMICBLOCK_YITH_WOOCOMMERCE_PRODUCT_BRAND && $tax_query['operator'] == 'NOT IN') {
                            foreach ($tax_query['terms'] as $terms) {
                                $new_brand_not_in_list[] = $terms;
                            }
                        }
                    }
                    $i++;
                }
            }

            $new_tax_query['relation'] = 'AND';
            if ($new_cat_in_list) {
                $new_tax_query[] = array(
                    'taxonomy' => 'product_cat',
                    'field'    => 'term_id',
                    'terms'    => array_unique($new_cat_in_list),
                    'operator' => 'IN',
                );
            }
            if ($new_cat_not_in_list) {
                $new_tax_query[] = array(
                    'taxonomy' => 'product_cat',
                    'field'    => 'term_id',
                    'terms'    => array_unique($new_cat_not_in_list),
                    'operator' => 'NOT IN',
                );
            }
            if ($new_product_type_in_list) {
                $new_tax_query[] = array(
                    'taxonomy' => 'product_type',
                    'field'    => 'term_id',
                    'terms'    => array_unique($new_product_type_in_list),
                    'operator' => 'IN',
                );
            }
            if ($new_product_type_not_in_list) {
                $new_tax_query[] = array(
                    'taxonomy' => 'product_type',
                    'field'    => 'term_id',
                    'terms'    => array_unique($new_product_type_not_in_list),
                    'operator' => 'NOT IN',
                );
            }
            if ($new_product_tag_in_list) {
                $new_tax_query[] = array(
                    'taxonomy' => 'product_tag',
                    'field'    => 'term_id',
                    'terms'    => array_unique($new_product_tag_in_list),
                    'operator' => 'IN',
                );
            }
            if ($new_product_tag_not_in_list) {
                $new_tax_query[] = array(
                    'taxonomy' => 'product_tag',
                    'field'    => 'term_id',
                    'terms'    => array_unique($new_product_type_not_in_list),
                    'operator' => 'NOT IN',
                );
            }

            if ($new_pa_color_in_list) {
                $new_tax_query[] = array(
                    'taxonomy' => 'pa_color',
                    'field'    => 'term_id',
                    'terms'    => array_unique($new_pa_color_in_list),
                    'operator' => 'IN',
                );
            }
            if ($new_pa_color_not_in_list) {
                $new_tax_query[] = array(
                    'taxonomy' => 'pa_color',
                    'field'    => 'term_id',
                    'terms'    => array_unique($new_pa_color_not_in_list),
                    'operator' => 'NOT IN',
                );
            }
            if ($new_pa_size_in_list) {
                $new_tax_query[] = array(
                    'taxonomy' => 'pa_size',
                    'field'    => 'term_id',
                    'terms'    => array_unique($new_pa_size_in_list),
                    'operator' => 'IN',
                );
            }
            if ($new_pa_size_not_in_list) {
                $new_tax_query[] = array(
                    'taxonomy' => 'pa_size',
                    'field'    => 'term_id',
                    'terms'    => array_unique($new_pa_size_not_in_list),
                    'operator' => 'NOT IN',
                );
            }
            if ($new_brand_in_list) {
                $new_tax_query[] = array(
                    'taxonomy' => DYNAMICBLOCK_YITH_WOOCOMMERCE_PRODUCT_BRAND,
                    'field'    => 'term_id',
                    'terms'    => array_unique($new_brand_in_list),
                    'operator' => 'IN',
                );
            }
            if ($new_brand_not_in_list) {
                $new_tax_query[] = array(
                    'taxonomy' => DYNAMICBLOCK_YITH_WOOCOMMERCE_PRODUCT_BRAND,
                    'field'    => 'term_id',
                    'terms'    => array_unique($new_brand_not_in_list),
                    'operator' => 'NOT IN',
                );
            }
            $custom_array['tax_query'] = $new_tax_query;

            $the_query  = new WP_Query($custom_array);
            $posts      = $the_query->posts;
            $last_posts = self::run_relation_query($the_query, $posts, $last_posts, 'or');
        }

        return $last_posts;
    }

    public static function array_unique_and_remove_dup($data)
    {

        $mydata  = array();
        $mydata1 = array();
        $aa      = array();
        $arr     = $data;
        $i       = 0;
        foreach ($data as $vk => $data) {
            foreach ($data as $key => $d) {
                if (! is_array($d)) {
                    $mydata[$vk][$key] = $d;
                } else {
                    if (count($d) <= 2) {
                        $mydata[$vk][$key] = array_unique($d);
                    } else {
                        $mydata[$vk][$key] = $d;
                        if ($key == "tax_query") {
                            $nrr  = $mydata[$vk][$key];
                            $nrr2 = $mydata[$vk][$key];
                            $c    = 0;
                            foreach ($nrr as $k => $n1) {
                                if (empty($mydata1)) {
                                    $mydata1[$k] = $n1;
                                } else {
                                    foreach ($mydata1 as $j => $m) {
                                        if ($j != 'relation') {
                                            if ($m['taxonomy'] === $n1['taxonomy'] && $m['field'] === $n1['field'] && $m['operator'] === $n1['operator']) {
                                                $aa[$i] = $nrr[$k];
                                                $i++;
                                                unset($nrr[$k]);
                                            } else {
                                                $mydata1[$k] = $n1;
                                            }
                                        }
                                    }
                                }
                            }
                            $b   = 0;
                            $arr = array();
                            foreach ($nrr as $k => $nr) {
                                foreach ($aa as $j => $a) {
                                    if ($nr['terms']) {
                                        if ($nr['taxonomy'] == $a['taxonomy']) {
                                            if (is_array($a['terms'])) {
                                                foreach ($a['terms'] as $a) {
                                                    array_push($nrr[$k]['terms'], $a);
                                                }
                                            } else {
                                                if (is_array($nr['terms'])) {
                                                    array_push($nrr[$k]['terms'], $a['terms']);
                                                } else {
                                                    array_push($arr, $a['terms']);
                                                    if (in_array($nr['terms'], $arr)) {
                                                    } else {
                                                        array_push($arr, $nr['terms']);
                                                    }
                                                    $nrr[$k]['terms'] = $arr;
                                                }
                                            }
                                            $b++;
                                        }
                                    }
                                }
                            }
                            foreach ($nrr as $kj => $nr) {

                                if ($kj != 'relation') {
                                    $nrr[$kj]['terms'] = (is_array($nr['terms'])) ? array_unique($nr['terms']) : $nr['terms'];
                                }
                            }
                            $mydata[$vk][$key] = $nrr;
                        }
                        if ($key == "meta_query") {
                            $mydata[$vk][$key] = array_map("unserialize",
                                array_unique(array_map("serialize", $mydata[$vk][$key])));
                        }
                    }
                }
            }
        }

        return $mydata;
    }
}

DynamicBlock_Conditional_Content_Display::register();

/**
 * Creates an instance of a rule object
 *
 * @param type   $rule_type The slug of the rule type to load.
 *
 * @return WC_Conditional_Content_Rule_Base or superclass of WC_Conditional_Content_Rule_Base
 * @global array $woocommerce_conditional_content_rules
 */
function dynamicblock_conditional_content_get_rule_object($rule_type)
{
    global $dynamicblock_conditional_content_rules;

    if (isset($dynamicblock_conditional_content_rules[$rule_type])) {
        return $dynamicblock_conditional_content_rules[$rule_type];
    }

    $class = 'Dynamicblock_Conditional_Content_Rule_' . $rule_type;
    if (class_exists($class)) {
        $dynamicblock_conditional_content_rules[$rule_type] = new $class;

        return $dynamicblock_conditional_content_rules[$rule_type];
    } else {
        return null;
    }
}