<?php
/**
 * DynamicBlock Register Post Type, Taxonomy, Custom Meta Fields
 */

defined('ABSPATH') || exit;

class DynamicBlock_Public
{
    function __construct()
    {
        add_action('wp_enqueue_scripts', [$this, 'dynamicblock_scripts']);
        //include some defaults via our filters
        add_filter('wc_conditional_content_get_rule_types', array(&$this, 'default_rule_types'), 0);
        add_filter('wc_conditional_content_get_rule_operators', array(&$this, 'default_rule_operators'), 0);
        add_filter('wc_get_template_part', array(&$this, 'bfs_wc_get_template_part'), 10, 3);
        add_action('dynamicblock_woocommerce_after_shop_loop_item_title', array(&$this, 'dynamicblock_add_short_description'), 0);
        add_filter('woocommerce_loop_product_link', array(&$this, 'dynamicblock_woocommerce_loop_product_link'), 10, 2);
        add_action('template_redirect', array(&$this, 'bfs_woocommerce_ajax_add_to_cart_redirect'));
        add_action('woocommerce_cart_calculate_fees', array($this, 'add_custom_fees'));
        add_action('woocommerce_check_cart_items', array($this, 'check_cart_outlet_items'));
        add_action('woocommerce_remove_cart_item', array($this, 'after_remove_product_from_cart'), 10, 2);
        add_action('woocommerce_cart_item_removed', array($this, 'after_remove_product_from_cart'), 10, 2);
        $this->includes();
    }

    function after_remove_product_from_cart($removed_cart_item_key, $cart)
    {
        $block_type = WC()->session->get('block_type');

        if($block_type && $block_type != "accessories" && $block_type == "bundle-pack") {
            foreach ($cart as $cart_item_key => $cart_item) {
                $main_product_data = array();
                if (isset($cart_item[$removed_cart_item_key]['data'])) {
                    $_data = $cart_item[$removed_cart_item_key]['data'];
                    $main_product_data = $_data->get_data();
                }
                if ($cart_item_key == 'cart_contents') {
                    $post_block_type = isset($cart_item[$removed_cart_item_key]['block_type']) ? $cart_item[$removed_cart_item_key]['block_type'] : '';
                    if ($post_block_type == 'bundle-pack') {
                        WC()->session->set('block_type', '');
                        $isMain = array_key_exists('isMainProduct', $cart_item[$removed_cart_item_key]) ? $cart_item[$removed_cart_item_key]['isMainProduct'] : 0;
                        if ($isMain == 1) {
                            $customID = isset($cart_item[$removed_cart_item_key]['customID']) ? $cart_item[$removed_cart_item_key]['customID'] : 0;
                            foreach ($cart_item as $child_key => $child_product) {
                                if (isset($child_product['mainProduct_customeId']) && $child_product['mainProduct_customeId'] == $customID) {
                                    $child_product_data = array();
                                    if (isset($child_product['data'])) {
                                        $child_data = $child_product['data'];
                                        $child_product_data = $child_data->get_data();
                                    }
                                    // unset($cart->cart_contents[$child_key]);
                                    // WC()->cart->remove_cart_item($child_key);
                                    $wcCart = new WC_Cart();
                                    unset( $wcCart->removed_cart_contents[ $child_key ]['data'] );
                                    unset( $wcCart->cart_contents[ $child_key ] );
                                    //wc_add_notice(__('The product ' . $child_product['product_id'] . ' has been removed from cart because product ' . $cart_item[$removed_cart_item_key]['product_id'] . ' has been removed.', 'woocommerce'), 'notice');
                                    wc_add_notice(__('The product "' . $child_product_data['name'] . '" has been removed from cart because product "' . $main_product_data['name'] . '" has been removed.', 'woocommerce'), 'notice');
                                }
                            }
                        }
                    }
                }
            }
        }
    }


    function customized_cart_item_remove_link($button_link, $cart_item_key)
    {
        //SET HERE your specific products IDs
        $targeted_products_ids = array(25, 22);

        // Get the current cart item
        $cart_item = WC()->cart->get_cart()[$cart_item_key];

        // If the targeted product is in cart we remove the button link
        if (in_array($cart_item['data']->get_id(), $targeted_products_ids))
            $button_link = '';

        return $button_link;
    }

    function check_cart_outlet_items()
    {
        $_content = WC()->cart->get_cart();

        $categories = array('sports'); // Defined targeted product categories
        $threshold = 30; // Defined threshold amount

        $cart = WC()->cart;
        $cart_items = $cart->get_cart();
        $subtotal = $cart->subtotal;


        $subtotal -= $cart->get_cart_discount_total() + $cart->get_cart_discount_tax_total();
        $found = false;

        foreach ($cart_items as $cart_item_key => $cart_item) {
            if (has_term($categories, 'product_cat', $cart_item['product_id'])) {
                $found = true; // A category is found
                break;
            }
        }


        if ($found && $subtotal < $threshold) {
            // Display an error notice (and avoid checkout)
            wc_add_notice(sprintf(__("You must order at least %s"), wc_price($threshold)), 'error');
        }
    }

    function my_custom_cart_quantity($quantity)
    {
        $unique_cart_item_key = uniqid();
        $cart_item_data['unique_key'] = $unique_cart_item_key;
        return $cart_item_data;
    }

    function add_custom_fees(WC_Cart $cart)
    {
        $block_type = WC()->session->get('block_type');
        $_content = WC()->cart->get_cart();
        $_type = array();
        $j = 0;

        if (($block_type) || ($block_type == "accessories") || ($block_type == "bundle-pack")) {

            $discount_percentage = WC()->session->get('discountpercentage');
            $allow_main_product = WC()->session->get('allow_main_product');
            $main_product_id = WC()->session->get('main_product_id');
            if ($allow_main_product != 1) {
                $product = wc_get_product($main_product_id);
                // $price = !empty($product->get_sale_price()) ? (float) $product->get_sale_price() : (float) $product->get_regular_price();
                $price = (float) $product->get_price();
                $discount = ($cart->subtotal - $price) * $discount_percentage;
            } else {
                $discount = $cart->subtotal * $discount_percentage;
            }

            foreach ($_content as $content) {
                @$_type[$j] = array(
                    'block_type' => $content['block_type'],
                    'block_name' => $content['block_name'],
                    'block_id' => $content['block_id'],
                    'isMainProduct' => $content['isMainProduct'],
                    'blockQty' => $content['quantity'],
                    'subTotal' => $content['line_subtotal'],
                    'subDiscount' => $content['discountpercentage'],
                    'product_id' => $content['product_id'],
                    'customID' => $content['customID']
                );
                $j++;
            }

            $temp = [];
            $subTotal = 0;

            foreach ($_type as $key => $value) {
                if (array_key_exists($value['block_id'], $temp)) {
                    $subTotal += $value['subTotal'];
                    $temp[$value['block_id']] = array(
                        'total' => $subTotal,
                        'discount' => $value['subDiscount'],
                        'block_id' => $value['block_id'],
                        'block_name' => $value['block_name']
                    );
                } else {
                    $subTotal = $value['subTotal'];
                    $temp[$value['block_id']] = array(
                        'total' => $subTotal,
                        'discount' => $value['subDiscount'],
                        'block_id' => $value['block_id'],
                        'block_name' => $value['block_name']
                    );
                }
            }

            foreach ($temp as $key => $value) {
                $discount_value = (float) $value['discount'];
                if ($discount_value > 1) {
                    if ($allow_main_product != 1) {
                        $value['total'] = $value['total'] - $price;
                    }
                    $discount = $discount_value;
                    $cart->add_fee('You choose ' . strtoupper($value['block_name']) . ' in your cart, ' . $discount . ' discount has been added.', -$discount);
                } else {
                    if ($allow_main_product == 1) {
                        $discount = $discount_value * $value['total'];
                    } else {
                        $product = wc_get_product($main_product_id);
                        // $price = !empty($product->get_sale_price()) ? (float) $product->get_sale_price() : (float) $product->get_regular_price();
                        $price = (float) $product->get_price();
                        $discount = ($block_type == "accessories") ? $discount_value * $value['total'] : $discount_value * ($value['total'] - $price);
                    }
                    if ($discount_value != 0) {
                        $cart->add_fee('You choose ' . strtoupper($value['block_name']) . ' in your cart, ' . ($discount_value * 100) . '% discount has been added.', - $discount);
                    }
                }
            }
        }
    }

    function bfs_woocommerce_ajax_add_to_cart_redirect()
    {
        if (isset($_POST['addCartProduct'])) {
            global $product;
            // $carts_item = WC()->cart->get_cart();
            // $prodArray = array();
            // $blockArray = array();
            // foreach ($carts_item as $item) {
            //     array_push($prodArray, $item['product_id']);
            //     if(!empty($item['block_id'])) {
            //         array_push($blockArray, $item['block_id']);
            //     }
            // }

            $main_product_id = isset($_POST['main_product_id']) ? $_POST['main_product_id'] : ''; // string
            $quantity = isset($_POST['main_qauntity']) ? (int) $_POST['main_qauntity'] : 1; // string
            $product_id = isset($_POST['add_product_id']) ? $_POST['add_product_id'] : ''; // string(csv)
            $product_qty = isset($_POST['add_product_qty']) ? $_POST['add_product_qty'] : ''; // string(csv)
            $block_type = isset($_POST['block_type']) ? $_POST['block_type'] : ''; // string
            $block_id = isset($_POST['block_id']) ? $_POST['block_id'] : ''; // string
            $discountpercentage_acc = isset($_POST['discountpercentage_acc']) ? $_POST['discountpercentage_acc'] : ''; // string
            $block_name = isset($_POST['block_name']) ? $_POST['block_name'] : ''; // string
            $allow_main_product = isset($_POST['allow_main_product']) ? (int) $_POST['allow_main_product'] : 0; // int
            $custom_id = time();

            //$discountpercentage = (10 / 100);
            // isset($_POST['discountpercentage_acc']) ? (substr($_POST['discountpercentage_acc'], -1) == "%") ? ((int) $_POST['discountpercentage_acc'] / 100) : (int) $_POST['discountpercentage_acc'] : (10 / 100)
            if($discountpercentage_acc) {
                $discountpercentage = (substr($discountpercentage_acc, -1) == "%") ? ((int) $discountpercentage_acc / 100) : (int) $discountpercentage_acc;
            }

            // Add main product to cart
            $custom_data = array(
                'block_type' => $block_type,
                'block_name'=> $block_name,
                'block_id' => $block_id,
                'isMainProduct' => 1,
                'customID' => $custom_id,
                'discountpercentage' => $discountpercentage
            );
            WC()->cart->add_to_cart($main_product_id, $quantity, 0, array(), $custom_data);

            $product_idArray = explode(",", $product_id);
            $product_qtyArray = explode(",", $product_qty);
            WC()->session->set('discountpercentage', $discountpercentage);
            WC()->session->set('allow_main_product', $allow_main_product);
            WC()->session->set('main_product_id', $main_product_id);
            WC()->session->set('block_type', $block_type);

            // Add bundle product to cart
            for ($i = 0; $i < count($product_idArray); $i++) {
                $prodid = $product_idArray[$i];
                $prodqty = $product_qtyArray[$i];
                //if (in_array($prodid, $prodArray) and in_array($block_id, $blockArray)) {
                $custom_data = array(
                    'block_type' => $block_type,
                    'block_name'=> $block_name,
                    'block_id' => $block_id,
                    'mainProduct_customeId' => $custom_id,
                    'isMainProduct' => 0,
                    'discountpercentage' => $discountpercentage
                );
                WC()->cart->add_to_cart($prodid, $prodqty, 0, array(), $custom_data);
            }
            // $carts_item = WC()->cart->get_cart();
            $link = get_permalink(wc_get_page_id('cart'));
            wp_redirect($link);
            exit();
        }

        if (isset($_POST['addCartAccessoriesProduct'])) {
            global $product;
            // $carts_item = WC()->cart->get_cart();
            // $prodArray = array();
            // $blockArray = array();
            // foreach ($carts_item as $item) {
            //     array_push($prodArray, $item['product_id']);
            //     if(!empty($item['block_id'])) {
            //         array_push($blockArray, $item['block_id']);
            //     }
            // }

            $main_product_id = isset($_POST['main_product_id_acc']) ? $_POST['main_product_id_acc'] : ''; // string
            $quantity = isset($_POST['main_qauntity_acc']) ? (int) $_POST['main_qauntity_acc'] : 1; // string
            $product_id = isset($_POST['add_product_id_acc']) ? $_POST['add_product_id_acc'] : ''; // string(csv)
            $product_qty = isset($_POST['add_product_qty_acc']) ? $_POST['add_product_qty_acc'] : ''; // string(csv)
            $block_type = isset($_POST['block_type']) ? $_POST['block_type'] : ''; // string
            $block_id = isset($_POST['block_id']) ? $_POST['block_id'] : ''; // string
            $discountpercentage_acc = isset($_POST['discountpercentage_acc']) ? $_POST['discountpercentage_acc'] : ''; // string
            $block_name = isset($_POST['block_name']) ? $_POST['block_name'] : ''; // string
            $allow_main_product = isset($_POST['allow_main_product']) ? (int) $_POST['allow_main_product'] : 0; // int

            //$discountpercentage = (10 / 100);
            // isset($_POST['discountpercentage_acc']) ? (substr($_POST['discountpercentage_acc'], -1) == "%") ? ((int) $_POST['discountpercentage_acc'] / 100) : (int) $_POST['discountpercentage_acc'] : (10 / 100)
            if($discountpercentage_acc) {
                $discountpercentage = (substr($discountpercentage_acc, -1) == "%") ? ((int) $discountpercentage_acc / 100) : (int) $discountpercentage_acc;
            }

            // Add main product to cart
            $custom_data = array(
                'block_type' => $block_type,
                'block_name'=> $block_name,
                'block_id' => $block_id,
                'isMainProduct' => 1,
                'customID' => $custom_id,
                'discountpercentage' => $discountpercentage
            );
            WC()->cart->add_to_cart($main_product_id, $quantity, 0, array(), $custom_data);


            // if (in_array($main_product_id, $prodArray) and in_array($block_id, $blockArray)) {
            // $custom_data = array(
            //     'block_type' => $block_type,
            //     'block_name'=> $block_name,
            //     'isMainProduct' => 1,
            //     'block_id' => $block_id,
            //     'discountpercentage' => $discountpercentage
            // );

            // Do not add this. Because, product is already added to the cart, now only need the accesories to the cart.
            // WC()->cart->add_to_cart($main_product_id, $quantity, 0, array(), $custom_data);

            $product_idArray = explode(",", $product_id);
            $product_qtyArray = explode(",", $product_qty);
            WC()->session->set('discountpercentage', $discountpercentage);
            WC()->session->set('allow_main_product', $allow_main_product);
            WC()->session->set('main_product_id', $main_product_id);
            WC()->session->set('block_type', $block_type);

            // Add accesorries to cart
            for ($i = 0; $i < count($product_idArray); $i++) {
                $prodid = $product_idArray[$i];
                $prodqty = $product_qtyArray[$i];
                //if (in_array($prodid, $prodArray) and in_array($block_id, $blockArray)) {
                $custom_data = array(
                    'block_type' => $block_type,
                    'isMainProduct' => 0,
                    'block_name'=> $block_name,
                    'block_id' => $block_id,
                    'discountpercentage' => $discountpercentage
                );
                WC()->cart->add_to_cart($prodid, $prodqty, 0, array(), $custom_data);

            }
            $link = get_permalink(wc_get_page_id('cart'));
            wp_redirect($link);
            exit();
        }
    }

    function bfs_woocommerce_ajax_add_to_cart()
    {
        $product_id = apply_filters('woocommerce_add_to_cart_product_id', absint($_POST['product_id']));
        $quantity = apply_filters('woocommerce_add_to_cart_qty', absint($_POST['product_qty']));
        $passed_validation = apply_filters('woocommerce_add_to_cart_validation', true, $product_id, $quantity);
        $product_status = get_post_status($product_id);

        echo $product_id . "------" . $quantity;
        if ($passed_validation && WC()->cart->add_to_cart($product_id, $quantity) && 'publish' === $product_status) {

            echo "1-->";
            do_action('woocommerce_ajax_added_to_cart', $product_id);

            if ('yes' === get_option('woocommerce_cart_redirect_after_add')) {
                echo "2-->";
                wc_add_to_cart_message(array($product_id => $quantity), true);
            }

            WC_AJAX:: get_refreshed_fragments();
        } else {
            echo "3-->";
            $data = array(
                'error' => true,
                'product_url' => apply_filters('woocommerce_cart_redirect_after_error', get_permalink($product_id), $product_id));

            echo wp_send_json($data);
        }

        wp_die();
    }


    function bfs_calculate_section()
    {
        global $product;
        $children = $product->get_children();

        $bundlePrice = 0;
        foreach ($children as $childproduct) {
            $product = get_product($childproduct);
            $bundlePrice += $product->get_regular_price();
        }
        $discount = count($children) >= 5 ? (10000 + $bundlePrice) * 0.1 : 0;
        $save_amount = ((10000 + $bundlePrice) - $discount);

        $table = "<table>
                <tr>
                <th>Main Product</th>
                <th>Accessories</th>
                <th>Bundle Discount</th>
                <th>Total Amount</th>
                <th>Save Amount</th>
                </tr>
                <tr>
                <td><input id='main_product' name='main_product' type='text' value='10000'></td>
                <td><input id='accessories' name='accessories' type='text' value='$bundlePrice'></td>
                <td><input id='discount' name='discount' type='text' value='10'></td>
                <td><input id='total_amount' name='total_amount' value='$save_amount' type='text'></td>
                <td><input id='save_amount' name='save_amount' value='$discount' type='text'></td>
                </tr>
                </table>";

        echo $table;
    }

    function woo_product_weight_column($columns)
    {
        $columns['total_weight'] = esc_html__('Weight', 'woocommerce');
        return $columns;
    }

// Populate weight column
    function woo_product_weight_column_data($column)
    {
        global $post;
        if ($column == 'total_weight') {
            $product = wc_get_product($post->ID);
            $weight = $product->get_weight();
            if ($weight > 0)
                print $weight . ' ' . esc_attr(get_option('woocommerce_weight_unit'));
            else print 'N/A';
        }
    }

    function bfs_woocommerce_grouped_add_to_cart()
    {
        global $product;
        $products = array_filter(array_map('wc_get_product', $product->get_children()), 'wc_products_array_filter_visible_grouped');
        if ($products) {
            DynamicBlock_Main::dynamicblock_get_template('public/grouped.php', array(
                'grouped_product' => $product,
                'grouped_products' => $products,
                'quantites_required' => false,
            ));
        }
    }

    public function dynamicblock_scripts()
    {
        wp_enqueue_script('jquery');
        wp_enqueue_script('wp-tinymce');
        wp_enqueue_script('dynamicblock-carousel-script', DYNAMICBLOCK_PLUGIN_URL . 'assets/js/owl.carousel.min.js');
        wp_enqueue_script('dynamicblock-public-script', DYNAMICBLOCK_PLUGIN_URL . 'assets/js/dynamicblock-public-script.js', IS_LOCALHOST ? array() : null, IS_LOCALHOST ? time() : '');
        wp_enqueue_style('dynamicblock-carousel-style', DYNAMICBLOCK_PLUGIN_URL . 'assets/css/owl.carousel.min.css');
        wp_enqueue_style('dynamicblock-carousel-theme', DYNAMICBLOCK_PLUGIN_URL . 'assets/css/owl.theme.default.min.css');
        wp_enqueue_style('dynamicblock-public-style', DYNAMICBLOCK_PLUGIN_URL . 'assets/css/dynamicblock-style-frontend.css', IS_LOCALHOST ? array() : null, IS_LOCALHOST ? time() : '');
        // echo get_site_url();
    }

// include custom jQuery
    /*public function shapeSpace_include_custom_jquery() {

        wp_deregister_script('jquery');
        wp_enqueue_script('jquery', 'http://gotourws.ddns.net/dev/2022/wuuplug/wp-includes/js/jquery/jquery.min.js?ver=3.6.0', array(), null, true);

    }
    add_action('wp_enqueue_scripts', 'shapeSpace_include_custom_jquery');*/



    public function includes()
    {
        include_once DYNAMICBLOCK_ABSPATH . 'public/class/class-dynamicblock-conditional-content-display.php';
    }

    /**
     * Hooked into wc_conditional_content_get_rule_types to get the default list of rule types.
     * @param array $types Current list, if any, of rule types.
     * @return array the list of rule types.
     */
    public function default_rule_types($types)
    {
        $types = array(
            __('Global', DYNAMICBLOCK_TEXT)     => array(
                'show_always' => __('Show Always', DYNAMICBLOCK_TEXT),
            ),
            __("Product", DYNAMICBLOCK_TEXT)    => apply_filters('dynamicblock_when_to_display_rule_type_product',
                array(
                    'products'          => __('Products', DYNAMICBLOCK_TEXT),
                    'product_type'      => __("Product Type", DYNAMICBLOCK_TEXT),
                    'product_category'  => __("Product Category", DYNAMICBLOCK_TEXT),
                    'product_attribute' => __("Product Attributes", DYNAMICBLOCK_TEXT),
                    'product_tag'       => __("Product Tag", DYNAMICBLOCK_TEXT),
                    'product_price'     => __("Product Price", DYNAMICBLOCK_TEXT),
                )),
            __('Stock', DYNAMICBLOCK_TEXT)      => apply_filters('dynamicblock_when_to_display_rule_type_stock', array(
                'pro_version' => __('Update to PRO version', DYNAMICBLOCK_TEXT),
            )),
            __('Sales', DYNAMICBLOCK_TEXT)      => apply_filters('dynamicblock_when_to_display_rule_type_sales', array(
                'sale_date'   => __('Sale Date', DYNAMICBLOCK_TEXT),
                'sale_status' => __('Sale Status', DYNAMICBLOCK_TEXT),
            )),
            __("Cart", DYNAMICBLOCK_TEXT)       => apply_filters('dynamicblock_when_to_display_rule_type_cart', array(
                'pro_version' => __('Update to PRO version', DYNAMICBLOCK_TEXT),
            )),
            __("User Role", DYNAMICBLOCK_TEXT) => apply_filters('dynamicblock_when_to_display_rule_type_membership',
                array(
                    'pro_version' => __('Update to PRO version', DYNAMICBLOCK_TEXT),
                )),
            __("Checkout", DYNAMICBLOCK_TEXT)   => apply_filters('dynamicblock_when_to_display_rule_type_checkout',
                array(
                    'pro_version' => __('Update to PRO version', DYNAMICBLOCK_TEXT),
                )),
        );

        return $types;
    }

    /**
     * Hooked into wc_conditional_content_get_rule_operators.  Get's the default list of rule operators.
     * @param array $operators The current list, if any, of operators for rule types.
     * @return array
     */
    public function default_rule_operators($operators)
    {
        $operators = array(
            '==' => __("is equal to", DYNAMICBLOCK_TEXT),
            '!=' => __("is not equal to", DYNAMICBLOCK_TEXT),
        );
        return $operators;
    }

    public function bfs_wc_get_template_part($template, $slug, $name)
    {
        if ($slug == 'dynamicblock-content') {
            $template = DYNAMICBLOCK_TEMPLATE_PATH . "public/{$slug}-{$name}.php";
        }
        return $template;
    }


    public function dynamicblock_add_short_description($product)
    {
        $product_data = $product->get_data();
        $short_description = $product_data['short_description'];
        ?>
        <div class="dynamicblock-short-description">
            <p><?php echo esc_html($short_description); ?></p>
        </div>
        <?php
    }

    public static function dynamicblock_woocommerce_loop_product_link($link, $product)
    {
        global $current_block_rules;
        if (isset($current_block_rules) && $current_block_rules) {
            if ($current_block_rules['settings']['utm-source'] || $current_block_rules['settings']['utm-medium'] || $current_block_rules['settings']['utm-campaign']) {
                $link .= '?';
            }

            $link .= ($current_block_rules['settings']['utm-source']) ? 'utm_source=' . ($current_block_rules['settings']['utm-source']) : '';
            if ($current_block_rules['settings']['utm-source'] && ($current_block_rules['settings']['utm-medium'] || $current_block_rules['settings']['utm-campaign'])) {
                $link .= '&';
            }
            $link .= ($current_block_rules['settings']['utm-medium']) ? 'utm_medium=' . ($current_block_rules['settings']['utm-medium']) : '';
            if ($current_block_rules['settings']['utm-medium'] && $current_block_rules['settings']['utm-campaign']) {
                $link .= '&';
            }
            $link .= ($current_block_rules['settings']['utm-campaign']) ? 'utm_campaign=' . ($current_block_rules['settings']['utm-campaign']) : '';
        }
        return $link;
    }


    function bfs_update_cart_on_item_qty_change()
    {
        if (is_cart()) :
            ?>
            <script type="text/javascript">
                jQuery(function ($) {
                    $(document.body).on('change input', '.qty', function () {
                        $('button[name="update_cart"]').trigger('click');
                    });
                });
            </script>
        <?php
        endif;
    }

}

new DynamicBlock_Public;